#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
#  MakeInvoice.py
#  
#  Copyright 2017 krasytod <krasytod@gmail.com>
###
'''

'''
import types
import psycopg2
import psycopg2.extras
import datetime, time, calendar
from decimal import *
import configparser, argparse
import logging, os, sys, signal
from logging.handlers import TimedRotatingFileHandler
from logging.handlers import RotatingFileHandler
from lockfile.pidlockfile import PIDLockFile
from lockfile import AlreadyLocked
from manualinvoice import MakeInvoiceAuto,email_invoice,get_smtp_info,GetEmailTemplate,getCdrLink
# from  manualinvoice import send_mail as new_send_mail
from dateutil.relativedelta import relativedelta
# import multiprocessing
from logging.handlers import TimedRotatingFileHandler
from logging.handlers import RotatingFileHandler
from lockfile.pidlockfile import PIDLockFile
from lockfile import AlreadyLocked

import traceback
import smtplib, requests, multiprocessing, signal, base64
from email.mime.multipart import MIMEMultipart
from email.mime.base import MIMEBase
from email.mime.text import MIMEText
from email.utils import  formatdate
from email import encoders
from mail import send_email_log, send_mail

config = ''
# pidfile = ''

CDR_UI = "/homes/cdr_from_invoice/%s"
from version import VERSION

logger = logging.getLogger('Auto invoice')
FTP_HOST = ""
PIDF = 'autoinvoice.pid'


class GracefulKiller:
	kill_now = False

	def __init__(self):
		signal.signal(signal.SIGINT, self.exit_gracefully)
		signal.signal(signal.SIGTERM, self.exit_gracefully)

	def exit_gracefully(self, signum, frame):
		pidfile = PIDLockFile(PIDF, timeout=-1)
		try:
			pidfile.break_lock()
		except Exception as e:
			msg = "Exception when trying to break lock %s" % e
			print(msg)
			try:
				os.remove(PIDF)
				exit(1)
			except Exception as e:
				msg = "Exception when trying to break lock and delete pid file %s" % e
				print(msg)
				exit(2)
		exit(0)


def pidCheck():
	pidfile = PIDLockFile(PIDF, timeout=-1)
	try:
		pidfile.acquire()
		msg = "Lock acquered! Daemon setup script ID %s" % pidfile.read_pid()
		print(msg)
		logger.info(msg)
	# pidfile.break_lock()
	except AlreadyLocked:
		msg = "AlreadyLocked! Please kill the current running instance and try again"
		print(msg)
		logger.info(msg)
		exit(1)

	except Exception as e:  # No process with locked PID
		msg = "General Exception when trying to get lock %s" % e
		print(msg)
		logger.info(msg)
		exit(1)


''' 
On an hourly basis, run the following:
- select all client that meets the following:
1. Has more than 1 ingress resource - select count(*) from resource where client_id = xxx
2. Is active - select active from client where client_id = xxx
3 has invoice_flag turned on
4 has a invoice payment_term specified in the payment_term_field. payment term can be associated with payment_term table'''

billing_type = {1: "Invoice every XX days", 2: 'Inovice once a month on a specific day',
				3: 'Invoice once a week on a specific day of week',
				4: 'invoice multiple times a month on a specific day(s) of month'}
DAYS_OF_WEEK = {1: "Monday", 2: "Tuesday", 3: "Wednesday", 4: "Thursday", 5: "Friday", 6: "Saturday", 7: "Sunday"}

INVOICE_SQL = '''select client.invoice_zone,current_timestamp(0) as balance_time,c4_client_balance.balance as current_balance, client.name as client_name,  
case when 
client.daily_balance_send_time is null then time '00:00:00' else daily_balance_send_time end as send_time , 
client.daily_balance_send_time_zone as send_time_zone,
case when invoice_zone is null then null when invoice_zone ~ '^\\\\s*\$' 
then null else current_date <= current_timestamp(0) at time zone invoice_zone::interval end as is_zone_hour,current_date as current_date,
current_date + grace_days as invoice_due_date, payment_term.name as payment_name ,
*,
extract('epoch' from invoice_start_from) as invoice_start_from,extract('epoch' from current_date) as now_date,
EXTRACT(DOW FROM now()) as now_week,EXTRACT(DAY from now()) as now_day,
to_char(to_timestamp(to_char(now()+'1 month','YYYY-MM-01'),'YYYY-MM-DD')-interval '1 day','DD') as month_day,
is_send_as_link
from client left join payment_term on client.payment_term_id=payment_term.payment_term_id left 
join c4_client_balance on c4_client_balance.client_id=client.client_id::text where client.auto_invoicing = true 
and payment_term.payment_term_id is not null and client.status= true '''

INVOICE_COUNT_SQL = '''select resource.client_id,count(resource.ingress is true) from resource join 
client on resource.client_id=client.client_id group by resource.client_id having 
count(resource.ingress is true) > 0;
'''

SWITCH_INVOICE_SETTINGS = '''SELECT invoice_name, tpl_number, pdf_tpl, company_info,overlap_invoice_protection,send_cdr_fields, invoice_send_mode,company_info_location,invoice_decimal_digits
 FROM system_parameter'''


def connect_to_postgresql(host, port, database, user, password=None):
	try:
		conn = psycopg2.connect(host=host, port=port, database=database, user=user, password=password)
		conn.autocommit = True
		cursor = conn.cursor(cursor_factory=psycopg2.extras.DictCursor)
		print("connected to DB")
	except Exception as e:
		print(" Exception in DB connecting:  %s" % (str(e)))
		# raise 'I am unable to connect to the PostgreSQL. %s'% (str(e))
		raise
	return conn, cursor


def makeClientDict(result_count):
	client_dict = dict()
	for client in result_count:
		client_info = {'client_id': client[0], 'ingress_count': client[1]}
		client_dict[str(client[0])] = client_info
	return client_dict


def checkClientsTest(result, client_dict, total_invoices, pg_cur):
	invoices_needed = []
	for client in result:
		last_invoiced = client['last_invoiced']
		# print (invoice_history,last_invoiced)
		if last_invoiced == None:
			last_invoiced = client['create_time']
		invoice_start_from = int(last_invoiced.timestamp())
		send_epoch = invoice_start_from + 24 * 3600 * client['days']
		end_timestamp = time.strftime("%Y-%m-%d", time.gmtime(send_epoch))
		total_invoices = total_invoices + 1
		invoices_needed.append([client, "2016/10/12", "2016/10/13", end_timestamp])
	return (total_invoices, invoices_needed)


def setTimeZone(time_zone):
	sign = time_zone[0]
	hour = int(time_zone[1:3])
	mins = int(time_zone[4:])
	mins = (10 / 6) * mins
	if sign == "+":
		hour = hour
	else:
		hour = -1 * hour

	return hour


def checkClients(result, client_dict, total_invoices, pg_cur, today):
	invoices_needed = []
	for client in result:
		'''try:
			for (k,v) in client.items():
				print( k , v)
		except:
			print ("CANT print that!")'''
		try:
			time_zone = setTimeZone(client.get('invoice_zone', "+00:00"))
		except:
			print("Do not get time zone for client auto invoice. setting to GMT")
			logger.debug("Do not get time zone for client auto invoice. setting to GMT")
			time_zone = 0
		today = today + datetime.timedelta(hours=time_zone)
		client['now_date']=today.timestamp()
		client["name"] = client["client_name"]
		print("Client to be cheched ", client["name"], "client days", client["days"], "time_zone ", time_zone)
		logger.debug("Client to be cheched name %s ID %s and payment type: %s" % (
		client["name"], client['client_id'], client['type']))
		if client['client_id'] not in client_dict.keys():
			print("Client_id: %s has not enough big ingress count" % (client['client_id']))
			logger.debug("Client_id: %s has not enough big ingress count" % (client['client_id']))
			continue
		##debug only!!
		'''if client['client_id']  != '498':
			print ("Not this client id!", client['client_id'],type(client['client_id']) , client['client_id'] == 498  )
			continue
		else:
			print("This is the client!",client['client_id'],client.get('name',client.get('client_name',"N/A")  ) )
		'''
		sql_mod = None
		# print(client['last_invoiced'])
		client_invoice_history_sql = "select last_invoice_for,extract('epoch' from last_invoice_for) as last_invoice_for_epoch from invoice_history where client_id = %s order by last_invoice_for desc limit 1" % (
		client['client_id'])
		pg_cur.execute(client_invoice_history_sql)
		invoice_history = pg_cur.fetchone()
		last_invoiced = client['last_invoiced']
		# print (invoice_history,last_invoiced)
		if last_invoiced == None:
			print("Never Invoiced")
			logger.debug("Client %s with Id:  %s is type %s is never invoiced before " % (
			client['name'], client['client_id'], billing_type[client['type']]))
			#last_invoiced = client['create_time'] - datetime.timedelta(weeks=100)
			last_invoiced = client['create_time']

		invoice_start_from = int(last_invoiced.timestamp())
		last_invoice_day = datetime.datetime(last_invoiced.year,last_invoiced.month,last_invoiced.day)
		if client['type'] == 1:  # Every Nth day
			print("Every Nth day")
			logger.debug("Every Nth day")
			send_epoch = invoice_start_from + 24 * 3600 * client['days']
			# today =   datetime.datetime.now() + datetime.timedelta(hours=time_zone)
			print(last_invoiced.date() == datetime.datetime.today().date(), last_invoiced.date(),
				  datetime.datetime.today().date())
			end_epoch = invoice_start_from + 24 * 3600 * (
						client['days'] - 1)  # end of billing period invoice is one day before send date
			print("Last invoice: %s Send: %s  Now: %s  End of billing period: %s" % (
			last_invoiced, str(send_epoch), str(client['now_date']), end_epoch))
			logger.debug("Last invoice: %s Send: %s  Now: %s  End of billing period: %s" % (
			last_invoiced, str(send_epoch), str(client['now_date']), end_epoch))
			if datetime.datetime.utcfromtimestamp(end_epoch).date() < datetime.datetime.utcnow().date():  # and  int(time.mktime(today.timetuple()) - invoice_start_from) > 24 * 3601 :  #   # send_epoch <= client['now_date']  and
				##today = datetime.datetime.now() + datetime.timedelta(hours=time_zone)
				today_day = today.date()
				send_datetime = today_day  # . replace(day=client_days) #same year, month  but the day is changed
				send_epoch = calendar.timegm(send_datetime.timetuple())
				send_epoch_str = datetime.datetime.utcfromtimestamp(send_epoch)
				end_timestamp = time.strftime("%Y-%m-%d", time.gmtime(send_epoch))
				invoice_end_time = send_datetime + datetime.timedelta(days=-1)
				invoice_start_time = send_datetime + datetime.timedelta(
					days=-client['days'])  # datetime.timedelta(months=-1)  # send_datetime.replace(day=1)
				invoice_start_time = str(last_invoice_day)
				# update invoice log
				end_timestamp = time.strftime("%Y-%m-%d", time.gmtime(send_epoch))
				invoice_start_time = str(invoice_start_time)[0:10] + " 00:00:00"
				invoice_end_time = str(invoice_end_time)[0:10] + " 23:59:59"
				sql_mod = "INSERT INTO invoice_history(last_invoice_for,client_id) values(%s,%s)" % (
				send_epoch, client['client_id'])
				# pg_cur.execute(sql_mod)
				# report = pg_cur.fetchall()
				# print (report)
				total_invoices = total_invoices + 1
				print(""" New Invoice for client %s , from: %s, To: %s """ % (
				client, invoice_start_time, invoice_end_time))
				logger.debug(""" New Invoice for client %s , from: %s, To: %s """ % (
				client, invoice_start_time, invoice_end_time))
				invoices_needed.append([client, invoice_start_time, invoice_end_time, end_timestamp])
			else:  # no invoice
				print(
					"No Invoice For client %s with Id %s with billing every %s-th day, Last invoice: %s   Now: %s  End of billing period: %s" % (
					client['name'], client['client_id'], client['days'], last_invoiced,
					time.strftime('%Y-%m-%d', time.localtime(client['now_date'])),
					time.strftime('%Y-%m-%d', time.localtime(end_epoch))))
				logger.debug(
					"No Invoice For client %s with Id %s with billing every %s-th day, Last invoice: %s   Now: %s  End of billing period: %s" % (
					client['name'], client['client_id'], client['days'], last_invoiced,
					time.strftime('%Y-%m-%d', time.localtime(client['now_date'])),
					time.strftime('%Y-%m-%d', time.localtime(end_epoch))))
		# - if it is weekly, then we check the day of week , the invoice should be generated from the last week’s till end of yesterday
		if client['type'] == 2:  # # Day of month
			# print ("type2: ",client['name'])
			#today = datetime.datetime.now() + datetime.timedelta(hours=time_zone)
			client_days = int(client['days'])  # which day
			send_datetime = today.replace(day=client_days)  # same year, month  but the day is changed
			send_epoch = calendar.timegm(send_datetime.timetuple())
			send_epoch_str = datetime.datetime.utcfromtimestamp(send_epoch)
			end_timestamp = time.strftime("%Y-%m-%d", time.gmtime(send_epoch))
			invoice_end_time = (send_datetime + datetime.timedelta(days=-1)).date()
			invoice_start_time = invoice_end_time - relativedelta(months=1)#today.replace(day=1)  # datetime.timedelta(months=-1)  # send_datetime.replace(day=1)
			invoice_start_time = str(last_invoice_day)
			print("invoice_start_time", invoice_start_time, invoice_end_time, relativedelta(months=-1))
			# strftime("%Y-%m-%d 23:59:59");
			invoice_start_time = str(invoice_start_time)[0:10] + " 00:00:00"
			invoice_end_time = str(invoice_end_time)[0:10] + " 23:59:59"
			if (today.day == int(client['days']) and int(
					client['now_date'] - invoice_start_from) > 24 * 3601):  # or client['days'] == 28:
				end_timestamp = str(send_datetime)
				sql_mod = "INSERT INTO invoice_history(last_invoice_for,client_id) values(%s,%s)" % (
				send_epoch, client['client_id'])
				# pg_cur.execute(sql_mod)

				total_invoices = total_invoices + 1
				# if client ['invoice_start_from'] != None:
				#	invoice_start_time = time.strftime('%Y-%m-%d %H:%M:%S', time.localtime(client ['invoice_start_from']) ) 
				invoices_needed.append([client, invoice_start_time, invoice_end_time, end_timestamp])
				logger.debug(
					"New once in month invoice For client %s with Id %s with billing at %s-th day, Last invoice: %s   Now: %s , Invoice start date: %s, Invoice end date %s" % (
					(client['name'], client['client_id'], client['days'], last_invoiced,
					 time.strftime('%Y-%m-%d ', time.localtime(client['now_date'])), invoice_start_time,
					 invoice_end_time)))
				print(
					"New once in month invoice For client %s with Id %s with billing at %s-th day, Last invoice: %s   Now: %s , Invoice start date: %s, Invoice end date %s" % (
					(client['name'], client['client_id'], client['days'], last_invoiced,
					 time.strftime('%Y-%m-%d ', time.localtime(client['now_date'])), invoice_start_time,
					 invoice_end_time)))
			else:
				print(
					"No new once in month invoice For client %s with Id %s with billing at %s-th day, Last invoice: %s   Now: %s " % (
					client['name'], client['client_id'], client['days'], last_invoiced,
					time.strftime('%Y-%m-%d', time.localtime(client['now_date']))))

		elif client['type'] == 3:  # Day of Week
			print("Day of Week")
			logger.debug("Day of Week")
			##today = datetime.datetime.now() + datetime.timedelta(hours=time_zone)
			day_of_week = today.weekday() + 1  # monday is zero in Python!!!
			diff = client['days'] - day_of_week
			print(client['days'], day_of_week)
			logger.debug("invoice_start_from %s" % invoice_start_from)
			print("invoice_start_from %s" % invoice_start_from)
			print("diff", diff, time.mktime(today.timetuple()), invoice_start_from,
				  client['now_date'] - invoice_start_from)
			print(int(time.mktime(today.timetuple()) - invoice_start_from), "24 * 3601", 24 * 3601)
			if diff == 0 and int(time.mktime(
					today.timetuple()) - invoice_start_from) > 24 * 3601:  # if not same day of week, Skip!! Also the diff in seconds between now and last invoice must be bigger from 24 * 3601(a day)
				client_days = client['days']  # which day of week
				send_epoch = calendar.timegm(today.timetuple())
				send_epoch_str = datetime.datetime.utcfromtimestamp(send_epoch)
				end_timestamp = time.strftime("%Y-%m-%d", time.gmtime(send_epoch))
				day = today.date()  # + datetime.timedelta(days=-2)
				invoice_start_time = str(day + datetime.timedelta(days=-7)) + " 00:00:00"  # str(last_invoiced)
				invoice_start_time = str(last_invoice_day)[0:10] + " 00:00:00"
				invoice_end_time = str(day + datetime.timedelta(days=-1))[0:10] + " 23:59:59"
				sql_mod = "INSERT INTO invoice_history(last_invoice_for,client_id) values(%s,%s)" % (
				send_epoch, client['client_id'])
				#pg_cur.execute(sql_mod)
				total_invoices = total_invoices + 1
				invoices_needed.append([client, invoice_start_time, invoice_end_time, end_timestamp])
				print(
					"For client with id %s ,with billing: Day of week: %s. New Invoiced is made.Last invoice was at: %s .Today  is %s,  %s" % (
					client['client_id'], DAYS_OF_WEEK[client['days']], invoice_start_time, DAYS_OF_WEEK[day_of_week],
					time.strftime("%Y-%m-%d", time.gmtime(client['now_date']))))
				logger.debug(
					"For client with id %s ,with billing: Day of week: %s. New Invoiced is made.Last invoice was at: %s .Today  is %s,  %s" % (
					client['client_id'], DAYS_OF_WEEK[client['days']], invoice_start_time, DAYS_OF_WEEK[day_of_week],
					time.strftime("%Y-%m-%d", time.gmtime(client['now_date']))))
			# Make Invoice!
			else:  # NO invoice!
				print(
					"For client with id %s ,with billing: Day of week: %s. New Invoiced is not neeeded now.Last invoice was at: %s .Today is %s, %s" % (
					client['client_id'], DAYS_OF_WEEK[client['days']], client['last_invoiced'],
					DAYS_OF_WEEK[day_of_week], time.strftime("%Y-%m-%d", time.gmtime(client['now_date']))))
				logger.debug(
					"For client with id %s ,with billing: Day of week: %s. New Invoiced is not neeeded now.Last invoice was at: %s .Today is %s, %s" % (
					client['client_id'], DAYS_OF_WEEK[client['days']], client['last_invoiced'],
					DAYS_OF_WEEK[day_of_week], time.strftime("%Y-%m-%d", time.gmtime(client['now_date']))))

		elif client['type'] == 4:  # Some Day of month
			payment_days = client['more_days'].split(",")
			today_day = today.date()
			invoice_end_time = today_day + datetime.timedelta(days=-1)
			invoice_start_time = str(
				today_day + relativedelta(months=-1)) + " 00:00:00"  # str(last_invoiced) datetime.timedelta(months=-1)
			invoice_start_time = str(last_invoice_day)[0:10]+ " 00:00:00"
			invoice_end_time = str(invoice_end_time)[0:10] + " 23:59:59"
			Make_invoice = False
			for day in payment_days:
				print(today.day, type(today_day.day), day, type(day))
				logger.debug("Today: %s, Payment Day %s" % (str(today.day), str(day)))
				if int(today_day.day) == int(day):
					if int(time.mktime(
							today.timetuple()) - invoice_start_from) > (24 * 3600)+1:  # if not same day of week, Skip!! Also the diff in seconds between now and last invoice must be bigger from 24 * 3601(a day)
						Make_invoice = True
						break
			if Make_invoice == True:
				print(
					"For client_id %s billing type : Days of month: %s . New Invoice is created. From: %s , To: %s .Last invoice was at: %s .Today is %s" % (
					client['client_id'], payment_days, invoice_start_time, invoice_end_time, last_invoiced,
					time.strftime('%Y-%m-%d %H:%M:%S', time.localtime(client['now_date']))))
				logger.debug(
					"For client_id %s billing type : Days of month: %s . New Invoice is created. From: %s , To: %s .Last invoice was at: %s .Today is %s" % (
					client['client_id'], payment_days, invoice_start_time, invoice_end_time, last_invoiced,
					time.strftime('%Y-%m-%d %H:%M:%S', time.localtime(client['now_date']))))
				send_epoch = calendar.timegm(today.timetuple())
				send_epoch_str = datetime.datetime.utcfromtimestamp(send_epoch)
				end_timestamp = time.strftime("%Y-%m-%d", time.gmtime(send_epoch))
				total_invoices = total_invoices + 1
				invoices_needed.append([client, invoice_start_time, invoice_end_time, end_timestamp])
				sql_mod = "INSERT INTO invoice_history(last_invoice_for,client_id) values(%s,%s)" % (
				send_epoch, client['client_id'])
			# pg_cur.execute(sql_mod)
			else:
				logger.debug(
					"For client_id %s billing type : Days of month: %s . Invoice is not created. Today is %s" % (
					client['client_id'], payment_days, time.strftime('%Y-%m-%d', time.localtime(client['now_date']))))
				print("For client_id %s billing type : Days of month: %s . Invoice is not created. Today is %s" % (
				client['client_id'], payment_days, time.strftime('%Y-%m-%d', time.localtime(client['now_date']))))
		elif client['type'] == 5:  # Twice of month
			payment_days = [1,16]
			today_day = today.date()
			invoice_end_time = today_day + datetime.timedelta(days=-1)
			if today_day.day<15:
				d=today_day-relativedelta(months=1)
				invoice_start_time = str(datetime.date(d.year,d.month,16)) + " 00:00:00"  # str(last_invoiced) datetime.timedelta(months=-1)
			else:
				d = today_day
				invoice_start_time = str(datetime.date(d.year, d.month,
				                                       1)) + " 00:00:00"  # str(last_invoiced) datetime.timedelta(months=-1)
			invoice_start_time = str(last_invoice_day)[0:10]+ " 00:00:00"
			invoice_end_time = str(invoice_end_time)[0:10] + " 23:59:59"
			Make_invoice = False
			for day in payment_days:
				print(today_day.day, type(today_day.day), day, type(day))
				logger.debug("Today: %s, Payment Day %s" % (str(today_day.day), str(day)))
				if int(today_day.day) == int(day):
					if int(time.mktime(
							today.timetuple()) - invoice_start_from) > (24 * 3600)+1:  # if not same day of week, Skip!! Also the diff in seconds between now and last invoice must be bigger from 24 * 3601(a day)
						Make_invoice = True
						break
			if Make_invoice == True:
				print(
					"For client_id %s billing type : Twice of month: %s . New Invoice is created. From: %s , To: %s .Last invoice was at: %s .Today is %s" % (
					client['client_id'], payment_days, invoice_start_time, invoice_end_time, last_invoiced,
					time.strftime('%Y-%m-%d %H:%M:%S', time.localtime(client['now_date']))))
				logger.debug(
					"For client_id %s billing type : Twice of month: %s . New Invoice is created. From: %s , To: %s .Last invoice was at: %s .Today is %s" % (
					client['client_id'], payment_days, invoice_start_time, invoice_end_time, last_invoiced,
					time.strftime('%Y-%m-%d %H:%M:%S', time.localtime(client['now_date']))))
				send_epoch = calendar.timegm(today.timetuple())
				send_epoch_str = datetime.datetime.utcfromtimestamp(send_epoch)
				end_timestamp = time.strftime("%Y-%m-%d", time.gmtime(send_epoch))
				total_invoices = total_invoices + 1
				invoices_needed.append([client, invoice_start_time, invoice_end_time, end_timestamp])
				sql_mod = "INSERT INTO invoice_history(last_invoice_for,client_id) values(%s,%s)" % (
				send_epoch, client['client_id'])
			# pg_cur.execute(sql_mod)
			else:
				logger.debug(
					"For client_id %s billing type : Days of month: %s . Invoice is not created. Today is %s" % (
					client['client_id'], payment_days, str(today)))
				print("For client_id %s billing type : Days of month: %s . Invoice is not created. Today is %s" % (
				client['client_id'], payment_days, time.strftime('%Y-%m-%d', time.localtime(client['now_date']))))

		if sql_mod != None:
			print("Updating DB invoice history with SQL: %s" % (sql_mod))

	return (total_invoices, invoices_needed)


def get_invoice_no(pg_cur):
	''' Gets new No for invoice'''
	get_number_sql = "SELECT nextval('class4_seq_invoice_no'::regclass) as no;"
	# изключено заради теста , нямам достъп до базата данни !
	try:
		pg_cur.execute(get_number_sql)
		result_id = pg_cur.fetchone()
		new_id = result_id['no']
	except Exception as e:
		print(" Exception in Getting invoice num:  %s" % (str(e)))
		logger.error(" Exception in Getting invoice num:  %s" % (str(e)))
	###exist_no_sql = "select invoice_number as no from invoice where invoice_number='$get_number' limit 1 " %s
	###v stariq code ima proverka dali ne se izpolsva ?!??! "select invoice_number as no from invoice where invoice_number='$get_number' limit 1 " Warning 01
	# new_id = 13
	return new_id;


def invoice_history(filename, client, pg_cur, start_time, end_time, email=False):
	if email == True:
		sql = ' select  notify_days as nd from payment_term where payment_term_id = %s' % client['payment_term_id']
		print("sql for getting notify_days: " + sql)
		pg_cur.execute(sql)
		result = pg_cur.fetchone()
		print("Result of notify_days  in invoice_history", result)
		notify_days = result['nd']
		if notify_days == None:
			notify_days = 0
		now = datetime.datetime.now()
		notify_time = now + datetime.timedelta(days=+notify_days)  # days
		epoch_secs_unblock = int(notify_time.timestamp())
		try:
			sql = "INSERT INTO invoice_history(client_id,reminders_count,next_reminder_timestamp,invoice_filename,start_time,end_time) values(%s,%s,%s,'%s','%s','%s')" % (
			client['client_id'], notify_days, epoch_secs_unblock, filename, start_time, end_time)
			logger.debug("Sql of updating client invoice history: %s" % sql)
			print("Sql of updating client invoice history: %s" % sql)
			pg_cur.execute(sql)
			sql = "UPDATE client SET last_invoiced = '%s'  WHERE client_id = %s" % (str(now), client['client_id'])
			logger.debug("Sql of updating client invoice history: %s" % sql)
			print("Sql of updating client invoice history: %s" % sql)
			pg_cur.execute(sql)
		except Exception as e:
			print(" Exception in history invoice:  %s" % (str(e)))
			logger.error("Exception in history invoice: :  %s" % (str(e)))

	else:
		now = datetime.datetime.now()
		try:
			sql = "UPDATE client SET last_invoiced = '%s'  WHERE client_id = %s" % (str(now), client['client_id'])
			logger.debug("Sql of updating client invoice history: %s" % sql)
			print("Sql of updating client invoice history: %s" % sql)
			pg_cur.execute(sql)
		except Exception as e:
			print(" Exception in history invoice:  %s" % (str(e)))
			logger.error("Exception in history invoice: :  %s" % (str(e)))


def notify_carrier(pg_cur):
	now = datetime.datetime.now()
	epoch_secs_unblock = int(now.timestamp())
	sql = '''SELECT * FROM  invoice_history WHERE reminders_count > 0 AND next_reminder_timestamp < %s ''' % epoch_secs_unblock
	print("query for getting reminders " + sql)
	pg_cur.execute(sql)
	result = pg_cur.fetchall()
	print("result Notify: ", result, len(result))
	jobs = []
	#fields = GetCdrFields(pg_cur)
	smtp_info = get_smtp_info(pg_cur)
	EmailTemplate = GetEmailTemplate(pg_cur)

	for reminder in result:
		# debug only
		if True:
			continue
		subject = '''Hello %s. This is reminder for your invoice'''
		sql = '''select * from client where client_id = %s ''' % reminder['client_id']
		print("Query for getting Client details for notifing", sql)
		try:
			pg_cur.execute(sql)
			client = pg_cur.fetchone()
			sql = '''update  invoice_history set  reminders_count=reminders_count -1  WHERE  invoice_filename= '%s' ''' % \
				  reminder['invoice_filename']
			pg_cur.execute(sql)
		except Exception as e:
			print(" Exception in getting client for reminder:  %s" % (str(e)))
			logger.error("Exception in getting client for reminder :  %s" % (str(e)))
			continue
		data = {'start_date': reminder['start_time'], 'end_date': reminder['end_time']}
		invoice_number = reminder['invoice_filename'][:-16]
		# email_invoice (filename, client[0],pg_cur,data,fields )
		p = multiprocessing.Process(target=email_invoice, args=(
		reminder['invoice_filename'], client, data, smtp_info, "",False, invoice_number, pg_cur))
		# p.daemon = True
		p.start()
		jobs.append(p)
		print(jobs)


def main(args):
	global config  # pidfile
	config = configparser.ConfigParser()
	config.read('config.ini')

	global DEBUG_LEVEL, cdr_table, FTP_HOST
	DEBUG_LEVEL = int(config["log"]["debug_level"])
	# cdr_table = ""  #config["other"]["cdr_table"]
	# logger

	FTP_HOST = config["other"].get('ftp', "").replace("http://", "")
	if int(config["log"].get('by_time', 0)) == 1:
		handler = TimedRotatingFileHandler(config["log"].get("invoice_auto_path", "invoice_auto.log"),
										   when=config["log"].get("invoice_auto_when", 'm'),
										   interval=int(config["log"].get("invoice_auto_interval", 0)),
										   backupCount=int(config["log"].get("invoice_auto_backupCount", 5)))
	else:  # by size
		handler = RotatingFileHandler(config["log"].get("invoice_auto_path", "invoice_auto.log"),
									  maxBytes=int(config["log"].get("invoice_auto_maxBytes", 1024 * 1024 * 128)),
									  backupCount=int(config["log"].get("invoice_auto_backupCount", 5)))

	if int(config["log"].get("debug_level", 0)) == 1:
		logger.setLevel(logging.DEBUG)
	else:
		logger.setLevel(logging.INFO)
	formatstr = '%(asctime)s - %(name)s - %(levelname)s - %(message)s'
	formatstr = '%(asctime)s: %(levelname)s:  %(message)s'
	formatter = logging.Formatter(formatstr)
	handler.setFormatter(formatter)
	logger.addHandler(handler)

	try:
		pg_conn, pg_cur = connect_to_postgresql(config["POSTGRES"]["hostaddr"], int(config["POSTGRES"]["port"]),
												config["POSTGRES"]["dbname"], config["POSTGRES"]["user"],
												config["POSTGRES"]["password"])  # config["POSTGRES"]["password"]
		return pg_conn, pg_cur
	except Exception as e:
		print(" Exception in connect_to_postgresql:  %s" % (str(e)))
		logger.error(" Exception in connect_to_postgresql:  %s" % (str(e)))
		return -1, -1


def main_start(pg_conn, pg_cur):
	# add to log table

	sql = "INSERT INTO scheduler_log (script_name, start_time) VALUES ('Invoice', current_timestamp(0))";
	pg_cur.execute(sql)
	logger.info(sql)
	# notify_carrier(pg_cur)
	invoice_log_start_time = datetime.date.today().strftime(("%Y-%m-%d %H:%M:%S"))
	pg_cur.execute(INVOICE_SQL)
	result = pg_cur.fetchall()
	# debug:
	for row in result:
		print(row.get("client_id", "N/A"), row.get("days", "N/A"))
	pg_cur.execute(INVOICE_COUNT_SQL)
	result_count = pg_cur.fetchall()
	# print ("result:", len (result) )
	# print ("result_count:", result_count)
	try:
		client_dict = makeClientDict(result_count)  # '14483': {'ingress_count': 2, 'client_id': 14483}
	except Exception as e:
		print("Error in makeClientDict %s" % e)
		logger.error("Error in makeClientDict %s" % e)
	# print ("client_dict:" ,client_dict )
	total_invoices = 0
	try:
		today = datetime.datetime.now()
		total_invoices, invoices_needed = checkClients(result, client_dict, total_invoices, pg_cur, today)
	except Exception as e:
		print("Error in checkClients %s" % e)
		logger.error("Error in checkClients %s" % e)
	print("total inv:", total_invoices)  # , invoices_needed
	##only for debug!!
	if not True:
		print("test exit")
		return 0
	sql_get_invoice_id = "SELECT nextval('invoice_log_id_seq'::regclass) as no"
	try:
		pg_cur.execute(sql_get_invoice_id)
		result_id = pg_cur.fetchone()
		invoice_log_id = result_id['no']

		sql_insert_log = "insert into invoice_log(id,start_time,status,cnt) values(%s,'%s',%s,%s) " % (
		invoice_log_id, invoice_log_start_time, 1, total_invoices)
		pg_cur.execute(sql_insert_log)
	except Exception as e:
		print("Error in Insert invoice_log %s" % e)
		logger.error("Error in Insert invoice_log %s" % e)
	# makeNewInv(invoices_needed,pg_cur )
	#fields = GetCdrFields(pg_cur)
	EmailTemplate = GetEmailTemplate(pg_cur)
	smtp_info = get_smtp_info(pg_cur)
	jobs = []
	print("Invoices neede on that loop %s" % len(invoices_needed))
	for client in invoices_needed:  # client[0] - client , start_date, end_date, invoice_due_date
		print("Client for invoicing", client[0].copy())
		try:
			status, filename, invoice_number = MakeInvoiceAuto(client, pg_cur)
		except Exception as e:
			print("Error in MakeInvoiceAuto %s" % e)
			logger.error("Error in MakeInvoiceAuto %s" % e)
			continue
		# check for invoice_zero & bill = 0
		# print (status,filename,invoice_number)
		if invoice_number == -10:
			print("Zero invoice is not generated")
			logger.debug("Zero invoice is not generated")
			continue

		if invoice_number == -11:
			print("Missing data from DB")
			logger.error("Missing data from DB")
			continue

		# print ("New Invoice:" , status,filename,invoice_number)
		data = {'start_date': client[1], 'end_date': client[2]}
		if status == -1:
			logger.error("Failed to generate Invoice for %s ", client[0].get('name', "N/A"))
			continue
		if client[0].get('email_invoice', False) == True:
			# email_invoice (filename, client[0],pg_cur,data,fields )
			# p = multiprocessing.Process(target=email_invoice, args=(filename, client[0],pg_cur,data ,fields,smtp_info,EmailTemplate,invoice_number ) )
			# p.daemon = True
			# p.start()
			# jobs.append(p)
			# print (jobs)
			try:
				pdf_file='{}/{}'.format(config['other']["pdf_path"],filename)
				email_invoice(pdf_file, client[0], data, smtp_info, "", False, invoice_number, pg_cur)
				invoice_history(filename, client[0], pg_cur, client[1], client[2], True)
			except Exception as e:
				print("Error in email_invoice %s" % e)
				logger.error("Error in email_invoice %s" % e)
				traceback.print_tb(e.__traceback__)
				continue

		else:
			print("Client %s  Do not want email" % client[0].get('client_id'))
			invoice_history(filename, client[0], pg_cur, client[1], client[2], True)
	sql_update_log = "update invoice_log  SET end_time =current_timestamp(0) ,status = 2,cnt = %s WHERE id = %s  " % (
	total_invoices, invoice_log_id)
	print("UPDATE IN INVOICE_LOG for FINISH %s" % sql_update_log)
	logger.debug("UPDATE IN INVOICE_LOG for FINISH %s" % sql_update_log)
	try:
		pg_cur.execute(sql_update_log)
	except Exception as e:
		print('Error in logging script execution in table invoice_log %s' % str(e))
		logger.error('Error in logging script execution in table invoice_log %s' % str(e))

	return 0



def GetClient(params, pg_cur):
	sql = """ select * from client where client_id =  %s """ % params['client_id']
	try:
		pg_cur.execute(sql)
		client = pg_cur.fetchone()
	except Exception as e:
		# print ("SQL for updating cdr path in DB:  %s" % (str(e) ) )
		logger.error("Exception in  Get Client Details  %s" % (str(e)))
		traceback.print_tb(e.__traceback__)
		return -1
	return client


def reSendEmail(task, pg_cur, logger, config):
	try:
		send_to = task['api_params']
		smtp_info = get_smtp_info(pg_cur)
		EmailTemplate = GetEmailTemplate(pg_cur)
		client = GetClient(task, pg_cur)
		params = task
		print("smtp_info", smtp_info, "EmailTemplate", EmailTemplate, "client", client)
		subject = '''Hello %s. This is resend of your invoice'''
		data = {'start_date': str(params['invoice_start']).replace("+00:00", ""),
				'end_date': str(params['invoice_end']).replace("+00:00", "")}
	except Exception as e:
		print("Error inside in reSendEmail getting parameters %s" % e)
		logger.error("Error inside in reSendEmail getting parameters %s" % e)

	try:
		cdr_link = getCdrLink(client, pg_cur, params['invoice_id'], data)
	except Exception as e:
		print("Error inside get cdr link ", e)
		return -2
	try:
		result = email_invoice(params['pdf_path'][params['pdf_path'].rfind('/') + 1:], client, data, smtp_info,
								 send_to, True, params['invoice_number'], pg_cur)
		print("result of EmailInvoice New", result)
		return result
	except Exception as e:
		print("Error inside email invoice ", e)
		return -1


def api_tasker(event):
	"""Task subprogram for performing DB invoice taks"""
	### setup, logging ,pg_cursor
	print("Api Tasker ver 0.1")
	global config  # pidfile
	config = configparser.ConfigParser()
	config.read('config.ini')

	global DEBUG_LEVEL
	DEBUG_LEVEL = int(config["log"]["debug_level"])
	# cdr_table = ""  #config["other"]["cdr_table"]
	# logger
	if int(config["log"].get('by_time', 0)) == 1:
		handler = TimedRotatingFileHandler(config["log"].get("invoice_task_path", "invoice_task.log"),
										   when=config["log"].get("invoice_task_when", 'm'),
										   interval=int(config["log"].get("invoice_task_interval", 0)),
										   backupCount=int(config["log"].get("invoice_task_backupCount", 5)))
	else:  # by size
		handler = RotatingFileHandler(config["log"].get("invoice_task_path", "invoice_task.log"),
									  maxBytes=int(config["log"].get("invoice_task_maxBytes", 1024 * 1024 * 128)),
									  backupCount=int(config["log"].get("invoice_task_backupCount", 5)))

	if int(config["log"].get("debug_level", 0)) == 1:
		logger.setLevel(logging.DEBUG)
	else:
		logger.setLevel(logging.INFO)
	formatstr = '%(asctime)s - %(name)s - %(levelname)s - %(message)s'
	formatstr = '%(asctime)s: %(levelname)s:  %(message)s'
	formatter = logging.Formatter(formatstr)
	handler.setFormatter(formatter)
	logger.addHandler(handler)

	try:
		pg_conn, pg_cur = connect_to_postgresql(config["POSTGRES"]["hostaddr"], int(config["POSTGRES"]["port"]),
												config["POSTGRES"]["dbname"], config["POSTGRES"]["user"],
												config["POSTGRES"]["password"])  # config["POSTGRES"]["password"]
	except Exception as e:
		print(" Exception in connect_to_postgresql:  %s" % (str(e)))
		logger.error(" Exception in connect to postgresql in api tasker:  %s" % (str(e)))
		return -1

	while True:
		if event.is_set():
			logger.info("TERM SIG! EXITING NOW ")
			print("Api tasker Process: TERM SIG! EXITING NOW ")
			exit(0)
		# getting taks
		sql = """SELECT * FROM invoice where api_requests  > 0 """
		pg_cur.execute(sql)
		tasks = pg_cur.fetchall()
		if tasks == None:
			time.sleep(10)
			continue
		for task in tasks:
			if event.is_set():
				logger.info("TERM SIG! EXITING NOW ")
				print("Api tasker Process: TERM SIG! EXITING NOW ")
				exit(0)
			if task['api_requests'] == 1:
				ret_msg = reSendEmail(task, pg_cur, logger, config)
				print("ret_msg: ", ret_msg)
				if ret_msg == 0:
					# update invoice that the task is complete
					sql = """UPDATE invoice set api_requests  = 0 where invoice_id = %s   """ % task['invoice_id']
					try:
						pg_cur.execute(sql)
					except Exception as e:
						print(" Exception in Updating invoice :  %s" % (str(e)))
						logger.error(" Exception in Updating invoice :  %s" % (str(e)))

		time.sleep(10)


def logVersion(pg_cur):
	global VERSION
	worker_info = VERSION  # VERSION = ['ver':"1.0.1",'date':'2017-05-25']
	now_time = time.strftime("%Y-%m-%d %H:%M:%S", time.gmtime()) + "+00"
	program_name_dmon = "dnl_invoice_AutoInvoice"
	program_name_proc = "dnl_invoice_AutoInvoice"
	git_ver = ""  # str( git_version()  , 'utf-8')

	sql = """   SELECT count(*) as count FROM version_information WHERE program_name = '%s'""" % program_name_dmon
	print(sql)
	pg_cur.execute(sql)
	count = pg_cur.fetchone()['count']
	if count == 0:
		sql = """INSERT INTO version_information (program_name ,major_ver,build_date ,start_time ) VALUES (%s,%s,%s,%s)  """

	else:
		sql = """UPDATE  version_information  SET  major_ver = '%s' , build_date = '%s',start_time = '%s' WHERE  program_name = '%s'; """ % (
		VERSION['ver'], VERSION['date'], now_time, program_name_dmon)

	print(sql)
	try:
		pg_cur.execute(sql, (program_name_dmon, VERSION['ver'], VERSION['date'], now_time))


	except Exception as e:
		print("Except in versioning %s" % e)
		raise


def worker(event):
	# jobs.append(p)
	# killer = GracefulKiller(p)

	while True:
		try:
			pg_conn, pg_cur = main(sys.argv)
			if pg_cur == -1:
				exit(1)
			main_start(pg_conn, pg_cur)
		except Exception as e:
			logger.error("Error in Main Auto start loop  %s" % str(e))
			print("Error in Main Auto start loop7 %s " % str(e))
			try:
				time.sleep(60)  # sleep for a minute before continue
				pg_conn, pg_cur = main(sys.argv)
				continue
			except Exception as e:
				logger.error("SECOND Error in Main Auto start loop  %s" % str(e))
				print("SECOND Error in Main Auto start loop7 %s " % str(e))
				# pg_conn.close()
				# pg_cur.close()
				exit(1)
		for dummy in range(60 * 6):
			if event.is_set():
				logger.info("TERM SIG! EXITING NOW ")
				print("Auto Process: TERM SIG! EXITING NOW ")
				exit(0)

			time.sleep(10)


def config_pg():
	import configparser, argparse
	global config  # pidfile
	config = configparser.ConfigParser()
	config.read('config.ini')
	try:
		pg_conn, pg_cur = connect_to_postgresql(config["POSTGRES"]["hostaddr"], int(config["POSTGRES"]["port"]),
		                                        config["POSTGRES"]["dbname"], config["POSTGRES"]["user"],
		                                        config["POSTGRES"]["password"])  # config["POSTGRES"]["password"]
	except Exception as e:
		print(" Exception in connect_to_postgresql:  %s" % (str(e)))
		logger.error(" Exception in connect_to_postgresql:  %s" % (str(e)))
	print('ok')
	return pg_cur

def autoTest(data):
	"""test auto invoice schedule"""

	test_client_id=int(data['client_id'])
	pg_cur=config_pg()

	pg_cur.execute(INVOICE_SQL + ' and client.client_id= {}'.format(test_client_id))
	result = pg_cur.fetchall()
	for row in result:
		print(row.get("client_id", "N/A"), row.get("days", "N/A"))
	pg_cur.execute("""select resource.client_id,count(resource.ingress is true) from resource join
        client on resource.client_id=client.client_id where resource.client_id= {} group by resource.client_id having
        count(resource.ingress is true) > 0;""".format(test_client_id))
	result_count = pg_cur.fetchall()
	# print ("result:", len (result) )
	# print ("result_count:", result_count)
	try:
		client_dict = makeClientDict(result_count)  # '14483': {'ingress_count': 2, 'client_id': 14483}
	except Exception as e:
		print("Error in makeClientDict %s" % e)
		logger.error("Error in makeClientDict %s" % e)
	print("client_dict:", client_dict)
	total_invoices = 0
	today = datetime.datetime.now()

	today = datetime.datetime.strptime(data['date'],"%Y-%m-%d")
	total_invoices, invoices_needed = checkClients(result, client_dict, total_invoices, pg_cur, today)
	result = ''
	if len(invoices_needed):
		for item in invoices_needed:
			result+= 'for client {} at {} will generated invoice start {} end {}'.format(test_client_id,today,item[1],item[2])
	else:
		result = 'no invoice for client {} at {} will generated'.format(test_client_id,today)
	return result

def autoFire(data):
	"""test auto invoice schedule"""

	test_client_id=int(data['client_id'])
	pg_cur=config_pg()

	pg_cur.execute(INVOICE_SQL + ' and client.client_id= {}'.format(test_client_id))
	result = pg_cur.fetchall()
	for row in result:
		print(row.get("client_id", "N/A"), row.get("days", "N/A"))
	pg_cur.execute("""select resource.client_id,count(resource.ingress is true) from resource join
        client on resource.client_id=client.client_id where resource.client_id= {} group by resource.client_id having
        count(resource.ingress is true) > 0;""".format(test_client_id))
	result_count = pg_cur.fetchall()
	# print ("result:", len (result) )
	# print ("result_count:", result_count)
	try:
		client_dict = makeClientDict(result_count)  # '14483': {'ingress_count': 2, 'client_id': 14483}
	except Exception as e:
		print("Error in makeClientDict %s" % e)
		logger.error("Error in makeClientDict %s" % e)
	print("client_dict:", client_dict)
	total_invoices = 0
	today = datetime.datetime.now()

	today = datetime.datetime.strptime(data['date'],"%Y-%m-%d")
	total_invoices, invoices_needed = checkClients(result, client_dict, total_invoices, pg_cur, today)
	result = ''
	smtp_info = get_smtp_info(pg_cur)
	if len(invoices_needed):
		for item in invoices_needed:
			result+= 'for client {} at {} will generated invoice start {} end {}'.format(test_client_id,today,item[1],item[2])
			for client in invoices_needed:  # client[0] - client , start_date, end_date, invoice_due_date
				print("Client for invoicing", client[0].copy())
				try:
					status, filename, invoice_number = MakeInvoiceAuto(client, pg_cur)
				except Exception as e:
					print("Error in MakeInvoiceAuto %s" % e)
					logger.error("Error in MakeInvoiceAuto %s" % e)
					continue
				# check for invoice_zero & bill = 0
				# print (status,filename,invoice_number)
				if invoice_number == -10:
					print("Zero invoice is not generated")
					logger.debug("Zero invoice is not generated")
					continue

				if invoice_number == -11:
					print("Missing data from DB")
					logger.error("Missing data from DB")
					continue

				# print ("New Invoice:" , status,filename,invoice_number)
				data = {'start_date': client[1], 'end_date': client[2]}
				if status == -1:
					logger.error("Failed to generate Invoice for %s ", client[0].get('name', "N/A"))
					continue
				if client[0].get('email_invoice', False) == True:
					# email_invoice (filename, client[0],pg_cur,data,fields )
					# p = multiprocessing.Process(target=email_invoice, args=(filename, client[0],pg_cur,data ,fields,smtp_info,EmailTemplate,invoice_number ) )
					# p.daemon = True
					# p.start()
					# jobs.append(p)
					# print (jobs)
					try:
						pdf_file = '{}/{}'.format(config['other']["pdf_path"], filename)
						email_invoice(pdf_file, client[0], data, smtp_info, "", False, invoice_number, pg_cur)
						invoice_history(filename, client[0], pg_cur, client[1], client[2], True)
					except Exception as e:
						print("Error in email_invoice %s" % e)
						logger.error("Error in email_invoice %s" % e)
						traceback.print_tb(e.__traceback__)
						continue

				else:
					print("Client %s  Do not want email" % client[0].get('client_id'))
					invoice_history(filename, client[0], pg_cur, client[1], client[2], True)
	else:
		result = 'no invoice for client {} at {} will generated'.format(test_client_id,today)
	return result

if __name__ == '__main__':

	parser = argparse.ArgumentParser(description='Invoice DID API')
	parser.add_argument('-m', '--minutes', default='60', help='port')  # default='start'
	# parser.add_argument('-j', '--host', default= '0.0.0.0' , help='host')
	args = parser.parse_args()
	try:
		minutes = int(args.minutes)
	except:
		minutes = 60
	pg_conn, pg_cur = main(sys.argv)
	if pg_cur == -1:
		exit(1)
	logVersion(pg_cur)
	pidCheck()
	GracefulKiller()
	while True:
		main_start(pg_conn, pg_cur)
		for dummy in range(20 * minutes):
			time.sleep(3)
	# pg_cur.close()
# pg_conn.close()

# sys.exit(main(sys.argv))
