#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# version from 08-11-2017

from reportlab.lib.pagesizes import A4, A3
from reportlab.lib.styles import getSampleStyleSheet
from reportlab.lib.units import mm, inch
from reportlab.pdfgen import canvas
from reportlab.platypus import Image, Paragraph, Table, TableStyle
from reportlab.lib.styles import ParagraphStyle
from reportlab.lib.enums import TA_LEFT, TA_CENTER, TA_RIGHT, TA_JUSTIFY
from reportlab.lib import colors
from reportlab.pdfbase.pdfmetrics import registerFont
from reportlab.pdfbase import pdfmetrics
from reportlab.pdfbase.ttfonts import TTFont
from reportlab.lib.fonts import addMapping
from reportlab.pdfbase.pdfmetrics import stringWidth
from reportlab.platypus import SimpleDocTemplate, Paragraph, Spacer, Image, Table, PageBreak
import math
from reportlab.platypus.flowables import KeepTogether

########################################################################
ROWHEIGHT = 15
MM_TO_PIXEL = 297 / 842
PIXEL_TO_MM = 842 / 297

import html
from html.parser import HTMLParser
import re

def cleanhtml(raw_html):
  cleanr = re.compile('<.*?>')
  raw_html=raw_html.replace('\r', '').replace('\n', '').replace('\t', '')
  cleantext = re.sub(cleanr, '', raw_html)
  return cleantext

def clean_quoted_html(raw_html):
  cleanr = re.compile('&lt;.*?&gt;')
  raw_html=raw_html.replace('&amp;', '').replace('&nbsp;', '').replace('nbsp;', '')
  cleantext = re.sub(cleanr, '', raw_html)
  return cleantext

def html_to_rl(html_src, styleSheet):
    elements = list()

    class Handler(HTMLParser):
        mode = ""
        buffer = ""
        listcounter = 0
        listtype = ""
        color = ""

        def handle_starttag(self, name, attrs):
            if name in ["strong", "em", "i", "b"]:
                self.mode = name
            elif name == "ol":
                self.listcounter = 1
                self.listtype = "ol"
            elif name == "ul":
                self.listtype = "ul"
            elif name == "hr":
                elements.append(PageBreak())
            elif name == "span":
                self.mode = name
                attrs = dict(attrs)
                if 'style' in attrs:
                    style = attrs['style'].split(':')
                    if len(style) == 2:
                        self.color = style[1]

        def handle_endtag(self, name):
            if name.startswith("h") and name[-1] in ["1", "2", "3", "4", "5", "6"]:
                elements.append(Paragraph(self.buffer, styleSheet["Heading%s" % name[-1]]))
            elif name in ["strong", "em", "i", "b", "span"]:
                self.mode = ""
            elif name == "p":
                # elements.append(Paragraph(self.buffer+"<br/>", styleSheet["BodyText"]))
                elements.append(Paragraph(self.buffer, styleSheet["InfoText"]))
            elif name == "br":
                elements.append(Paragraph(self.buffer, styleSheet["InfoText"]))
            elif name == "li":
                if self.listtype == "ul":
                    elements.append(Paragraph(self.buffer, styleSheet["InfoText"], bulletText=u"•"))
                else:
                    elements.append(Paragraph(self.buffer, styleSheet["InfoText"], bulletText="%s." % self.listcounter))
                    self.listcounter += 1
            elif name in ["ol", "ul"]:
                self.listcounter = 0
                self.listtype = ""

            if name in ["h1", "h2", "h3", "h4", "h5", "h6", "p", "li","br"]:
                self.buffer = ""

        def handle_data(self, chars):
            surrounding = None
            if self.mode in ["span"]:
                if self.color:
                    surrounding = "font color={}".format(self.color)
            elif self.mode in ["strong", "em", "i", "b"]:
                if self.mode in ["strong", "b"]:
                    surrounding = "b"
                else:
                    surrounding = "i"

            if surrounding:
                chars = "<%s>%s</%s>" % (surrounding, chars, surrounding)

            self.buffer += chars

    if '&lt;' in html_src:
        html_src = html.unescape(html_src)
        html_src =html_src.replace('<br>', '<br/>').replace('<BR>', '<br/>').replace('</BR>', '<br/>').replace('</br>', '<br/>')

    try:
        parseString = Handler()
        parseString.feed('<html>{}</html>'.format(html_src))
        parseString.close()
    except:
        elements = [Paragraph(html_src, styleSheet["InfoText"])]

    return elements

from  version import VERSION

class LetterMaker(object):
    """"""

    # ----------------------------------------------------------------------
    def __init__(self, pdf_file, org, client_info, summary_of_charges, acount_sumary, summary_of_payments, usage_detail,
                 get_jurisdict, day_usage, ingress_prefix,
                 show_trafic_code_name, show_trafic_country, show_code_summery, detail_trunk, show_calls_date,
                 include_origination_billing, logo_file, switch_settings, ShowDetailByCodeName,
                 breakdown_by_rate_table, is_jurisdict=False):
        if is_jurisdict == False:
            self.c = SimpleDocTemplate(pdf_file, pagesize=A4, rightMargin=10, leftMargin=10)
        else:
            self.c = SimpleDocTemplate(pdf_file, pagesize=A4, rightMargin=5, leftMargin=5)
        self.c.title = 'Invoice # {} for {}'.format(client_info[2],client_info[0])
        self.c.creator = 'dnl_invoice_generator ver. {ver} date {date}'.format(**VERSION)
        self.c.subject = 'Billing period: {} '.format(client_info[5])
        self.styles = getSampleStyleSheet()
        self.width, self.height = A4
        self.page_width = self.width
        # self.stylesp =  ParagraphStyle()
        self.organization = org
        # self.seconds  = 10
        self.client_info = client_info
        self.summary_of_charges = summary_of_charges
        self.acount_sumary = acount_sumary
        self.summary_of_payments = summary_of_payments
        self.usage_detail = usage_detail
        self.get_jurisdict = get_jurisdict
        self.day_usage = day_usage
        self.ingress_prefix = ingress_prefix
        self.ShowDetailByCodeName = ShowDetailByCodeName
        self.show_trafic_code_name = show_trafic_code_name
        self.show_trafic_country = show_trafic_country
        self.show_code_summery = show_code_summery
        self.detail_trunk = detail_trunk
        self.show_calls_date = show_calls_date
        self.include_origination_billing = include_origination_billing
        self.logo_file = logo_file
        self.switch_settings = switch_settings
        self.tpl_number = self.switch_settings.get('tpl_number', 0)
        # print ("self.tpl_number ",self.tpl_number)
        self.breakdown_by_rate_table = breakdown_by_rate_table
        self.parts = []
        self.Story = []
        self.styleSheet = getSampleStyleSheet()
        self.styleN = self.styleSheet["BodyText"]
        self.styleN.fontSize = 9
        self.styleN.name = "InfoText"
        self.styleSheet.add(self.styleN)
        self.style = self.styleSheet["BodyText"]
        self.drawHeight = 0.5 * inch
        self.drawWidth = 1 * inch
        self.is_jurisdict = is_jurisdict

    def get_total_by_column(self, table, column, column_type=float, round_digits=0):
        print(table[0][column])
        print(type(table[0][column]))
        print(float(table[0][column]))
        return round(sum([column_type(row[column]) for row in table]), round_digits)

    # ----------------------------------------------------------------------
    def createDocument(self):
        client_info = self.client_info
        print("client_info", client_info)
        client_info[1] = """Address:%s<br/> """ % client_info[1] if client_info[1] != None and len(
            client_info[1]) != 0 else ""
        if client_info[6] == None:
            client_info[6] = '+0'
        company_info = self.switch_settings.get("company_info", "N/A")
        if company_info == None or company_info.strip() == "":
            company_info = " N/A  "
        try:
            denovo_info = """<font size=9> %s
		</font>
		""" % company_info.replace("<br />", "").replace("<br/>", "").replace('</p>\r', '</p><br/> ').strip()  # [1:]
        except:
            denovo_info = "N/A "

        print("createDocument")
        print("denovo_info", denovo_info)

        bill_address = """<para align=right> <font size="9">
		Billed to:%s<br/>
		%s
		<b>Invoice#</b> %s<br/>
		<b>Invoice Date:</b>%s<br/>
		<b>Payment Due Date:</b> %s<br/>
		<b>Billing Period:</b> %s<br/>
		<b>Invoice time zone:</b> GMT %s</font></para>
		"""
        bill_address = bill_address % (
        client_info[0], client_info[1], client_info[2], client_info[3], client_info[4], client_info[5], client_info[6])
        logo_style = self.styles["Normal"]
        logo_style.alignment = TA_LEFT
        # ParagraphStyle
        # address_p = Paragraph(denovo_info,self.styleN )
        address_p = html_to_rl(denovo_info, self.styleSheet)
        bill_address_p = Paragraph(bill_address, self.style)  # self.style['right']
        # bill_address_p.wrapOn(self.c, self.width-200, self.height)

        # add a logo and size it
        logo = Image(self.logo_file)
        logo.drawHeight = logo.drawHeight * inch / 300  # inch
        logo.drawWidth = inch

        if self.tpl_number == 1:
            try:
                denovo_billing = """<font size=9>%s</font>""" % self.switch_settings.get("pdf_tpl",
                                                                                         "N/A")  # .replace('</p>\r','</p><br/>').replace('<br />','<br/>').replace('\r','<br/>').replace('：',':').strip()
                # denovo_billing_p = Paragraph(denovo_billing, self.styleN)
                denovo_billing_p = html_to_rl(denovo_billing, self.styleSheet)
            except:
                denovo_billing = "<font size=9> N/A</font>"
                denovo_billing_p = Paragraph(denovo_billing, self.styleN)

            print("denovo_billing", denovo_billing)
        else:
            denovo_billing_p = ""

        self.c.author = clean_quoted_html(self.switch_settings.get("pdf_tpl", "N/A"))

        data = [[logo, bill_address_p], ["", ""], [address_p, ""], [denovo_billing_p, ""]]  # ,["",bill_address_p]]
        table = Table(data)

        table.setStyle([('VALIGN', (0, 0), (1, 1), 'MIDDLE'), ('VALIGN', (0, 1), (0, 1), 'TOP'), (
        'ALIGN', (0, 1), (0, 1), 'LEFT')])  # .setStyle(TableStyle([('VALIGN',(-1,-1),(-1,-1),'MIDDLE')]))

        self.Story.append(table)
        self.Story.append(Spacer(self.width, 30))

        # ACOUNT SUMMARY
        if self.acount_sumary != None and len(self.acount_sumary) > 1:
            plabel = Paragraph("<b><font size=11 > Account Summary</font></b>", self.style)
            data = [[plabel, ""]]
            data.extend(self.acount_sumary)
            print("Account summery", len(data))
            summary_report = Table(data)
            summary_report.setStyle(TableStyle([
                ('VALIGN', (0, 0), (-1, -1), 'TOP'),
                ('BACKGROUND', (0, 0), (-1, -1), colors.HexColor('#f4f4f4')),
                ('SPAN', (0, 0), (-1, 0)),
                ('LINEABOVE', (0, 1), (-1, 1), 0.5, colors.black),
                # ('BACKGROUND', (0, 2), (-1, 2), colors.HexColor('#b5b5b5')),
                ##('BACKGROUND', (0, 3), (-1, -1), colors.HexColor('#e5e5e5')),
                ('INNERGRID', (0, -1), (-1, -1), 0.5, colors.white),
                ('ALIGN', (-1, 0), (-1, -1), 'RIGHT'),
                # ('SPAN', (0, -1), (1, -1)),
                # ('BACKGROUND', (0, -1), (-1, -1), colors.HexColor('#b5b5b5')),
            ]
            ))

            self.Story.append(summary_report)
            self.Story.append(Spacer(self.width, 30))

        # Summary of Charges
        print("Summary of Charges")
        plabel = Paragraph("<b><font size=11 > Summary of Charges</font></b>", self.style)
        data = [[plabel, ""]]
        data.extend(self.summary_of_charges)
        charges_report = Table(
            data, style=[
                ('VALIGN', (0, 0), (-1, -1), 'TOP'),
                ('BACKGROUND', (0, 0), (-1, -1), colors.HexColor('#f4f4f4')),
                ('SPAN', (0, 0), (-1, 0)),
                ('LINEABOVE', (0, 1), (-1, 1), 0.5, colors.black),
                ('BACKGROUND', (0, 1), (-1, 1), colors.HexColor('#b5b5b5')),
                ##('BACKGROUND', (0, 3), (-1, -1), colors.HexColor('#e5e5e5')),
                ('INNERGRID', (0, 1), (-1, -1), 0.5, colors.white),
                ('ALIGN', (1, 2), (-1, -1), 'RIGHT'),
                ('SPAN', (0, -1), (-2, -1)),
                # ('BACKGROUND', (0, -1), (-1, -1), colors.HexColor('#b5b5b5')),
            ])
        # print("charges_report",charges_report)
        self.Story.append(charges_report)
        self.Story.append(Spacer(self.width, 30))

        # summary_of_payments
        if self.summary_of_payments != None:
            if len(self.summary_of_payments) > 1:

                print("summary_of_payments ")
                plabel = Paragraph("<b><font size=11 > Summary of Payments</font></b>", self.style)
                data = [[plabel, ""]]
                data.extend(self.summary_of_payments)
                # print ("self.summary_of_payments",self.summary_of_payments)
                # total_row =  ["Total" ,str(self.get_total_by_column(self.summary_of_payments[1:], 1))]
                # print("total_row",total_row)
                # data.extend(total_row)
                payments_report = Table(
                    data, style=[
                        ('VALIGN', (0, 0), (-1, -1), 'TOP'),
                        ('BACKGROUND', (0, 0), (-1, -1), colors.HexColor('#f4f4f4')),
                        ('SPAN', (0, 0), (-1, 0)),
                        ('LINEABOVE', (0, 1), (-1, 1), 0.5, colors.black),
                        ('BACKGROUND', (0, 1), (-1, 1), colors.HexColor('#b5b5b5')),
                        # ('BACKGROUND', (0, 3), (-1, -1), colors.HexColor('#e5e5e5')),
                        ('INNERGRID', (0, 1), (-1, -1), 0.5, colors.white),
                        ('ALIGN', (1, 2), (1, -1), 'RIGHT'),
                    ])
            else:
                # http://jira.denovolab.com/browse/CLAS-5787
                print("summary_of_payments ")
                plabel = Paragraph("<b><font size=11 > Summary of Payments</font></b>", self.style)
                data = [[plabel, ""]]
                data.extend(self.summary_of_payments)
                # greyRow  = Paragraph("<b><font size=11 >  </font></b>",self.style  )
                data.extend(["", ""])
                # print ("self.summary_of_payments",self.summary_of_payments)
                # total_row =  ["Total" ,str(self.get_total_by_column(self.summary_of_payments[1:], 1))]
                # print("total_row",total_row)
                # data.extend(total_row)
                payments_report = Table(
                    data, style=[
                        ('VALIGN', (0, 0), (-1, -1), 'TOP'),
                        ('BACKGROUND', (0, 0), (-1, -1), colors.HexColor('#f4f4f4')),
                        ('SPAN', (0, 0), (-1, 0)),
                        ('LINEABOVE', (0, 1), (-1, 1), 0.5, colors.black),
                        ('BACKGROUND', (0, 1), (-1, 1), colors.HexColor('#b5b5b5')),  # table header
                        ('BACKGROUND', (0, 2), (-1, 2), colors.HexColor('#b5b5b5')),  # greyed row
                        ('INNERGRID', (0, 1), (-1, -1), 0.5, colors.white),
                        ('ALIGN', (1, 2), (1, -1), 'RIGHT'),
                    ])

            self.Story.append(KeepTogether(payments_report))
            self.Story.append(Spacer(self.width, 30))

        # day_usage

        if self.day_usage != None and len(self.day_usage) > 1:
            print("day_usage_report")
            if not self.is_jurisdict:
                plabel = Paragraph("<b><font size=11 > Daily Usage</font></b>", self.style)
                data = [[plabel, ""]]
                data.extend(self.day_usage)
                day_usage_report = Table(
                    data, style=[
                        ('VALIGN', (0, 0), (-1, -1), 'TOP'),
                        ('BACKGROUND', (0, 0), (-1, -1), colors.HexColor('#f4f4f4')),
                        ('SPAN', (0, 0), (-1, 0)),
                        ('LINEABOVE', (0, 1), (-1, 1), 0.5, colors.black),
                        ('BACKGROUND', (0, 1), (-1, 1), colors.HexColor('#b5b5b5')),
                        # ('BACKGROUND', (0, 3), (-1, -1), colors.HexColor('#e5e5e5')),
                        ('INNERGRID', (0, 1), (-1, -1), 0.5, colors.white),
                        ('ALIGN', (0, 2), (-1, -1), 'RIGHT'),
                        ('SPAN', (0, -1), (1, -1)),
                        ('BACKGROUND', (0, -1), (-1, -1), colors.HexColor('#b5b5b5')), ])

            else:
                plabel = Paragraph("<b><font size=11 > Daily Usage</font></b>", self.style)
                data = [[plabel, ""]]
                data.extend(self.day_usage)
                day_usage_report = Table(
                    data, style=[
                        ('VALIGN', (0, 0), (-1, -1), 'TOP'),
                        ('BACKGROUND', (0, 0), (-1, -1), colors.HexColor('#f4f4f4')),
                        ('SPAN', (0, 0), (-1, 0)),  # label
                        ('LINEABOVE', (0, 1), (-1, 1), 0.5, colors.black),
                        ('BACKGROUND', (0, 1), (-1, 2), colors.HexColor('#b5b5b5')),
                        ('FONTSIZE', (0, 1), (-1, 2), 10),  # headers
                        ('FONTSIZE', (0, 3), (-1, -1), 9),  # cells
                        ('FONTSIZE', (0, -1), (0, -1), 10),  # Total cell
                        ('INNERGRID', (0, 1), (-1, -1), 0.5, colors.white),
                        ('ALIGN', (0, 3), (-1, -1), 'RIGHT'),
                        ('ALIGN', (0, 1), (-1, 2), 'CENTER'),  # headers one
                        ('SPAN', (1, 1), (2, 1)), ('SPAN', (3, 1), (4, 1)), ('SPAN', (5, 1), (6, 1)),
                        ('SPAN', (7, 1), (8, 1)),
                        ('BACKGROUND', (0, -1), (-1, -1), colors.HexColor('#b5b5b5')), ], colWidths=62.5)

            # print ("day_usage_report new:",day_usage_report)
            #self.Story.append(KeepTogether(day_usage_report))
            self.Story.append(Spacer(self.width, 30))
            self.Story.append(day_usage_report)
            self.Story.append(Spacer(self.width, 30))

        # ingress_prefix
        if self.ingress_prefix != None and len(self.ingress_prefix) > 1:
            plabel = Paragraph("<b><font size=11 > Summary By Trunk</font></b>", self.style)
            inp = self.ingress_prefix
            st_wrap = self.style
            st_wrap.wordWrap = 'LTR'
            for row in inp:
                row[0] = Paragraph(row[0], st_wrap)
            data = [[plabel, ""]]
            data.extend(inp)  # self.ingress_prefix)
            print("Summary By Trunk")
            if not self.is_jurisdict:
                # data.extend(total_row)
                tb_report_st = Table(
                    data, style=[
                        ('VALIGN', (0, 0), (-1, -1), 'TOP'),
                        ('BACKGROUND', (0, 0), (-1, -1), colors.HexColor('#f4f4f4')),
                        ('SPAN', (0, 0), (-1, 0)),
                        ('LINEABOVE', (0, 1), (-1, 1), 0.5, colors.black),
                        ('BACKGROUND', (0, 1), (-1, 1), colors.HexColor('#b5b5b5')),
                        # ('BACKGROUND', (0, 3), (-1, -1), colors.HexColor('#e5e5e5')),
                        ('INNERGRID', (0, 1), (-1, -1), 0.5, colors.white),
                        ('ALIGN', (2, 2), (-1, -1), 'RIGHT'),
                        ('SPAN', (0, -1), (1, -1)),
                        ('BACKGROUND', (0, -1), (-1, -1), colors.HexColor('#b5b5b5')), ])
                self.Story.append(KeepTogether(tb_report_st))
                self.Story.append(Spacer(self.width, 30))

            else:
                tb_report_st = Table(
                    data, style=[
                        ('VALIGN', (0, 0), (-1, -1), 'TOP'),
                        ('BACKGROUND', (0, 0), (-1, -1), colors.HexColor('#f4f4f4')),
                        ('SPAN', (0, 0), (-1, 0)),  # label
                        ('LINEABOVE', (0, 1), (-1, 1), 0.5, colors.black),
                        ('BACKGROUND', (0, 1), (-1, 2), colors.HexColor('#b5b5b5')),
                        ('FONTSIZE', (0, 1), (-1, 2), 10),  # headers
                        ('FONTSIZE', (0, 3), (-1, -1), 9),  # cells
                        ('FONTSIZE', (0, -1), (0, -1), 10),  # Total cell
                        ('INNERGRID', (0, 1), (-1, -1), 0.5, colors.white),
                        ('ALIGN', (1, 3), (-1, -1), 'RIGHT'),
                        ('ALIGN', (0, 3), (1, -2), 'LEFT'),
                        ('ALIGN', (0, 1), (-1, 2), 'CENTER'),  # headers one
                        ('SPAN', (2, 1), (4, 1)), ('SPAN', (5, 1), (7, 1)), ('SPAN', (8, 1), (-1, 1)),
                        ('BACKGROUND', (0, -1), (-1, -1), colors.HexColor('#b5b5b5')), ],
                    # colWidths=[75,56,47.5,47.5,47.5,47.5,47.5,47.5,47.5,47.5,47.5 ]
                    colWidths=[105, 46, 45, 45, 45, 45, 45, 45, 45, 45, 47.5]
                )  # Summary By Trunk

                self.Story.append(KeepTogether(tb_report_st))
                self.Story.append(Spacer(self.width, 30))

            # usage_detail
        if self.usage_detail != None and len(self.usage_detail) > 1:
            print("Usage Detail")
            print(self.usage_detail)
            plabel = Paragraph("<b><font size=11 > Usage Detail</font></b>", self.style)
            data = [[plabel, ""]]
            data.extend(self.usage_detail)
            tb_usage_detail = Table(
                data, style=[
                    ('VALIGN', (0, 0), (-1, -1), 'TOP'),
                    ('BACKGROUND', (0, 0), (-1, -1), colors.HexColor('#f4f4f4')),
                    ('SPAN', (0, 0), (-1, 0)),  # label
                    ('LINEABOVE', (0, 1), (-1, 1), 0.5, colors.black),
                    ('BACKGROUND', (0, 1), (-1, 1), colors.HexColor('#b5b5b5')),
                    ('FONTSIZE', (0, 1), (-1, 1), 10),  # headers
                    ('FONTSIZE', (0, 2), (-1, -1), 9),  # cells
                    ('FONTSIZE', (0, -1), (0, -1), 10),  # Total cell
                    ('INNERGRID', (0, 1), (-1, 1), 0.5, colors.white),
                    ('INNERGRID', (0, 2), (-1, -1), 0.5, colors.black),
                    ('ALIGN', (0, 2), (-1, -1), 'RIGHT'),
                    ('ALIGN', (0, 0), (-1, 1), 'CENTER'),  # headers one
                    # ('SPAN', (2, 1), (4, 1)),('SPAN', (5, 1), (7, 1)) , ('SPAN', (8, 1), (-1, 1)),
                    # ('BACKGROUND', (0, -1), (-1, -1), colors.HexColor('#b5b5b5')),
                ],
                colWidths=[100, 90, 90, 50, 120, 110])  # 560: 260,50,50,50,100,50

            self.Story.append(tb_usage_detail)  # KeepTogether(
            self.Story.append(Spacer(self.width, 30))

        # show_trafic_code_name
        if self.show_trafic_code_name != None and len(self.show_trafic_code_name) > 1:
            print("Show traffic code name")
            print(self.show_trafic_code_name)
            plabel = Paragraph("<b><font size=11 >Code name breakdown</font></b>", self.style)
            data = [[plabel, ""]]
            data.extend(self.show_trafic_code_name)
            if not self.is_jurisdict:
                tb_report_code_name = Table(
                    data, style=[
                        ('VALIGN', (0, 0), (-1, -1), 'TOP'),
                        ('BACKGROUND', (0, 0), (-1, -1), colors.HexColor('#f4f4f4')),
                        ('SPAN', (0, 0), (-1, 0)),
                        ('LINEABOVE', (0, 1), (-1, 1), 0.5, colors.black),
                        ('BACKGROUND', (0, 1), (-1, 1), colors.HexColor('#b5b5b5')),
                        ('BACKGROUND', (0, 3), (-1, -1), colors.HexColor('#e5e5e5')),
                        ('INNERGRID', (0, 1), (-1, -1), 0.5, colors.white),
                        # ('ALIGN', (2, 3), (4, -1), 'RIGHT'),
                        ('SPAN', (0, -1), (1, -1)),
                        ('BACKGROUND', (0, -1), (-1, -1), colors.HexColor('#b5b5b5')), ])
                self.Story.append(KeepTogether(tb_report_code_name))
                self.Story.append(Spacer(self.width, 30))
            elif len(self.show_trafic_code_name) > 3:
                tb_report_code_name = Table(
                    data, style=[
                        ('VALIGN', (0, 0), (-1, -1), 'TOP'),
                        ('BACKGROUND', (0, 0), (-1, -1), colors.HexColor('#f4f4f4')),
                        ('SPAN', (0, 0), (-1, 0)),  # label
                        ('LINEABOVE', (0, 1), (-1, 1), 0.5, colors.black),
                        ('BACKGROUND', (0, 1), (-1, 2), colors.HexColor('#b5b5b5')),
                        ('FONTSIZE', (0, 1), (-1, 2), 9),  # headers
                        ('FONTSIZE', (0, 3), (-1, -1), 7),  # cells
                        ('FONTSIZE', (0, -1), (0, -1), 8),  # Total cell
                        ('INNERGRID', (0, 1), (-1, -1), 0.5, colors.white),
                        ('ALIGN', (0, 3), (-1, -1), 'RIGHT'),
                        ('ALIGN', (0, 1), (-1, 2), 'CENTER'),  # headers one
                        ('INNERGRID', (0, 2), (-1, -1), 0.5, colors.black),
                        ('SPAN', (1, 1), (3, 1)),
                        ('SPAN', (4, 1), (6, 1)),
                        ('SPAN', (7, 1), (9, 1)),
                        ('SPAN', (10, 1), (12, 1)),
                        ('BACKGROUND', (0, -1), (-1, -1), colors.HexColor('#b5b5b5')), ],
                    colWidths=[60, 41.7, 41.7, 41.7, 41.7, 41.7, 41.7, 41.7, 41.7, 41.7, 41.7, 41.7, 41.7])  # ]  )
                self.Story.append(KeepTogether(tb_report_code_name))
                self.Story.append(Spacer(self.width, 30))

            # show_trafic_country
        if self.show_trafic_country != None and len(self.show_trafic_country) > 1:
            print("Show traffic by Country")
            plabel = Paragraph("<b><font size=11 >Show traffic by Country</font></b>", self.style)
            data = [[plabel, ""]]
            data.extend(self.show_trafic_country)
            if not self.is_jurisdict:

                tb_report_trafic_country = Table(
                    data, style=[
                        ('VALIGN', (0, 0), (-1, -1), 'TOP'),
                        ('BACKGROUND', (0, 0), (-1, -1), colors.HexColor('#f4f4f4')),
                        ('SPAN', (0, 0), (-1, 0)),
                        ('LINEABOVE', (0, 1), (-1, 1), 0.5, colors.black),
                        ('BACKGROUND', (0, 1), (-1, 1), colors.HexColor('#b5b5b5')),
                        # ('BACKGROUND', (0, 3), (-1, -1), colors.HexColor('#e5e5e5')),
                        ('INNERGRID', (0, 1), (-1, -1), 0.5, colors.white),
                        # ('ALIGN', (2, 3), (4, -1), 'RIGHT'),
                        # ('SPAN', (0, -1), (1, -1)),
                        ('BACKGROUND', (0, -1), (-1, -1), colors.HexColor('#b5b5b5')), ])
                self.Story.append(KeepTogether(tb_report_trafic_country))
                self.Story.append(Spacer(self.width, 30))

            elif len(self.show_trafic_country) > 3:
                tb_report_trafic_country = Table(
                    data, style=[
                        ('VALIGN', (0, 0), (-1, -1), 'TOP'),
                        ('BACKGROUND', (0, 0), (-1, -1), colors.HexColor('#f4f4f4')),
                        ('SPAN', (0, 0), (-1, 0)),  # label
                        ('LINEABOVE', (0, 1), (-1, 1), 0.5, colors.black),
                        ('BACKGROUND', (0, 1), (-1, 2), colors.HexColor('#b5b5b5')),
                        ('FONTSIZE', (0, 1), (-1, 2), 9),  # headers
                        ('FONTSIZE', (0, 3), (-1, -1), 7),  # cells
                        ('FONTSIZE', (0, -1), (0, -1), 8),  # Total cell
                        ('INNERGRID', (0, 1), (-1, -1), 0.5, colors.white),
                        ('ALIGN', (0, 3), (1, -1), 'LEFT'),
                        ('ALIGN', (1, 3), (-1, -1), 'RIGHT'),
                        ('ALIGN', (0, 1), (-1, 2), 'CENTER'),  # headers one
                        ('SPAN', (1, 1), (3, 1)),
                        ('SPAN', (4, 1), (6, 1)),
                        ('SPAN', (7, 1), (9, 1)),
                        ('SPAN', (10, 1), (12, 1)),
                        ('BACKGROUND', (0, -1), (-1, -1), colors.HexColor('#b5b5b5')), ],
                    colWidths=[60, 41.7, 41.7, 41.7, 41.7, 41.7, 41.7, 41.7, 41.7, 41.7, 41.7, 41.7, 41.7])  # ]  )
                self.Story.append(KeepTogether(tb_report_trafic_country))
                self.Story.append(Spacer(self.width, 30))

        if self.show_code_summery != None and len(self.show_code_summery) > 1:
            plabel = Paragraph("<b><font size=11 >Show Code Summary (top 10)</font></b>", self.style)
            print("Show Code Summary (top 10)")
            data = [[plabel, ""]]
            data.extend(self.show_code_summery)
            # total_row =  ["Total" ,self.get_total_by_column(self.show_code_summery [1:], 1, int),self.get_total_by_column(self.show_code_summery [1:], 2),
            # self.get_total_by_column(self.show_code_summery [1:], 3)]
            tb_report_10 = Table(
                data, style=[
                    ('VALIGN', (0, 0), (-1, -1), 'TOP'),
                    ('BACKGROUND', (0, 0), (-1, -1), colors.HexColor('#f4f4f4')),
                    ('SPAN', (0, 0), (-1, 0)),
                    ('LINEABOVE', (0, 1), (-1, 1), 0.5, colors.black),
                    ('BACKGROUND', (0, 1), (-1, 1), colors.HexColor('#b5b5b5')),
                    # ('BACKGROUND', (0, 3), (-1, -1), colors.HexColor('#e5e5e5')),
                    ('INNERGRID', (0, 1), (-1, -1), 0.5, colors.white),
                    ('ALIGN', (1, 2), (-1, -1), 'RIGHT'),
                    # ('SPAN', (0, -1), (1, -1)),
                    ('BACKGROUND', (0, -1), (-1, -1), colors.HexColor('#b5b5b5')), ])
            self.Story.append(KeepTogether(tb_report_10))
            self.Story.append(Spacer(self.width, 30))

        # detail_trunk
        # self.detail_trunk = None
        if self.detail_trunk != None and len(self.detail_trunk) > 2:
            plabel = Paragraph("<b><font size=11 >Detail By Trunk</font></b>", self.style)
            print("Detail By Trunk")
            data = [[plabel, ""]]
            data.extend(self.detail_trunk)
            if True:  # not self.is_jurisdict:

                tb_report_detail_trunk = Table(
                    data, style=[
                        ('VALIGN', (0, 0), (-1, -1), 'TOP'),
                        ('BACKGROUND', (0, 0), (-1, -1), colors.HexColor('#f4f4f4')),
                        ('SPAN', (0, 0), (-1, 0)),
                        ('LINEABOVE', (0, 1), (-1, 1), 0.5, colors.black),
                        ('BACKGROUND', (0, 1), (-1, 1), colors.HexColor('#b5b5b5')),
                        # ('BACKGROUND', (0, 3), (-1, -1), colors.HexColor('#e5e5e5')),
                        ('INNERGRID', (0, 1), (-1, -1), 0.5, colors.white),
                        # ('ALIGN', (2, 3), (4, -1), 'RIGHT'),
                        # ('SPAN', (0, -1), (1, -1)),
                        ('SPAN', (0, -1), (-2, -1)),
                        ('BACKGROUND', (0, -1), (-1, -1), colors.HexColor('#b5b5b5')), ])
                self.Story.append(KeepTogether(tb_report_detail_trunk))
                self.Story.append(Spacer(self.width, 30))

            elif len(self.detail_trunk) > 3:
                tb_report_detail_trunk = Table(
                    data, style=[
                        ('VALIGN', (0, 0), (-1, -1), 'TOP'),
                        ('BACKGROUND', (0, 0), (-1, -1), colors.HexColor('#f4f4f4')),
                        ('SPAN', (0, 0), (-1, 0)),  # label
                        ('LINEABOVE', (0, 1), (-1, 1), 0.5, colors.black),
                        ('BACKGROUND', (0, 1), (-1, 2), colors.HexColor('#b5b5b5')),
                        ('FONTSIZE', (0, 1), (-1, 2), 10),  # headers
                        ('FONTSIZE', (0, 3), (-1, -1), 9),  # cells
                        ('FONTSIZE', (0, -1), (0, -1), 10),  # Total cell
                        ('INNERGRID', (0, 1), (-1, -1), 0.5, colors.white),
                        ('ALIGN', (0, 3), (-1, -1), 'RIGHT'),
                        ('ALIGN', (0, 1), (-1, 2), 'CENTER'),  # headers one
                        ('SPAN', (1, 1), (3, 1)), ('SPAN', (4, 1), (6, 1)), ('SPAN', (7, 1), (9, 1)),
                        ('SPAN', (10, 1), (-1, 1)),
                        ('BACKGROUND', (0, -1), (-1, -1), colors.HexColor('#b5b5b5')), ],
                    colWidths=[60, 41.7, 41.7, 41.7, 41.7, 41.7, 41.7, 41.7, 41.7, 41.7, 41.7, 41.7, 41.7])  # ]
                self.Story.append(KeepTogether(tb_report_detail_trunk))
                self.Story.append(Spacer(self.width, 30))

        # Show Calls By Date Summary

        if self.show_calls_date != None and len(self.show_calls_date) > 1:
            plabel = Paragraph("<b><font size=11 >Show Calls By Date Summary</font></b>", self.style)
            data = [[plabel, ""]]
            data.extend(self.show_calls_date)
            tb_report_calls_date = Table(
                data, style=[
                    ('VALIGN', (0, 0), (-1, -1), 'TOP'),
                    ('BACKGROUND', (0, 0), (-1, -1), colors.HexColor('#f4f4f4')),
                    ('SPAN', (0, 0), (-1, 0)),
                    ('LINEABOVE', (0, 1), (-1, 1), 0.5, colors.black),
                    ('BACKGROUND', (0, 1), (-1, 1), colors.HexColor('#b5b5b5')),
                    # ('BACKGROUND', (0, 3), (-1, -1), colors.HexColor('#e5e5e5')),
                    ('INNERGRID', (0, 1), (-1, -1), 0.5, colors.white),
                    ('ALIGN', (1, 2), (-1, -1), 'RIGHT'),
                    # ('SPAN', (0, -1), (1, -1)),
                    ('BACKGROUND', (0, -1), (-1, -1), colors.HexColor('#b5b5b5')), ])

            self.Story.append(KeepTogether(tb_report_calls_date))
            self.Story.append(Spacer(self.width, 30))

        # include Origination Billing
        if self.include_origination_billing != None and len(self.include_origination_billing) > 0:
            print("include Origination Billing")
            plabel = Paragraph("<b><font size=11 >Origination Billing</font></b>", self.style)
            data = [[plabel, ""]]
            data.extend(self.include_origination_billing)
            tb_report_origination_billing = Table(
                data, style=[
                    ('VALIGN', (0, 0), (-1, -1), 'TOP'),
                    ('BACKGROUND', (0, 0), (-1, -1), colors.HexColor('#f4f4f4')),
                    ('SPAN', (0, 0), (-1, 0)),
                    ('LINEABOVE', (0, 1), (-1, 1), 0.5, colors.black),
                    ('BACKGROUND', (0, 1), (-1, 1), colors.HexColor('#b5b5b5')),
                    # ('BACKGROUND', (0, 3), (-1, -1), colors.HexColor('#e5e5e5')),
                    ('INNERGRID', (0, 1), (-1, -1), 0.5, colors.white),
                    # ('ALIGN', (2, 3), (4, -1), 'RIGHT'),
                    ('SPAN', (0, -1), (1, -1)),
                    ('BACKGROUND', (0, -1), (-1, -1), colors.HexColor('#b5b5b5')), ])

            self.Story.append(KeepTogether(tb_report_origination_billing))
            self.Story.append(Spacer(self.width, 30))

        # ShowDetailByCodeName
        if self.ShowDetailByCodeName != None and len(self.ShowDetailByCodeName) > 1:
            print("SDCN")
            plabel = Paragraph("<b><font size=11 >Detail by Code Name</font></b>", self.style)
            data = [[plabel, ""]]
            data.extend(self.ShowDetailByCodeName)
            tb_report_ByCodeName = Table(
                data, style=[
                    ('VALIGN', (0, 0), (-1, -1), 'TOP'),
                    ('BACKGROUND', (0, 0), (-1, -1), colors.HexColor('#f4f4f4')),
                    ('SPAN', (0, 0), (-1, 0)),
                    ('LINEABOVE', (0, 1), (-1, 1), 0.5, colors.black),
                    ('BACKGROUND', (0, 1), (-1, 1), colors.HexColor('#b5b5b5')),
                    # ('BACKGROUND', (0, 3), (-1, -1), colors.HexColor('#e5e5e5')),
                    ('INNERGRID', (0, 1), (-1, -1), 0.5, colors.white),
                    # ('ALIGN', (2, 3), (4, -1), 'RIGHT'),
                    ('SPAN', (0, -1), (1, -1)),
                    ('BACKGROUND', (0, -1), (-1, -1), colors.HexColor('#b5b5b5')), ])

            self.Story.append(KeepTogether(tb_report_ByCodeName))
            self.Story.append(Spacer(self.width, 30))

        tpl = self.switch_settings.get("pdf_tpl", "N/A")
        if tpl == None or tpl.strip() == "":
            print("tpl none")
        # pass
        elif self.switch_settings.get("pdf_tpl", "N/A").replace("<p>&nbsp;</p>\r", "").replace("\r", "").replace(" ",
                                                                                                                 "").strip() == "":
            print("tpl none1")

        else:
            if self.tpl_number == 0:
                denovo_billing = """<font  size="9">%s</font> """ % self.switch_settings.get("pdf_tpl", "N/A")
                print("denovo_billing", denovo_billing)
                denovo_billing_p = html_to_rl(denovo_billing, self.styleSheet)
                # denovo_billing_p = Paragraph(denovo_billing, self.style)
                print("denovo_billingP", denovo_billing_p)
                # data = [ [denovo_billing_p ,"",""]]
                data = [[denovo_billing_p]]
                tb_report = Table(data, style=[('INNERGRID', (0, 0), (-1, -1), 0.5, colors.white)])
                print("tb_report1", tb_report)
                self.Story.append(tb_report)  # KeepTogether(

            elif self.tpl_number == 2:  # bottem and middle

                denovo_billing = """<font  size="9">%s</font>""" % self.switch_settings.get("pdf_tpl", "N/A")
                # denovo_billing_p = Paragraph(denovo_billing, self.style)
                denovo_billing_p = html_to_rl(denovo_billing, self.styleSheet)

                data = [["", denovo_billing_p, ""]]
                tb_report = Table(data, style=[('INNERGRID', (0, 0), (-1, -1), 0.5, colors.white)])
                print("tb_report2", tb_report)
                self.Story.append(tb_report)  # KeepTogether()

            else:
                pass

    def addPageNumber(canvas, doc):
        page_num = canvas.getPageNumber()
        text = "Page #%s" % page_num
        canvas.drawRightString(200 * mm, 20 * mm, text)

    # ----------------------------------------------------------------------
    def savePDF(self):
        """"""
        self.c.build(self.Story)

# ----------------------------------------------------------------------

class st:
    def __init__(self):
        self.styleSheet = getSampleStyleSheet()
        self.styleN = self.styleSheet["BodyText"]
        self.styleN.fontSize = 9
        self.styleN.name = "InfoText"
        self.styleSheet.add(self.styleN)
        self.style = self.styleSheet["BodyText"]

if __name__ == "__main__":
    # doc = LetterMaker("example.pdf", "The MVP", 10)
    # doc.createDocument()
    # doc.savePDF()
    import os

    text = '<p>12/c M G Road kolkata dankuni wb-789456 ddd&nbsp; &nbsp; &nbsp;&nbsp;12/c M G Road kolkata dankuni wb-789456 ddd&nbsp; &nbsp; &nbsp;&nbsp;12/c M G Road kolkata dankuni wb-789456 ddd&nbsp; &nbsp; &nbsp;&nbsp;</p> <p>12/c M G Road kolkata dankuni wb-789456 ddd&nbsp; &nbsp; &nbsp;12/c M G Road kolkata dankuni wb-789456 ddd&nbsp; &nbsp; &nbsp;&nbsp;12/c M G Road kolkata dankuni wb-789456 ddd&nbsp; &nbsp; &nbsp;&nbsp;</p> <p>12/c M G Road kolkata dankuni wb-789456 ddd&nbsp; &nbsp;&nbsp;12/c M G Road kolkata dankuni wb-789456 ddd&nbsp; &nbsp; &nbsp; &nbsp;&nbsp;12/c M G Road kolkata dankuni wb-789456 ddd&nbsp; &nbsp; &nbsp;&nbsp;</p> <p>12/c M G Road kolkata dankuni wb-789456 ddd&nbsp; &nbsp;</p> <p>12/c M G Road kolkata dankuni wb-789456 ddd&nbsp; &nbsp;</p> <p>12/c M G Road kolkata dankuni wb-789456 ddd&nbsp; &nbsp;&nbsp;</p> <p>12/c M G Road kolkata dankuni wb-789456 ddd</p> <p>12/c M G Road kolkata dankuni wb-789456 ddd</p> <p>12/c M G Road kolkata dankuni wb-789456 ddd</p>'
    text = """
&lt;p&gt;14/2 , R B C &lt;span style="color:#FF0000"&gt;Extension&lt;/span&gt; road room no- 45 bangalore-987654 dfff&lt;/p&gt;

&lt;ul&gt;
	&lt;li&gt;14/2 , R B C Extension road room no- 45 bangalore-987654 dfff&lt;/li&gt;
	&lt;li&gt;14/2 , R B C Extension road room no- 45 bangalore-987654 dfff&lt;/li&gt;
&lt;/ul&gt;

&lt;p&gt;&amp;nbsp;&lt;/p&gt;
	"""
    st = getSampleStyleSheet()
    styleN = st["BodyText"]
    styleN.fontSize = 9
    styleN.name = "InfoText"
    st.add(styleN)
    ret = html_to_rl(text, st)
    print(ret)
    pdf_file = '/tmp/tmp.pdf'
    doc = SimpleDocTemplate(pdf_file, pagesize=A4, rightMargin=10, leftMargin=10)
    doc.build(ret)
    os.system('okular {}'.format(pdf_file))
