#!/usr/bin/env python3

from app import app
import psycopg2,requests
import psycopg2.extras
import  os,signal,logging
import configparser,argparse,sys,time
import lockfile
from lockfile.pidlockfile import PIDLockFile
from lockfile import AlreadyLocked
from logging.handlers import TimedRotatingFileHandler
from logging.handlers import RotatingFileHandler

config = ''

from version import VERSION

PIDF = "_runFlask.pid"  #manual server pid
PIDF2 = "runFlask.pid"  # flask is starting this two times


def pidCheck(logger):
	pidfile = PIDLockFile(PIDF, timeout=-1)
	try:
		pidfile.acquire()
		msg = "Lock acquered! Daemon setup script ID %s" %pidfile.read_pid()
		print(msg)
		logger.info(msg)
		#pidfile.break_lock()
	except AlreadyLocked:
		msg = "AlreadyLocked!.. Trying to delete file.."
		print(msg)
		logger.info(msg)
		try:
			os.remove(PIDF)
		except Exception as e: 
			msg = "Exception %s .  Continuing anyway"%e
		
	except Exception as e:   #No process with locked PID
		msg = "General Exception when trying to get lock %s" %e
		print(msg)
		logger.info(msg)
		exit(1)

class GracefulKiller:
	kill_now = False
	def __init__(self,logger):
		signal.signal(signal.SIGINT, self.exit_gracefully)
		signal.signal(signal.SIGTERM, self.exit_gracefully)
		self.logger= logger

	def exit_gracefully(self,signum, frame):
		pidfile = PIDLockFile(PIDF, timeout=-1)
		try:
			pidfile.break_lock()
		except Exception as e:
			msg = "Exception when trying to break lock %s" %e
			print (msg)
			logger.error(msg)
			try:
				os.remove(PIDF)
				exit(1)
			except Exception as e:
				msg = "Exception when trying to break lock and delete pid file %s" %e
				print (msg)
				logger.error(msg)
				exit(2)
		exit(0)

def logVersion(pg_cur):
	worker_info = VERSION
	now_time = time.strftime("%Y-%m-%d %H:%M:%S", time.gmtime())+"+00"
	#program_name_dmon = "dnl_invoice_server"
	program_name_proc = "dnl_invoice_REST_API"
	git_ver = ""# str( git_version()  , 'utf-8')
	
	sql = """   SELECT count(*) as count FROM version_information WHERE program_name = '%s'""" % program_name_proc
	print(sql)
	pg_cur.execute(sql)
	count = pg_cur.fetchone()['count']
	if count == 0:
		sql = """INSERT INTO version_information (program_name ,major_ver,build_date ,start_time ) VALUES (%s,%s,%s,%s)  """
	
	else:
		sql = """UPDATE  version_information  SET  major_ver = '%s' , build_date = '%s',start_time = '%s' WHERE  program_name = '%s'; """ % (VERSION['ver'],VERSION['date'],now_time,program_name_proc) 
	
	print (sql)
	try:
		pg_cur.execute(sql, (program_name_proc , VERSION['ver'],VERSION['date'],now_time)  )
		
	
	except Exception as e:  
		print ("Except in versioning %s"%e )
		raise


def connect_to_postgresql(host, port, database, user, password=None):
	try:
		conn = psycopg2.connect(host=host, port=port, database=database, user=user, password=password)
		conn.autocommit = True
		cursor = conn.cursor(cursor_factory=psycopg2.extras.DictCursor)
		print("connected to DB")
	except Exception as e: 
		print (" Exception in DB connecting:  %s" % (str(e) ) )
		raise 'I am unable to connect to the PostgreSQL. %s'% (str(e))
	return conn, cursor


def postgres():
	global config #pidfile
	config = configparser.ConfigParser()
	config.read('config.ini')
	
	try:
		pg_conn, pg_cur = connect_to_postgresql(config["POSTGRES"]["hostaddr"],int(config["POSTGRES"]["port"]), config["POSTGRES"]["dbname"], config["POSTGRES"]["user"],config["POSTGRES"]["password"]) #config["POSTGRES"]["password"]
		
		return pg_conn, pg_cur
	except Exception as e: 
		print (" Exception in connect_to_postgresql:  %s" % (str(e) ) )
		return -1,-1



#logger = setup()
def main_start(host,port):
	try:
		app.run(threaded=True,debug=False,host= host,port= port) 
	except Exception as e: 
		#logger.error(" Exception in running server:  %s" % (str(e) ) )
		print (" Exception in running server:  %s" % (str(e) ) )
		logger.error(" Exception in running server:  %s" % (str(e) ) )

def worker(host,port):
	#not used in the moment
	print ("Manual Invoice API. ver {ver} date {date}".format_map(VERSION))
	main_start(host,port)

if __name__=='__main__':
	pg_conn,pg_cur = postgres()
	if pg_cur == -1:
		exit (1)
	logVersion(pg_cur)
	
	parser = argparse.ArgumentParser(description='Invoice API')
	parser.add_argument('-H', '--host', default='0.0.0.0' ,  help='host for Invoice API')
	parser.add_argument('-P', '--port', default= 5000 ,  help='port for Invoice API')
	args = parser.parse_args()
	host = args.host
	
	port = int(args.port)
	if port == 5000:
		print("Getting port from config")
		config = configparser.ConfigParser()
		config.read('config.ini')
		port = int(config['other']['port'])
	
	#####logger#####
	
	logger = logging.getLogger('Invoice REST API')
	handler = RotatingFileHandler("logs/rest_api.log",maxBytes=1024*1024*128 ,backupCount= 10 )
	logger.setLevel(logging.DEBUG)

	formatstr = '%(asctime)s: %(levelname)s:  %(message)s'
	formatter = logging.Formatter(formatstr)
	handler.setFormatter(formatter)
	logger.addHandler(handler)
	logger.debug("Starting Rest Api on host:%s port:%s" % (host ,port  )   ) 
	pidCheck(logger)

	killer = GracefulKiller(logger)


	main_start(host,port)


#print("Do not run directly this module")
#worker(1000)
