from flask import render_template, flash, redirect, request, send_from_directory
from flask import jsonify
from app import app
from .forms import LoginForm
from manualinvoice import MakeInvoice, VoidInvoicePost, GetInvoiceDetails, GetInvoiceDetailsAll, reSendEmail, \
    DeleteInvoicePost, apiTest,  markForResend
from AutoInvoice import autoTest, autoFire

import os, traceback
from datetime import datetime

import logging
from logging.handlers import RotatingFileHandler

logger = logging.getLogger('Invoice REST API trace')
handler = RotatingFileHandler("logs/rest_api_trace.log", maxBytes=1024 * 1024 * 128, backupCount=10)
logger.setLevel(logging.DEBUG)
formatstr = '%(asctime)s: %(levelname)s:  %(message)s'
formatter = logging.Formatter(formatstr)
handler.setFormatter(formatter)
logger.addHandler(handler)

# only for testing !
'''
@app.route('/')
@app.route('/index')
def index():
	return render_template('test.html')
'''


def debug(route, data):
    curl = ' '.join(
        ["curl -X POST -H 'Content-Type: multipart/form-data' -F '{}={}'".format(k, v) for k, v in data.items()])
    logger.debug('{} http://localhost:5000/{}'.format(curl, route))


@app.route('/apitest', methods=['GET'])
def manualtest():
    result = apiTest()
    return "running"  # jsonify(result) , 500

@app.route('/auto_invoice_check', methods=['POST'])
def autotest():
    data = request.form
    debug('check', data)
    try:
        result = autoTest(data)
        return result
    except Exception as e:
        return str(e)


@app.route('/auto_invoice_fire', methods=['POST'])
def autofire():
    data = request.form
    debug('fire', data)
    try:
        result = autoFire(data)
        return result
    except Exception as e:
        return str(e)

@app.route('/ManualInvoiceTest', methods=['POST'])
def manualinvoice():
    # form = LoginForm()
    filename = 'n/a'
    invoice_number = 'n/a'
    try:
        try:
            data = request.form
            debug('ManualInvoiceTest', data)
            print('Manual invoice Request in %s  with Request: %s' % (
            datetime.now().strftime('%Y-%m-%d %H:%M:%S'), str(data)))
            print(data, type(data))
            data_dict = dict(data)
            data_list = []
            client_id = data_dict.get('client_id', None)
            logger.debug("client_id {}".format(client_id))
            if type(client_id)==type([]):
                client_id_list = client_id[0].split(",")  # .strip()
            else:
                client_id_list = client_id.split(",")  # .strip()
            logger.debug("client_id_list {}".format(client_id_list))
        except Exception as e:
            html = """ Server error.Wrong Call to API %s""" % str(e)
            print(html)
            logger.debug(html)
            return html
        try:
            for client_id in client_id_list:
                print("client_id", client_id)
                data_dict = dict(data)
                data_dict['client_id'] = [str(client_id)]
                filename, invoice_number = MakeInvoice(data_dict)
        except Exception as e:
            html = """ Server error in generating new Invoice: %s """ % str(e)
            logger.debug(html)
            return html

        if filename == -1:
            logger.debug('-1 ' + invoice_number)
            return invoice_number
        if filename == -3:
            logger.debug('-3 ' + invoice_number)
            return invoice_number

        if invoice_number == -10:
            logger.debug("Client has set invoice_zero to True and current billing is zero. Invoice is not generated")
            return "Client has set invoice_zero to True and current billing is zero. Invoice is not generated"
        if invoice_number == -11:
            logger.debug("Server returned error: %s" % filename)
            return "Server returned error: %s" % filename
        # print (filename)
        try:
            # link_pdf = "/invoice/"+filename[1]
            # link_log = "./invoice/"+filename[1]+'.txt'
            # html = """ <a href = %s> pdf </a>    <a href = %s> log </a>  """ % (link_pdf,link_log)
            html = "OK"
            logger.debug(html)
            return html
        except Exception as e:
            html = """ Server error in generating new Invoice: %s """ % str(e)
            logger.debug(html)
            return html
    except Exception as e:
        html = """ Server error %s """ % str(e)
        print(html)
        logger.debug(html)
        return html


@app.route('/regenerate', methods=['POST'])
def regenerateInvoice():
    # form = LoginForm()
    data = request.form
    debug('regenerate', data)
    print('Regenerate invoice Request in %s  with Request: %s' % (
    datetime.now().strftime('%Y-%m-%d %H:%M:%S'), str(data)))
    # print ("27-01-2017 14:31")
    try:
        params = GetInvoiceDetails(data)
        print("PARAMS in View: ", str(params))
    except Exception as e:
        print(str(e))
        traceback.print_tb(e.__traceback__)
        return "Error!"
    if params == -2:
        data = dict(data)
        invoice_number = data['invoice_number'][0]
        return "Wrong invoice number:  %s" % invoice_number
    result = DeleteInvoicePost(data)

    if params.get('create_type', 0) == 1:  # autoinvoice or manial invoice regen?
        auto = False
    else:
        auto = True
    filename, invoice_number = MakeInvoice(params, True, True, auto)  # regen , overlapping ,create_type
    if filename == -1:
        return "ERROR ! %s" % invoice_number
    print(filename)
    link_pdf = "/invoice/" + filename
    # link_log = "./invoice/"+filename+'.txt'
    html = """ <a href = %s> pdf </a>   """ % (link_pdf)  # ,link_log

    return html


@app.route('/resendemail', methods=['POST'])
# @app.route('/resendemail/', methods=['POST'])
def resendemail():
    # form = LoginForm()
    data = request.form
    debug('resendemail', data)
    print("Resend option", data)
    try:
        # result = markForResend(data)
        params, pg_cur = GetInvoiceDetailsAll(data)
        if params == None:
            print("Non existend invoice number or the invoice is void!")
            return "Non existend invoice number or the invoice is void!"
        print("PARAMS in View: ", params.get('client_id'), params.get('invoice_number'), str(params.copy()))
    except Exception as e:
        print(str(e))
        traceback.print_tb(e.__traceback__)
        return "Error!"
    if params == -1:
        return "Invalid Invoice Number"
    # print ( str(params) )
    html = """Client Id: %s , Invoice_path: %s """ % (params['client_id'], params['pdf_path'])
    try:
        result = reSendEmail(params, data, pg_cur)
    except Exception as e:
        print(e)
        print(traceback.format_exc())
    return "OK"


@app.route('/voidinvoicepost', methods=['POST'])
def voidinvoicepost():
    data = request.form
    debug('voidinvoicepost', data)
    print("VOID DATA", str(data))
    result = VoidInvoicePost(data)

    return (result)


@app.route('/invoice/<path:path>')
def send_file(path):
    print("path ", path)
    logger.debug('/invoice/{}'.format(path))
    # root_dir = os.path.dirname(os.getcwd())
    # print(root_dir)
    # return send_from_directory('./invoice', path)
    # return send_from_directory(app.config['pdf_path'], path)
    import configparser
    config = configparser.ConfigParser()
    config.read('config.ini')
    dr = config['other']['pdf_path']
    print(dr)
    return send_from_directory(dr, path)
