#!/usr/bin/env python3

from app import app
import logging
import psycopg2
import psycopg2.extras
from logging.handlers import TimedRotatingFileHandler
from logging.handlers import RotatingFileHandler
import subprocess, signal , time
from lockfile.pidlockfile import PIDLockFile
from lockfile import AlreadyLocked
import configparser,os, sys, psutil
import multiprocessing, AutoInvoice, argparse ,RunFlask
from time import gmtime, strftime
pidfile = ''
PORT = 55000
#logger = ""
#auto_process = ""
pg_cur = ""
config = ""

DID_RELATIVE_PATH = "/did-billing/server.py"

from version import VERSION

def setup():
	# Get config.ini
	global pidfile,PORT ,config,pg_cur
	config = configparser.ConfigParser()
	config.read('./config.ini')
	# set logger
	logger = logging.getLogger('Dmon Inovice Runner')
	if int(config["log"].get('by_time',0) )== 1:
		handler = TimedRotatingFileHandler(config["log"].get ("invoice_dmon_path","dmon.log"),when=config["log"].get ("invoice_dmon_when",'m'),
		interval= int( config["log"].get ("invoice_dmon_interval",0) ),backupCount= int( config["log"].get ("invoice_dmon_backupCount",5)  ) )
	else: # by size
		handler = RotatingFileHandler(config["log"].get ("invoice_dmon_path","invoice_dmon.log"),maxBytes=int( config["log"].get ("invoice_dmon_maxBytes", 1024*1024*128 ) ),
		backupCount= int( config["log"].get ("invoice_dmon_backupCount",5)  ) )
	
	if int (config["log"].get ("debug_level",0) ) == 1: 
		logger.setLevel(logging.DEBUG)
	else:
		logger.setLevel(logging.INFO)
	formatstr = '%(asctime)s - %(name)s - %(levelname)s - %(message)s'
	formatstr = '%(asctime)s: %(levelname)s:  %(message)s'
	formatter = logging.Formatter(formatstr)
	handler.setFormatter(formatter)
	logger.addHandler(handler)
	try:
		#print ("Trying to get port from config!")
		PORT = int (config["other"]['port'] )
		
	except Exception as e: 
		print ("Error in getting port %s"%str(e))
		logger.error("Error in getting port %s"%str(e))
	# setting PID
	pidf = "invoice_manual.pid"
	pidfile = PIDLockFile(pidf, timeout=-1)
	try:
		pidfile.acquire()
		print("Setup Daemon Reporting: Lock acquered!")
		logger.info("Setup Daemon Reporting: Lock acquered!")
		print("Setup Daemon Reporting: Daemon setup script ID %s" %pidfile.read_pid())
		logger.info("Setup Daemon Reporting: Daemon setup script ID %s" %pidfile.read_pid())
		pidfile.break_lock()
		print("Setup Daemon Reporting: Lock released for main program execution! ")
		logger.info("Setup Daemon Reporting: Lock released for main program execution! ")
	
	except Exception as e: 
		print(e)
		logger.error(e)
		logger.info("Ignoring pid file error")
	
	try:
		pg_conn, pg_cur = connect_to_postgresql(config["POSTGRES"]["hostaddr"],int(config["POSTGRES"]["port"]), config["POSTGRES"]["dbname"], config["POSTGRES"]["user"],config["POSTGRES"]["password"]) #config["POSTGRES"]["password"]
	except Exception as e: 
		print (" Exception in connect_to_postgresql:  %s" % (str(e) ) )
		logger.error(" Exception in connect_to_postgresql:  %s" % (str(e) ) )
		return -1
		
		
	return logger
	

class GracefulKiller():
	kill_now = False
	def __init__(self,event,auto_process,man_proc,logger):
		print ("Init of Grace Killer") 
		logger.debug ("Init of Grace Killer") 
		self.logger = logger
		signal.signal(signal.SIGINT, self.exit_gracefully)
		signal.signal(signal.SIGTERM, self.exit_gracefully)
		self.event = event
		self.auto_process = auto_process
		self.man_proc = man_proc
		#self.did_proc= did_proc
	def exit_gracefully(self,signum, frame):
		print ("GracefulKiller is performing cleaning up of the three subprocesses") 
		self.logger.debug ("GracefulKiller is performing cleaning up of the two subprocesses") 
		self.man_proc.join (5) 
		self.event.set()
		self.auto_process.join (15)
		exit(0)

def startAuto():
	event = multiprocessing.Event()
	auto_process = multiprocessing.Process(target=AutoInvoice.worker,args =(event,) )
	auto_process.daemon = True
	auto_process.start()
	
	api_requests = multiprocessing.Process(target=AutoInvoice.api_tasker, args=(event,) )
	api_requests.daemon = True
	api_requests.start()
	
	return (auto_process,event,api_requests)

def startMan(host,port):
	#event = multiprocessing.Event()
	man_process = multiprocessing.Process(target=RunFlask.worker,args =(host,port,) )
	man_process.daemon = True
	man_process.start()
	return (man_process)


def startDid(port= 18881):
	#event = multiprocessing.Event()
	#man_process = multiprocessing.Process(target=runFlask.worker,args =(port,) )
	#man_process.daemon = True
	#man_process.start()
	dir_path = os.path.dirname(os.path.realpath(__file__))
	run_file =dir_path+DID_RELATIVE_PATH
	port = "-l %s"%port
	port = port.strip().replace("\n","")
	#port = "--port %s"%port
	#port = port.strip().replace("\n","")
	args = ["python3",run_file , port  ]
	
	print (args)
	proc = subprocess.Popen(args, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
	return proc


def run (daemon = True):
	
	logger.info("Starting daemon ...")
	print("Starting daemon ...")
	file_name = os.path.basename(__file__)
	now_time = strftime("%Y-%m-%d %H:%M:%S", gmtime())
	cur_path = os.path.dirname(os.path.abspath(__file__))
	if daemon:
		cmd= "nohup python3 %s --action nur --date '%s' --path %s </dev/null >/dev/null 2>&1 &" % (file_name,now_time,cur_path  )
	else:
		cmd= "python3 %s --action nur --date '%s' --path %s" % (file_name,now_time,cur_path  )

	proc = subprocess.Popen([cmd], shell = True,stdin=None, stdout=None,stderr=None, close_fds=True)
	logger.info("Invoice Daemon is started")
	
def stop():
	pidfile_manual = PIDLockFile("invoice_manual.pid", timeout=-1)
	try:
		os.kill(pidfile_manual.read_pid(), signal.SIGTERM)
		time.sleep(1)
		os.remove("./invoice_manual.pid")
		exit(0)
	except OSError:  #No process with locked PID
		print( "OS ERROR!" )
		print ("No process with locked PID. Probably you must start the daemon first")
		exit(1)
		
	except Exception as e:  
		print ( "General error! %s" % str(e) )
		exit(1)

def restart_program():
    """Restarts the current program, with file objects and descriptors
       cleanup
    """

    try:
        p = psutil.Process(os.getpid())
        for handler in p.get_open_files() + p.connections():
            os.close(handler.fd)
    except Exception as e: 
        logger.error(e)

def MakeNewPid():
	global pidfile
	logger.debug ("Making new pid file")
	# setting PID
	pidf = "invoice_manual.pid"
	pidfile = PIDLockFile(pidf, timeout=-1)
	try:
		pidfile.acquire()
		print("Lock aquered!")
		logger.debug ("success")
	except:
		print("Lock is blocked. Waiting 3 secs")
		time.sleep(3)
		
		try:
			pidfile.acquire()
			print("Lock aquered!")
			logger.debug ("success from second try")
		except AlreadyLocked:
			try:
				os.kill(pidfile.read_pid(), 0)
				print ('Process already running!!')
				logger.error('Process already running!!')
				#logger.error('update from 2017.11.')
				#print ('Trying to kill it !')
				#exit(1)
			except Exception as e:   #No process with locked PID
				#pidfile.break_lock()
				print ("General error %s" % str(e))
				logger.error("General error %s" % str(e))
				
	return pidfile


def connect_to_postgresql(host, port, database, user, password=None):
	conn = psycopg2.connect(host=host, port=port, database=database, user=user, password=password)
	conn.autocommit = True
	cursor = conn.cursor(cursor_factory=psycopg2.extras.DictCursor)
	print("connected to DB" )
		
	return conn, cursor


def git_version():
    from subprocess import Popen, PIPE
    gitproc = Popen(['git', 'rev-parse','HEAD'], stdout = PIPE)
    (stdout, _) = gitproc.communicate()
    return stdout.strip()


def logVersion(logger):
	global pg_cur
	worker_info = AutoInvoice.VERSION  #VERSION = ['ver':"1.0.1",'date':'2017-05-25']
	now_time = time.strftime("%Y-%m-%d %H:%M:%S", time.gmtime())+"+00"
	program_name_dmon = "dnl_invoice_server"
	program_name_proc = "dnl_invoice_AutoInvoice"
	git_ver = ""# str( git_version()  , 'utf-8')
	
	sql = """   SELECT count(*) as count FROM version_information WHERE program_name = '%s'""" % program_name_dmon
	print(sql)
	pg_cur.execute(sql)
	count = pg_cur.fetchone()['count']
	if count == 0:
		sql = """INSERT INTO version_information (program_name ,major_ver,build_date ,start_time ) VALUES (%s,%s,%s,%s)  """
	
	else:
		sql = """UPDATE  version_information  SET  major_ver = '%s' , build_date = '%s',start_time = '%s' WHERE  program_name = '%s'; """ % (VERSION['ver'],VERSION['date'],now_time,program_name_dmon) 
	
	print (sql)
	try:
		pg_cur.execute(sql, (program_name_dmon, VERSION['ver'],VERSION['date'],now_time)  )
		
	
	except Exception as e:  
		print ("Except in versioning %s"%e )
		logger.error("Except in versioning %s"%e )


if __name__=='__main__':
	global logger,auto_process,event 
	if os.geteuid() == 0:
		exit("You do not need to have root privileges to run this script.\nPlease try again, this time without using sudo. Exiting.")
	

	parser = argparse.ArgumentParser(description='Invoice API')
	parser.add_argument('-a', '--action',   help='Action to be taken. start , stop')  #default='start'
	parser.add_argument('-H', '--host', default='0.0.0.0' ,  help='host for Invoice API')
	parser.add_argument('-P', '--port', default= None ,  help='port for Invoice API')
	now_time = strftime("%Y-%m-%d %H:%M:%S", gmtime())
	parser.add_argument('-d', '--date',  default=now_time , help='The time of script execution')
	cur_path = os.path.dirname(os.path.abspath(__file__))
	parser.add_argument('-p', '--path',  default=str(cur_path ) , help='the path of script ')
	args = parser.parse_args()
	host = args.host
	port = args.port

	#print (str(args))
	print ("DAEMON SERVER is invoked with option %s" % args.action )
	time.sleep(1)
	if args.action == "start":
		logger = setup()
		run ()
		exit(0)
	elif args.action == "stop":
		stop()
		exit(0)
		
	elif args.action == "restart":
		stop()
		logger = setup()
		restart_program()
		python = sys.executable
		os.execl(python, python, *sys.argv)

	elif args.action == "debug":
		logger = setup()
		print("starting in foreground")
		time.sleep(1)
		#run (False)
		#exit(0)

	elif args.action == "nur":
		logger = setup()
		#MakeNewPid()
		#print("starting daemon")
		
	else:
		print ("Unrecognized command")
		exit (0)
	
	if port != None:
		PORT = port
	man_proc = startMan(host,PORT)
	print("Starting server... on port %s"% PORT )
	time.sleep(2)
	auto_process,event,api_requests = startAuto()
	#did_proc = startDid()
	logVersion(logger)
	try:
		killer = GracefulKiller(event,auto_process,man_proc,logger)
		logger.info("Starting server... on port %s"% PORT )
		
		
		#app.run(debug=True,host= '0.0.0.0',port = PORT  ) 
		fail_save_counter = 0 
		while True:
			
			if auto_process.is_alive() == False:
				#send_fail_report("Autoprocess died")
				logger.error(" Autoprocess is dead"  )
				#print (" Autoprocess died: Restarting"  )
				#auto_process,event = startAuto()
			if man_proc.is_alive() == False:
				#send_fail_report("Rest Api Died")
				logger.error(" REST API is dead"  )
				#print (" REST API died: Restarting"  )
				#man_proc = startMan(PORT)
			time.sleep(59*10)

	except Exception as e: 
		logger.error(" Exception in running server:  %s" % (str(e) ) )
		print (" Exception in running server:  %s" % (str(e) ) )
		exit(1)



