import time

import requests

# from manualinvoice import logger, config


v5_to_rtapi_map = {
    'call_duration': 31,
    'origination_destination_host_name': 12,
    'dynamic_route': 82,
    '(case release_tod when 0 then null else to_timestamp(release_tod/1000000) end)': 4,
    'final_route_indication': 29,
    'trunk_id_origination': 37,
    'ingress_bill_minutes': 57,
    'ingress_client_bill_result': 44,
    'ingress_client_bill_time': 43,
    'ingress_client_cost': 45,
    'ingress_client_currency': 42,
    'ingress_client_id': 38,
    'ingress_client_rate': 41,
    'ingress_client_rate_table_id': 39,
    'ingress_dnis_type': 58,
    'ingress_id': 37,
    'ingress_rate_id': 65,
    'ingress_rate_type': 59,
    'lrn_dnis': 60,
    'lrn_number_vendor': 46,
    'lnp_dipping_cost': 49,
    'origination_codec_list': 18,
    'origination_destination_number': 11,
    'origination_source_host_name': 10,
    'origination_source_number': 9,
    'first_release_dialogue': 8,
    'orig_call_duration': 87,
    'orig_code': 67,
    'orig_code_name': 68,
    'orig_country': 69,
    'orig_delay_second': 85,
    'origination_remote_payload_ip_address': 14,
    'origination_remote_payload_udp_address': 15,
    'origination_call_id': 13,
    'pdd': 32,
    'release_cause': 1,
    'rerate_time': 4,
    'release_cause_from_protocol_stack': 6,
    'binary_value_of_release_cause_from_protocol_stack': 7,
    'ring_time': 33,
    'route_plan': 83,
    '(case start_time_of_date when 0 then null else to_timestamp(start_time_of_date/1000000) end)': 2,
    'static_route': 81,
    'time': 0,
    'translation_ani': 64,
    'routing_digits': 30,
    'q850_cause': 106,
    'q850_cause_string': 107,
    'conf_id': 35,
    'egress_client_id': 51,
    'egress_cost': 49,
    'egress_dnis_type': 61,
    'egress_id': 46,
    'termination_source_number': 19,
    '(case answer_time_of_date when 0 then null else to_timestamp(answer_time_of_date/1000000) end)': 3,
}

FIELDS = {0: 'time', 1: 'release_cause', 2: 'start_time_of_date', 3: 'answer_time_of_date', 4: 'release_tod',
          5: 'minutes_west_of_greenwich_mean_time', 6: 'release_cause_from_protocol_stack',
          7: 'binary_value_of_release_cause_from_protocol_stack', 8: 'first_release_dialogue',
          9: 'origination_source_number', 10: 'origination_source_host_name', 11: 'origination_destination_number',
          12: 'origination_destination_host_name', 13: 'origination_call_id',
          14: 'origination_remote_payload_ip_address', 15: 'origination_remote_payload_udp_address',
          16: 'origination_local_payload_ip_address', 17: 'origination_local_payload_udp_address',
          18: 'origination_codec_list', 19: 'termination_source_number', 20: 'termination_source_host_name',
          21: 'termination_destination_number', 22: 'termination_destination_host_name', 23: 'termination_call_id',
          24: 'termination_remote_payload_ip_address', 25: 'termination_remote_payload_udp_address',
          26: 'termination_local_payload_ip_address', 27: 'termination_local_payload_udp_address',
          28: 'termination_codec_list', 29: 'final_route_indication', 30: 'routing_digits', 31: 'call_duration',
          32: 'pdd', 33: 'ring_time', 34: 'callduration_in_ms', 35: 'conf_id', 36: 'call_type', 37: 'ingress_id',
          38: 'ingress_client_id', 39: 'ingress_client_rate_table_id', 40: 'ingress_client_currency_id',
          41: 'ingress_client_rate', 42: 'ingress_client_currency', 43: 'ingress_client_bill_time',
          44: 'ingress_client_bill_result', 45: 'ingress_client_cost', 46: 'egress_id', 47: 'egress_rate_table_id',
          48: 'egress_rate', 49: 'egress_cost', 50: 'egress_bill_time', 51: 'egress_client_id',
          52: 'egress_client_currency_id', 53: 'egress_client_currency',
          # 54:'egress_six_seconds',
          55: 'egress_bill_minutes', 56: 'egress_bill_result', 57: 'ingress_bill_minutes', 58: 'ingress_dnis_type',
          59: 'ingress_rate_type', 60: 'lrn_dnis', 61: 'egress_dnis_type', 62: 'egress_rate_type', 63: 'item_id',
          64: 'translation_ani', 65: 'ingress_rate_id', 66: 'egress_rate_id', 67: 'orig_code', 68: 'orig_code_name',
          69: 'orig_country', 70: 'term_code', 71: 'term_code_name', 72: 'term_country',
          73: 'ingress_rate_effective_date', 74: 'egress_rate_effective_date', 75: 'egress_erro_string', 76: 'order_id',
          77: 'order_type', 78: 'is_final_call', 79: 'egress_code_asr', 80: 'egress_code_acd', 81: 'static_route',
          82: 'dynamic_route', 83: 'route_plan', 84: 'route_prefix', 85: 'orig_delay_second', 86: 'term_delay_second',
          87: 'orig_call_duration', 88: 'trunk_type', 89: 'origination_profile_port', 90: 'termination_profile_port',
          91: 'o_trunk_type2', 92: 'o_billing_method', 93: 't_trunk_type2', 94: 't_billing_method', 95: 'campaign_id',
          96: 'agent_id', 97: 'agent_rate', 98: 'agent_cost', 99: 'orig_jur_type', 100: 'term_jur_type',
          101: 'ring_epoch', 102: 'end_epoch', 103: 'paid_user', 104: 'rpid_user', 105: 'timeout_type',
          106: 'q850_cause'
    , 107: 'q850_cause_string'
          }
FIELD_MAP = dict((v, k) for k, v in FIELDS.items())
UIFIELDS = {0: 'Time', 1: 'Release Cause', 2: 'Start Time', 3: 'Answer Time', 4: 'End Time', 5: 'Timezone',
            6: 'Response to Egress', 7: 'Response from Ingress', 8: 'Orig/Term Release', 9: 'Orig Src Number',
            10: 'Orig IP', 11: 'Orig DST Number', 12: 'orig_switch_ip', 13: 'Orig Call ID', 14: 'Orig Media IP',
            15: 'Orig Media Port', 16: '', 17: 'Origination Local Payload Port', 18: 'Orig Codecs',
            19: 'Term Src Number', 20: 'Class4 IP (Egress)', 21: 'Term DST Number', 22: 'Term IP', 23: 'Term Call ID',
            24: 'Term Media IP', 25: 'Term Media Port', 26: 'Term Media IP', 27: 'Term Media Port', 28: 'Term Codecs',
            29: 'final_route', 30: 'Translational DNIS', 31: 'Call Duration', 32: 'PDD', 33: 'Ring Time',
            34: 'Call Duration in MS', 35: '', 36: 'Call Type', 37: 'Ingress ID', 38: 'Ingress Client ID',
            39: 'Ingress Rate Table ID', 40: 'Ingress Client Currency ID', 41: 'Ingress Client Rate',
            42: 'Ingress Client Currency', 43: 'Ingress Bill Time', 44: 'Ingress Bill Result',
            45: 'Ingress Client Cost', 46: 'Egress ID', 47: 'Egress Rate table ID', 48: 'Egress Rate',
            49: 'Egress Cost', 50: 'Egress Bill Time', 51: 'Egress Carrier ID', 52: 'Egress Currency ID',
            53: 'Egress Currency',
            # 54:'Egress Six Second',
            55: 'Egress Bill Minutes', 56: 'Egress Bill Result', 57: 'Egress Bill Minutes', 58: 'Ingress DNIS type',
            59: 'Egress Rate type', 60: 'LRN Number', 61: 'Egress DNIS Type', 62: 'Egress Rate Type', 63: '',
            64: 'Translational ANI', 65: 'Ingress Rate ID', 66: 'Egress Rate ID', 67: 'Orig Code', 68: 'Orig Code Name',
            69: 'Orig Country', 70: 'Term Code', 71: 'Term Code Name', 72: 'Term Country',
            73: 'Ingress Rate Effective Date', 74: 'Egress Rate Effective Date', 75: 'Egress Trunk Trace', 76: '',
            77: '', 78: '', 79: 'Egress Code ASR', 80: 'Egress Code ACD', 81: 'Start Route Name',
            82: 'Dynamic Route Name', 83: 'Routing Plan', 84: 'Routing Prefix', 85: '', 86: '',
            87: 'Orig Call Duration', 88: 'Trunk Type', 89: 'Orig Profile Port', 90: 'Term Profile Port', 91: '',
            92: '', 93: '', 94: 'T Billing method', 95: 'Campaign ID', 96: 'Agent ID', 97: 'Agent Rate',
            98: 'Agent Cost', 99: 'Orig Jurisdiction type', 100: 'Term Jurisdiction Type', 101: 'Ring Start time',
            102: 'End Time', 103: 'PAID user', 104: 'Rpid user', 105: 'Timeout type', 106: 'Q850 Media Q850'
    , 107: 'Q850 Media CAUSE'
            }


def GetCdrFields(pg_cur):
    from manualinvoice import logger, config
    try:
        sql = '''SELECT invoices_cdr_fields as fields  FROM system_parameter LIMIT 1;'''  # gets cdr link fields
        print("Getting send_cdr_fields from System parameter: %s" % sql)
        pg_cur.execute(sql)
        fields = pg_cur.fetchone()

        print("Returned result from query for getting send cdr fields:", fields)
        logger.debug("Returned result from query for getting send cdr fields: %s" % str(fields))
        # change comma delimiter to "&nbsp;"
        # fields =fields['fields'].replace( ',',',&nbsp;' )
        if fields and fields['fields']:
            all_f = fields['fields'].split(',')
        else:
            all_f = ['time', 'binary_value_of_release_cause_from_protocol_stack', 'origination_source_number',
                     'origination_source_host_name', 'origination_destination_number',
                     'origination_destination_host_name',
                     'origination_call_id', 'call_duration', 'ingress_id', 'ingress_client_rate',
                     'ingress_client_bill_time',
                     'ingress_client_cost', 'ingress_rate_type', 'lrn_dnis', 'orig_code', 'orig_code_name',
                     'orig_country']

        return all_f
    except Exception as e:
        print(" Exception in getting send_cdr_fields from System parameter:  %s" % (str(e)))
        logger.error("Exception in getting send_cdr_fields from System parameter:  %s" % (str(e)))
        return None

def GetCdrLabels(pg_cur,fields):
    try:
        all_l = []
        for f in fields:
            sql = "select label from daily_cdr_fields where field ='{}'".format(f)
            pg_cur.execute(sql)
            label = pg_cur.fetchone()
            if label:
                all_l.append(label['label'])
            else:
                all_l.append(f)
        return all_l
    except Exception as e:
        print(" Exception in getting daily_cdr_fields:  %s" % (str(e)))
        logger.error(" Exception in getting daily_cdr_fields:  %s" % (str(e)))
        return None

def getCdrLink(client, pg_cur, invoice_id, data, fields=None):
    from manualinvoice import logger, config
    sql = "select cdr_link,cdr_token from invoice where invoice_id = %s" % invoice_id
    pg_cur.execute(sql)
    result = pg_cur.fetchone()
    cdr_link = result['cdr_link']
    token = result['cdr_token']
    if not fields:
        fields = GetCdrFields(pg_cur)
    labels = GetCdrLabels(pg_cur,fields)
    sql = """select resource_id from resource where client_id = %s and ingress is true """ % client['client_id']
    print(sql)

    pg_cur.execute(sql)
    resource_ids = pg_cur.fetchall()
    if resource_ids == None:
        msg = "Client do not have resource ids"
        print(msg)
        logger.error(msg)
        return [msg]

    resources_list = []
    for resource in resource_ids:
        resources_list.append(resource['resource_id'])
    resources_list_str = ','.join(str(e) for e in resources_list)
    print(type(data['start_date']), data['end_date'])
    start = str(data['start_date'])
    end = str(data['end_date'])
    print(start, end)
    # filter = "time between {} and {} and ingress_client_id = {}".format(start,end,client['client_id'])
    filter = "and ingress_id in ({})".format(resources_list_str)
    operator_user = 'dnl_invoice_generator'
    sql = """insert into cdr_export_task(cdr_start_time,cdr_end_time,csv_file_headers,cdr_headers,cdr_filter,
	operator_user) values('{}','{}','{}','{}','{}','{}');
	select currval('cdr_export_task_id_seq') as id;
	""".format(start, end, ','.join(labels), ','.join(fields), filter, operator_user)
    print(sql)
    ret = pg_cur.execute(sql)
    ret = pg_cur.fetchone()
    print(ret['id'])
    cdr_export_task_id = ret['id']
    switch = config['cdr']["switch"]

    save_counter = 0
    cdr_link = '{}cdr_download/{}'.format(switch,cdr_export_task_id)

    if cdr_link != None:
        sql = """UPDATE invoice set cdr_link = '%s'  where invoice_id = %s """ % (
            cdr_link,  invoice_id)
        print(sql)
        pg_cur.execute(sql)
    else:
        print("cdr link is None")

    return 0


if __name__ == '__main__':
    import configparser, argparse
    from AutoInvoice import connect_to_postgresql, logger, checkClients,INVOICE_SQL,INVOICE_COUNT_SQL,makeClientDict

    global config  # pidfile
    config = configparser.ConfigParser()
    config.read('config.ini')
    try:
        pg_conn, pg_cur = connect_to_postgresql(config["POSTGRES"]["hostaddr"], int(config["POSTGRES"]["port"]),
                                                config["POSTGRES"]["dbname"], config["POSTGRES"]["user"],
                                                config["POSTGRES"]["password"])  # config["POSTGRES"]["password"]
    except Exception as e:
        print(" Exception in connect_to_postgresql:  %s" % (str(e)))
        logger.error(" Exception in connect_to_postgresql:  %s" % (str(e)))
    print('ok')
    # #test cd_link
    # client = {'client_id': '58'}
    # invoice_id = 55
    # data = {'start_date': '2020-07-01 00:00:00', 'end_date': '2020-07-31 23:59:59'}
    # getCdrLink(client, pg_cur, invoice_id, data, fields=None)

    # test checkClients
    test_client_id = 62
    pg_cur.execute(INVOICE_SQL+' and client.client_id= {}'.format(test_client_id))
    result = pg_cur.fetchall()
    for row in result:
        print(row.get("client_id", "N/A"), row.get("days", "N/A"))
    pg_cur.execute("""select resource.client_id,count(resource.ingress is true) from resource join 
client on resource.client_id=client.client_id where resource.client_id= {} group by resource.client_id having 
count(resource.ingress is true) > 0;""".format(test_client_id) )
    result_count = pg_cur.fetchall()
    # print ("result:", len (result) )
    # print ("result_count:", result_count)
    try:
        client_dict = makeClientDict(result_count)  # '14483': {'ingress_count': 2, 'client_id': 14483}
    except Exception as e:
        print("Error in makeClientDict %s" % e)
        logger.error("Error in makeClientDict %s" % e)
    print("client_dict:" ,client_dict )
    total_invoices = 0

    #total_invoices, invoices_needed = checkClients(result, client_dict, total_invoices, pg_cur)
