#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#  api_test.py
#  
#  Copyright 2017 krasytod <krasytod@krasytod-OptiPlex-755>
#  
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#  
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#  
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#  MA 02110-1301, USA.
#  
#  
import csv
from collections import defaultdict
import requests,datetime, time




CDR_FIELDS = ("time","release_cause","start_time_of_date","answer_time_of_date","release_tod","minutes_west_of_greenwich_mean_time","release_cause_from_protocol_stack","binary_value_of_release_cause_from_protocol_stack","first_release_dialogue","origination_source_number","origination_source_host_name","origination_destination_number","origination_destination_host_name","origination_call_id","origination_remote_payload_ip_address","origination_remote_payload_udp_address","origination_local_payload_ip_address","origination_local_payload_udp_address","origination_codec_list","termination_source_number","termination_source_host_name","termination_destination_number","termination_destination_host_name","termination_call_id","termination_remote_payload_ip_address","termination_remote_payload_udp_address","termination_local_payload_ip_address","termination_local_payload_udp_address","termination_codec_list","final_route_indication","routing_digits","call_duration","pdd","ring_time","callduration_in_ms","conf_id","call_type","ingress_id","ingress_client_id","ingress_client_rate_table_id","ingress_client_currency_id","ingress_client_rate","ingress_client_currency","ingress_client_bill_time","ingress_client_bill_result","ingress_client_cost","egress_id","egress_rate_table_id","egress_rate","egress_cost","egress_bill_time","egress_client_id","egress_client_currency_id","egress_client_currency","egress_six_seconds","egress_bill_minutes","egress_bill_result","ingress_bill_minutes","ingress_dnis_type","ingress_rate_type","lrn_dnis","egress_dnis_type","egress_rate_type","item_id","translation_ani","ingress_rate_id","egress_rate_id","orig_code","orig_code_name","orig_country","term_code","term_code_name","term_country","ingress_rate_effective_date","egress_rate_effective_date","egress_erro_string","order_id","order_type","is_final_call","egress_code_asr","egress_code_acd","static_route","dynamic_route","route_plan","route_prefix","orig_delay_second","term_delay_second","orig_call_duration","trunk_type","origination_profile_port","termination_profile_port","o_trunk_type2","o_billing_method","t_trunk_type2","t_billing_method","campaign_id","Nothing","agent_id","agent_rate","agent_cost","orig_jur_type","term_jur_type","ring_epoch","end_epoch","paid_user","rpid_user","timeout_type","q850_cause","q850_cause_string")
#CDR_FIELDS = ('connection_type', 'session_id', 'release_cause', 'start_time_of_date', 'answer_time_of_date', 'release_tod', 'minutes_west_of_greenwich_mean_time', 'release_cause_from_protocol_stack', 'binary_value_of_release_cause_from_protocol_stack', 'first_release_dialogue', 'trunk_id_origination', 'voip_protocol_origination', 'origination_source_number', 'origination_source_host_name', 'origination_destination_number', 'origination_destination_host_name', 'origination_call_id', 'origination_remote_payload_ip_address', 'origination_remote_payload_udp_address', 'origination_local_payload_ip_address', 'origination_local_payload_udp_address', 'origination_codec_list', 'origination_ingress_packets', 'origination_egress_packets', 'origination_ingress_octets', 'origination_egress_octets', 'origination_ingress_packet_loss', 'origination_ingress_delay', 'origination_ingress_packet_jitter', 'trunk_id_termination', 'voip_protocol_termination', 'termination_source_number', 'termination_source_host_name', 'termination_destination_number', 'termination_destination_host_name', 'termination_call_id', 'termination_remote_payload_ip_address', 'termination_remote_payload_udp_address', 'termination_local_payload_ip_address', 'termination_local_payload_udp_address', 'termination_codec_list', 'termination_ingress_packets', 'termination_egress_packets', 'termination_ingress_octets', 'termination_egress_octets', 'termination_ingress_packet_loss', 'termination_ingress_delay', 'termination_ingress_packet_jitter', 'final_route_indication', 'routing_digits', 'call_duration', 'pdd', 'ring_time', 'callduration_in_ms', 'conf_id', 'call_type', 'ingress_id', 'ingress_client_id', 'ingress_client_rate_table_id', 'ingress_client_currency_id', 'ingress_client_rate', 'ingress_client_currency', 'ingress_client_bill_time', 'ingress_client_bill_result', 'ingress_client_cost', 'egress_id', 'egress_rate_table_id', 'egress_rate', 'egress_cost', 'egress_bill_time', 'egress_client_id', 'egress_client_currency_id', 'egress_client_currency', 'egress_six_seconds', 'egress_bill_minutes', 'egress_bill_result', 'ingress_bill_minutes', 'ingress_dnis_type', 'ingress_rate_type', 'lrn_number_vendor', 'lrn_dnis', 'egress_dnis_type', 'egress_rate_type', 'item_id', 'translation_ani', 'ingress_rate_id', 'egress_rate_id', 'orig_code', 'orig_code_name', 'orig_country', 'term_code', 'term_code_name', 'term_country', 'ingress_rate_effective_date', 'egress_rate_effective_date', 'egress_erro_string', 'order_id', 'order_type', 'lnp_dipping_cost', 'is_final_call', 'egress_code_asr', 'egress_code_acd', 'static_route', 'dynamic_route', 'route_plan', 'route_prefix', 'orig_delay_second', 'term_delay_second', 'orig_call_duration', 'trunk_type', 'origination_profile_port', 'termination_profile_port', 'o_trunk_type2', 'o_billing_method', 't_trunk_type2', 't_billing_method', 'campaign_id', 'tax', 'agent_id', 'agent_rate', 'agent_cost', 'orig_jur_type', 'term_jur_type', 'ring_epoch', 'end_epoch', 'paid_user', 'rpid_user', 'timeout_type','q850_cause','q850_cause_string')
INVOICE_TABLE_FIELDS_ShowCodeSummery = ('calls','ingress_client_bill_time','ingress_client_cost')
INVOICE_TABLE_FIELDS_GetUsage= ('count','rate')
URL = "http://192.99.10.113:8889"
#curl --data "{'start': 1483833600, 'i': 15, 'end': 1487116800}" 192.99.10.113:8889
#curl "192.99.10.113:8889?token=XXXXXXX"


def ShowCodeSummery(start,end,ingress_id ,url ):
	#start_date = "2017-02-01 00:00:00"
	#end_date = "2017-02-16 11:00:00"   {'start': 1485907200, 'end': 1487375999, 'i': 15} 
	# curl --data "start=1485907200&end=1487375999&i=29&method=non_zero_count" 192.99.10.113:8890   #field=ingress_cost
	print (start,end)
	start_datetime_ts =  int( datetime.datetime.strptime(str(start), "%Y-%m-%d %H:%M:%S").timestamp() )
	end_datetime_ts = int(datetime.datetime.strptime(str(end), "%Y-%m-%d %H:%M:%S")	.timestamp()  )
	data={'start': start_datetime_ts, 'end':end_datetime_ts } #'i': 20   , 'i': ingress_id   , 'egress_rcause':"NORMAL_CLEARING"  ,'s':'sipp' 'i': ingress_id,,'s':'sipp'  
	r = requests.post(URL, data) 
	print (data)
	print(r.url)
	#print(r.status_code, r.reason,r.text)
	try:
		token = r.json()['token']
	except Exception as e: 
		print (" %s" % (str(e) ))
		print(r.status_code, r.reason)
		print("exiting")
		exit(1)
	data =''
	payload = {'token': token}
	print ("payload:", payload)
	while True:
		r = requests.get(URL, params=payload)
		print(r.json()['msg'],"status code ",r.status_code , r.json().keys()  )
		if r.json()['msg'] == None :
			data = r.json()['data']
		if r.json()['msg'] == 'Bad request' or r.json()['msg'] == 'Requested token was not found' :
			print ("exiting from the loop")
			break
		time.sleep(1)
	cdr_detail = defaultdict(lambda : dict.fromkeys(INVOICE_TABLE_FIELDS_ShowCodeSummery, 0) )   #dict of dict
	reader = csv.DictReader(data, delimiter='?', fieldnames=CDR_FIELDS)
	#orig_code,sum(ingress_client_bill_time) as bill_time ,sum(ingress_client_cost ) as cost ,sum(not_zero_calls)  as calls
	for row in reader:
		cdr_detail [row['orig_code']]['ingress_client_bill_time'] +=  int(row['ingress_client_bill_time'])
		cdr_detail [row['orig_code']]['ingress_client_cost'] +=  float(row['ingress_client_cost'])
		cdr_detail [row['orig_code']]['calls'] +=  1
		#if row['orig_code'] != '9':
		#	print  (['ingress_client_bill_time'],['ingress_client_cost'])
	print (cdr_detail)
	return_list = []
	for key in cdr_detail.keys():
		elem_dict  =cdr_detail[key]
		elem_dict['orig_code'] = key
		return_list.append( elem_dict)
	print ("return_list len",len(return_list) )
	return return_list

def GetUsage(start,end,ingress_id ,url):
	#curl --data "start=1485907200&end=1487375999" 192.99.10.113:8889
	#select  ingress_rate_effective_date,orig_code_name  , avg (ingress_client_rate)  as rate 
	#start_date = "2017-02-15 00:00:00"
	#end_date = "2017-02-16 11:00:00"
	#print (type(start),end)
	start_datetime_ts =  int( datetime.datetime.strptime(str(start), "%Y-%m-%d %H:%M:%S").timestamp() )
	end_datetime_ts = int(datetime.datetime.strptime(str(end), "%Y-%m-%d %H:%M:%S")	.timestamp()  )
	data={'start': start_datetime_ts, 'end':end_datetime_ts ,'z':1 } #'i': 20    'i': ingress_id  'egress_rcause':"NORMAL_CLEARING",'source_number':'sipp'  ,'s':'sipp'
	r = requests.post(URL, data)
	print (data)
	print(r.url)
	#print(r.status_code, r.reason,r.text)
	try:
		token = r.json()['token']
	except Exception as e: 
		print (" %s" % (str(e) ))
		print(r.status_code, r.reason)
		print("exiting")
		exit(1)
	data =''
	payload = {'token': token}
	while True:
		r = requests.get(URL, params=payload)
		print(r.json()['msg'],"status code ",r.status_code , r.json().keys()  )
		if r.json()['msg'] == None :
			data = r.json()['data']
		if r.json()['msg'] == 'Requested token was not found':
			print ("exiting from the loop")
			break
		time.sleep(1)
	
	cdr_detail = defaultdict(lambda : dict.fromkeys(INVOICE_TABLE_FIELDS_GetUsage, 0) )   #dict of dict
	cdr_orig_code_name = defaultdict(lambda : cdr_detail )
	reader = csv.DictReader(data, delimiter='?', fieldnames=CDR_FIELDS)
	
	counter= 0 
	#orig_code,sum(ingress_client_bill_time) as bill_time ,sum(ingress_client_cost ) as cost ,sum(not_zero_calls)  as calls orig_code_name ,ingress_rate_effective_date
	for row in reader:
		counter +=1
		if int(row['call_duration']) != 0:
			cdr_orig_code_name [row['orig_code_name']][row['ingress_rate_effective_date']]['rate'] +=  float(row['ingress_client_rate'])
			cdr_orig_code_name [row['orig_code_name']][row['ingress_rate_effective_date']]['count'] +=1
		
	print ("count:",counter)
	return_list = []
	for orig_code_key in cdr_orig_code_name.keys():
		#print ("key:",orig_code_key )
		for effective_date_key in cdr_orig_code_name[orig_code_key].keys():
			#print ("keyLow:",effective_date_key )
			lst = [orig_code_key,datetime.datetime.fromtimestamp(int(effective_date_key)).strftime('%c')]
			lst.append( cdr_orig_code_name[orig_code_key][effective_date_key]['rate'] /  cdr_orig_code_name[orig_code_key][effective_date_key] ['count'] )
			return_list.append(lst)
	print ("return_list GetUsage",return_list )
	return return_list

def main():
	start = "2017-02-01 00:00:00"
	end = "2017-02-17 23:59:59"
	ingress_id = 15
	url = URL
	ShowCodeSummery(start,end,ingress_id ,url)


def Old_main():
	print("2017-02-16")
	start_date = "2017-02-15 00:00:00"
	end_date = "2017-02-16 11:00:00"
	start_datetime_ts =  int( datetime.datetime.strptime(start_date, "%Y-%m-%d %H:%M:%S").timestamp() )
	end_datetime_ts = int(datetime.datetime.strptime(end_date, "%Y-%m-%d %H:%M:%S")	.timestamp()  )
	data={'start': start_datetime_ts, 'end':end_datetime_ts  , 'egress_rcause':"NORMAL_CLEARING" } #'i': 20   
	r = requests.post(URL, data)
	print (data)
	print(r.url)
	#print(r.status_code, r.reason,r.text)
	try:
		token = r.json()['token']
	except Exception as e: 
		print (" %s" % (str(e) ))
		print(r.status_code, r.reason)
		print("exiting")
		exit(1)
	data =''
	payload = {'token': token}
	while True:
		r = requests.get(URL, params=payload)
		print(r.json()['msg'],"status code ",r.status_code , r.json().keys()  )
		if r.json()['msg'] == None :
			data = r.json()['data']
		if r.json()['msg'] == 'Bad request':
			print ("exiting from the loop")
			break
		time.sleep(5)
	
	#print (type(data),len(data) )
	#,data	
	#curl --data "start=1483228800&end=1487116800" 192.99.10.113:8889
	
	
	cdr_detail = defaultdict(lambda : dict.fromkeys(INVOICE_TABLE_FIELDS, 0) )   #dict of dict
	#cdr_detail =  defaultdict(lambda: 0)
	reader = csv.DictReader(data, delimiter='?', fieldnames=CDR_FIELDS)
	i = 0
	#orig_code,sum(ingress_client_bill_time) as bill_time ,sum(ingress_client_cost ) as cost ,sum(not_zero_calls)  as calls
	for row in reader:
		cdr_detail [row['orig_code']]['ingress_client_bill_time'] +=  int(row['ingress_client_bill_time'])
		cdr_detail [row['orig_code']]['ingress_client_cost'] +=  float(row['ingress_client_cost'])
		cdr_detail [row['orig_code']]['ingress_client_cost'] +=  1
		#cdr_detail['call_duration'] +=  int(row['call_duration'])
	print (cdr_detail)
	return_list = []
	for key in cdr_detail.keys():
		elem_dict  =cdr_detail[key]
		elem_dict['orig_code'] = key
		return_list.append( elem_dict)
	print ("return_list",return_list )
	return 0



if __name__ == '__main__':
	main()

