#!/usr/bin/env python3
# latest - math.abs() fixed
from decimal import Decimal
import InvoiceCreate as IC
# import InvoiceCreateJur as IC_Jur
import psycopg2
import psycopg2.extras
import datetime
import configparser, get_cdr_records
import shutil
import time, math, string, pytz
import datetime as dt
import base64
from timeit import default_timer as timer

import logging, os, traceback
from logging.handlers import TimedRotatingFileHandler
from logging.handlers import RotatingFileHandler

import requests

from get_cdr import getCdrLink, GetCdrFields
from mail import get_smtp_info, send_mail, send_email_log
import PIL
from PIL import Image, ImageFile
from collections import defaultdict as def_dict

TABLE_NAMES = ""
CDR_UI = "/homes/cdr_from_invoice/%s"  # invoice_id

LOGGER_LIST = []
LOGO_URL = "http://yaro.denovolab.com/upload/images/ilogo.png"  # http://switchui.denovolab.com/dial_master/web/upload/images/ilogo.png

logger = logging.getLogger('manual invoice')
pidfile = ''
pg_cur = 1
logo_file = "logo.jpg"
log_id = ""
invoice_log_id = 0
config = {}

SQL_DICT = {
    'buy_cdr_field':
        'trunk_id_origination,route_prefix,orig_code_name,ingress_rate_effective_date,ingress_client_bill_time,ingress_client_cost,ingress_client_id,answer_time_of_date,ingress_client_bill_result,time,call_duration,ingress_client_rate,ingress_rate_type',
    'sell_cdr_field':
        'term_code_name,egress_rate_effective_date,egress_bill_time,egress_cost,egress_client_id,answer_time_of_date,egress_bill_result,time,call_duration,egress_rate,egress_rate_type',
    'buy_group_field': 'trunk_id_origination,route_prefix,orig_code_name,ingress_rate_effective_date',
    'sell_group_field': 'term_code_name,egress_rate_effective_date',
    'cdr_split': '\t',
    'group_split': ';'
}  # maybe can be deleted!

SWITCH_INVOICE_SETTINGS = '''SELECT invoice_name, tpl_number, pdf_tpl, company_info,overlap_invoice_protection,send_cdr_fields, invoice_send_mode,company_info_location,invoice_decimal_digits
 FROM system_parameter'''
CLIENT_INVOICE_SETTINGS = '''SELECT invoice_name, tpl_number, pdf_tpl, company_info,overlap_invoice_protection,send_cdr_fields, invoice_send_mode,company_info_location,invoice_decimal_digits
 FROM invoice_settings INNER JOIN client_invoice_settings on invoice_settings.id = client_invoice_settings.invoice_setting_id where client_invoice_settings.client_id = %s'''

float_form0 = "%.f"
float_form2 = "%0.2f"
float_form3 = "%0.3f"


def log_to_file_with_date(filename, new_line, mode="a"):
    '''Logging to file from subproccess started from the daemon. Accept filename and mode for opening the file. Valid options a (for append),w ,w+ "" and line to be written'''
    time_log_str = datetime.datetime.now().strftime("%Y-%m-%d %H:%M:%S")
    with open(filename, mode) as f:
        f.write("\n%s:\n" % (time_log_str))
        f.write(str(new_line))
        f.write("\n\n")
        f.flush()


def connect_to_postgresql(host, port, database, user, password=None):
    time_log_str = datetime.datetime.now().strftime("%Y-%m-%d %H:%M:%S")
    try:
        conn = psycopg2.connect(host=host, port=port, database=database, user=user, password=password)
        conn.autocommit = True
        cursor = conn.cursor(cursor_factory=psycopg2.extras.DictCursor)
        logger.debug("connected to DB")
    except Exception as e:
        logger.error(" Exception in DB connecting:  %s" % (str(e)))
        # print ("errorInv.txt"," Exception in DB connecting:  %s" % (str(e) ) )
        print(" Exception in DB connecting:  %s" % (str(e)))
        return -1, -1
    return conn, cursor


def DownloadLogoNoResize():
    path = 'resized_' + logo_file

    r = requests.get(LOGO_URL, stream=True)
    if r.status_code == 200:
        with open(path, 'wb') as f:
            r.raw.decode_content = True
            shutil.copyfileobj(r.raw, f)


def apiTest():
    """Api Test"""
    if True:
        return "running"




def DownloadLogo(client_id=None):
    path = logo_file
    try:
        LOGO_URL = config["other"]['logo_file_get']
        if client_id:
            LOGO_URL = '{}?client_id={}'.format(LOGO_URL, client_id)
        r = requests.get(LOGO_URL)
        if r.status_code == 200:
            LOGO_URL = r.text
        print("Logo URL is set to " + LOGO_URL)
        logger.debug("Logo URL is set to " + LOGO_URL)
        LOGGER_LIST.append("Logo URL is set to " + LOGO_URL)
        r = requests.get(LOGO_URL, stream=True)
        if r.status_code == 200:
            with open(path, 'wb') as f:
                r.raw.decode_content = True
                shutil.copyfileobj(r.raw, f)
    except:
        pass  # get default file

    basewidth = 300
    ImageFile.LOAD_TRUNCATED_IMAGES = True
    img = Image.open(path).convert('RGB')
    wpercent = (basewidth / float(img.size[0]))
    hsize = int((float(img.size[1]) * float(wpercent)))
    img = img.resize((basewidth, hsize), PIL.Image.ANTIALIAS)
    img.save('resized_' + path)


def setup():
    global logo_file, config
    # Get config.ini
    config = configparser.ConfigParser()
    config.read('./config.ini')
    # set logger
    if int(config["log"].get('by_time', 0)) == 1:
        handler = TimedRotatingFileHandler(config["log"].get("invoice_manual_path", "invoice_manual.log"),
                                           when=config["log"].get("invoice_manual_when", 'm'),
                                           interval=int(config["log"].get("invoice_manual_interval", 0)),
                                           backupCount=int(config["log"].get("invoice_manual_backupCount", 5)))
    else:  # by size
        handler = RotatingFileHandler(config["log"].get("invoice_manual_path", "invoice_manual.log"),
                                      maxBytes=int(config["log"].get("invoice_manual_maxBytes", 1024 * 1024 * 128)),
                                      backupCount=int(config["log"].get("invoice_manual_backupCount", 5)))

    if int(config["log"].get("debug_level", 0)) == 1:
        logger.setLevel(logging.DEBUG)
    else:
        logger.setLevel(logging.INFO)
    formatstr = '%(asctime)s - %(name)s - %(levelname)s - %(message)s'
    formatstr = '%(asctime)s: %(levelname)s:  %(message)s'
    formatter = logging.Formatter(formatstr)
    handler.setFormatter(formatter)
    logger.addHandler(handler)

    try:
        LOGO_URL = config['other']['logo_file_get']
    except Exception as e:
        logger.error(" Exception in getting logo path. Possible error in config.ini:  %s" % (str(e)))
        LOGGER_LIST.append(" Exception in getting logo path. Possible error in config.ini:  %s" % (str(e)))

    try:
        pg_cur = -1
        while pg_cur == -1:
            print("Connecting to DB")
            pg_conn, pg_cur = connect_to_postgresql(config["POSTGRES"]["hostaddr"], int(config["POSTGRES"]["port"]),
                                                    config["POSTGRES"]["dbname"], config["POSTGRES"]["user"],
                                                    config["POSTGRES"]["password"])  # config["POSTGRES"]["password"]
            time.sleep(.5)
        return pg_cur, logger
    except Exception as e:
        logger.error(" Exception in connecting postgress:  %s" % (str(e)))
        print(" Exception in connecting postgress:  %s" % (str(e)))
        LOGGER_LIST.append(" Exception in connecting postgress:  %s" % (str(e)))
        return -1, logger


def new_cursor():
    '''Returns new DB cursor '''

    # Get config.ini
    config = configparser.ConfigParser()
    config.read('./config.ini')
    pg_cur = None
    try:
        pg_conn, pg_cur = connect_to_postgresql(config["POSTGRES"]["hostaddr"], int(config["POSTGRES"]["port"]),
                                                config["POSTGRES"]["dbname"], config["POSTGRES"]["user"],
                                                config["POSTGRES"]["password"])  # config["POSTGRES"]["password"]
    except Exception as e:
        print(" Exception in connecting postgress:  %s" % (str(e)))
        return None

    return pg_cur


def main(args):
    global logo_file, config
    # Get config.ini
    config = configparser.ConfigParser()
    config.read('./config.ini')
    # set logger
    if int(config["log"].get('by_time', 0)) == 1:
        handler = TimedRotatingFileHandler(config["log"].get("invoice_manual_path", "invoice_manual.log"),
                                           when=config["log"].get("invoice_manual_when", 'm'),
                                           interval=int(config["log"].get("invoice_manual_interval", 0)),
                                           backupCount=int(config["log"].get("invoice_manual_backupCount", 5)))
    else:  # by size
        handler = RotatingFileHandler(config["log"].get("invoice_manual_path", "invoice_manual.log"),
                                      maxBytes=int(config["log"].get("invoice_manual_maxBytes", 1024 * 1024 * 128)),
                                      backupCount=int(config["log"].get("invoice_manual_backupCount", 5)))

    if int(config["log"].get("debug_level", 0)) == 1:
        logger.setLevel(logging.DEBUG)
    else:
        logger.setLevel(logging.INFO)
    formatstr = '%(asctime)s - %(name)s - %(levelname)s - %(message)s'
    formatstr = '%(asctime)s: %(levelname)s:  %(message)s'
    formatter = logging.Formatter(formatstr)
    handler.setFormatter(formatter)
    logger.addHandler(handler)

    try:
        LOGO_URL = config['other']['logo_file_get']
    except Exception as e:
        logger.error(" Exception in getting logo path. Possible error in config.ini:  %s" % (str(e)))
        LOGGER_LIST.append(" Exception in getting logo path. Possible error in config.ini:  %s" % (str(e)))

    pg_cur = None
    try:
        pg_conn, pg_cur = connect_to_postgresql(config["POSTGRES"]["hostaddr"], int(config["POSTGRES"]["port"]),
                                                config["POSTGRES"]["dbname"], config["POSTGRES"]["user"],
                                                config["POSTGRES"]["password"])  # config["POSTGRES"]["password"]
    except Exception as e:
        logger.error(" Exception in connecting postgress:  %s" % (str(e)))
        print(" Exception in connecting postgress:  %s" % (str(e)))
        LOGGER_LIST.append(" Exception in connecting postgress:  %s" % (str(e)))

    return pg_cur, logger


def get_invoice_no():
    ''' Gets new No for invoice'''
    get_number_sql = "SELECT nextval('class4_seq_invoice_no'::regclass) as no;"
    print("get_invoice_no")
    # изключено заради теста , нямам достъп до базата данни !
    #
    start = timer()
    logger.debug("Entering get invoice number")
    try:
        pg_cur.execute(get_number_sql)
        result_id = pg_cur.fetchone()
        print("result Id:", str(result_id))
        new_id = int(result_id['no'])
    except Exception as e:
        logger.error(" Exception in getting invoice num:  %s" % (str(e)))
        print(" Exception in getting invoice number:  %s" % (str(e)))
        # print ("result Id:",str(result_id) )
        ErrorInLogInvoice(str(e))
        # logger.debug (" Exception in getting invoice number:  %s" % (str(e) ) )
        return -1

    while True:
        exist_no_sql = "select invoice_number as no from invoice where invoice_number='%s' limit 1 " % (new_id)
        pg_cur.execute(exist_no_sql)
        result_check = pg_cur.fetchone()
        if result_check != None:
            print("exisiting invoice number %s." % new_id)
            logger.debug("exisiting invoice number")
            new_id = new_id + 1
        else:
            print("NOT exisiting invoice number %s. " % (new_id))
            print(str(result_check))
            break

    ###v stariq code ima proverka dali ne se izpolsva ?!??! "select invoice_number as no from invoice where invoice_number='$get_number' limit 1 " Warning 01
    # new_id = 13
    end = timer()
    logger.debug("execution time  in milliseconds: %s" % ((end - start) * 1000))

    return new_id


def GetParams(options, regenInvoice=False):
    '''Getting and setting params from api request '''
    global pg_cur
    invoice_option = dict(options)
    #
    start = timer()
    logger.debug("Entering get client options/params")

    # if regenInvoice == False:   even regenerated invoices must have new invoice number
    invoice_option['invoice_number'] = get_invoice_no()
    # else:
    #	pass #it must be in dict

    try:
        print('Getting and setting params from api request ', invoice_option, invoice_option['client_id'])
        client_id = invoice_option['client_id'][0]
    except:
        client_id = invoice_option['client_id']
        print(type(options), options.get('client_id'), type(invoice_option), invoice_option.get('client_id'))

    sql = """select current_timestamp(0) as balance_time,c4_client_balance.balance as current_balance,* from client left join c4_client_balance on 
	c4_client_balance.client_id=client.client_id::text where client.client_id=%s""" % (client_id)
    print("SQL for getting sql data: ", sql)
    LOGGER_LIST.append("SQL for getting sql data: %s" % sql)
    try:
        pg_cur.execute(sql)
        client_data = pg_cur.fetchone()
        if client_data == None:
            print("ERROR! No CLIENT_DATA!!")
            logger.error("ERROR! No CLIENT_DATA!!")
            LOGGER_LIST.append("ERROR! No CLIENT_DATA!!")
            return -1
        invoice_option['scc_bellow'] = client_data.get('scc_bellow')
        invoice_option['scc_percent'] = client_data.get('scc_percent')
        invoice_option['scc_charge'] = client_data.get('scc_charge')
        invoice_option['scc_type'] = client_data.get('scc_type')
        invoice_option['invoice_file_type'] = client_data.get('invoice_format')
        invoice_option['invoice_zero'] = client_data.get('invoice_zero')
        invoice_option['current_balance'] = client_data.get('current_balance')
        invoice_option['balance_time'] = client_data.get('balance_time')
        invoice_option['is_send_as_link'] = 0;
        invoice_option['balance_time'] = client_data.get('balance_time')
        invoice_option['is_invoice_account_summary'] = client_data.get('is_invoice_account_summary')

        invoice_option['jurisdict'] = invoice_option.get('dayusage', False)  # ('dayusage')
        if client_data['include_tax']:
            invoice_option['tax'] = client_data['tax']

        else:
            invoice_option['tax'] = 0

    except Exception as e:
        # log_to_file("errorManInv.txt","Exception in fetching client data:  %s" % (str(e) ) )
        print("Exception in fetching client data:  %s" % (str(e)))
        logger.debug("Exception in fetching client data:  %s" % (str(e)))
        ErrorInLogInvoice(str(e))
        return -1
    end = timer()
    logger.debug("execution time  in milliseconds: %s" % ((end - start) * 1000))
    return invoice_option


def GetParamsAuto(options):
    '''Getting and setting params from api request '''
    global pg_cur
    invoice_option = dict(options[0])
    invoice_option['end_date'] = options[2].split(".")[0]  # .replace("+00:00","")
    invoice_option['start_date'] = options[1].split(".")[0]  # .replace("+00:00","")
    invoice_option['balance_time'] = options[3].split(".")[0]  # .replace("+00:00","")
    invoice_option['invoice_date'] = options[3].split(".")[0]  # .replace("+00:00","")
    invoice_option['is_invoice_usage_detail'] = invoice_option['is_show_daily_usage']
    invoice_option['jurisdict'] = invoice_option['invoice_jurisdictional_detail']
    invoice_option['is_short_duration_call_surcharge_detail'] = invoice_option[
        'is_short_duration_call_surcharge_detail']
    invoice_option['is_invoice_account_summary'] = invoice_option[
        'is_invoice_account_summary']  # is_invoice_account_summary
    invoice_option['detail_by_trunk'] = invoice_option['is_show_detail_trunk']
    # invoice_option['include_origination_billing'] =invoice_option ['']
    invoice_option['show_calls_date'] = invoice_option['is_show_by_date']
    invoice_option['show_code_summery'] = invoice_option['is_show_code_100']
    invoice_option['show_trafic_country'] = invoice_option['is_show_country']
    invoice_option['show_trafic_code_name'] = invoice_option['is_show_code_name']  # Show Traffic Analysis by Code Name
    invoice_option['summary_of_payments'] = invoice_option['invoice_include_payment']
    invoice_option['ingress_prefix'] = invoice_option['is_show_total_trunk']

    #
    #
    start = timer()
    logger.debug("Entering get client options/params")

    ''''
{'current_balance': Decimal('20.250000'), 'is_invoice_usage_detail': ,   : ['show_code_summery'],, 
'summary_of_payments': ['summary_of_payments'], 'tax': 0, 
, 'dayusage': ['dayusage'], , 
: ['show_trafic_code_name'],  'is_send_as_link': 0, 'invoice_number': 1021, 'show_calls_date': ['show_calls_date'], 
'scc_bellow': 50, : ['include_origination_billing'], 
: datetime.datetime(2016, 12, 19, 13, 52, 2, tzinfo=psycopg2.tz.FixedOffsetTimezone(offset=0, name=None)), 
 'invoice_zero': True, : ['ingress_prefix'], 'Submit': ['1'], 'client_id': '297'} 

	'''
    invoice_option['invoice_number'] = get_invoice_no()

    sql = """select current_timestamp(0) as balance_time,c4_client_balance.balance as current_balance,* from client left join c4_client_balance on 
	c4_client_balance.client_id=client.client_id::text where client.client_id=%s""" % (invoice_option['client_id'])
    print("SQL for getting sql data: ", sql)
    try:
        pg_cur.execute(sql)
        client_data = pg_cur.fetchone()
        if client_data == None:
            print("ERROR! No CLIENT_DATA!!")
            return -1
        invoice_option['scc_bellow'] = client_data.get('scc_bellow')
        invoice_option['scc_percent'] = client_data.get('scc_percent')
        invoice_option['scc_charge'] = client_data.get('scc_charge')
        invoice_option['scc_type'] = client_data.get('scc_type')
        invoice_option['invoice_file_type'] = client_data.get('invoice_format')
        invoice_option['invoice_zero'] = client_data.get('invoice_zero')
        invoice_option['current_balance'] = client_data.get('current_balance')
        invoice_option['balance_time'] = client_data.get('balance_time')
        invoice_option['is_send_as_link'] = 0;

        if client_data['include_tax']:
            invoice_option['tax'] = client_data['tax']
            invoice_option['include_tax'] = client_data['include_tax']
        else:
            invoice_option['tax'] = 0
    # print invoice_option['tax']
    except Exception as e:
        # log_to_file("errorManInv.txt","Exception in fetching client data:  %s" % (str(e) ) )
        print("Exception in fetching client data:  %s" % (str(e)))
        logger.error("Exception in fetching client data:  %s" % (str(e)))

        return -1
    end = timer()
    logger.debug("execution time  in milliseconds: %s" % ((end - start) * 1000))
    return invoice_option


def GetDetailTable(param_dict, is_one_day, today, delta, start_date_obj):
    '''return the details table'''
    #
    start = timer()
    logger.debug("Entering Get Detail Table")
    LOGGER_LIST.append("Entering Get Detail Table")
    client_id = param_dict.get('client_id')
    if is_one_day:
        daily_query_detail = """select ingress_code_name ,ingress_client_id ,ingress_code,ingress_rate_date,ingress_rate,sum(ingress_bill_time) as bill_time,sum(ingress_call_cost) as cost,sum(not_zero_calls) as calls,sum(ingress_call_cost_intra) as cost_intra,
sum(ingress_call_cost_inter) as cost_inter,sum(ingress_bill_time_intra) as bill_time_intra,sum(ingress_bill_time_inter) as bill_time_inter from cdr_report_detail%s where ingress_client_id=%s 
and not_zero_calls>0 group by ingress_code,ingress_rate_date,ingress_rate,ingress_code_name ,ingress_client_id""" % (
            today, client_id)
        print("daily_detail query: ", daily_query_detail)

        pg_cur.execute(daily_query_detail)
        detail_result = pg_cur.fetchall()
    else:
        detail_quer_part1 = """ select ingress_code_name ,ingress_client_id ,ingress_rate_date,ingress_rate,sum(ingress_bill_time) as bill_time,sum(ingress_call_cost) as cost,sum(not_zero_calls) as calls,sum(ingress_call_cost_intra) as cost_intra,
sum(ingress_call_cost_inter) as cost_inter,sum(ingress_bill_time_intra) as bill_time_intra,sum(ingress_bill_time_inter) as bill_time_inter  from ( select ingress_code_name ,ingress_client_id ,ingress_code,ingress_rate_date,
ingress_rate,ingress_bill_time,ingress_call_cost,not_zero_calls,ingress_call_cost_intra,
ingress_call_cost_inter,ingress_bill_time_intra,ingress_bill_time_inter  from cdr_report_detail%s""" % (
            start_date_obj.strftime("%Y%02m%02d"))
        detail_quer_part3 = """) t where ingress_client_id = %s  and not_zero_calls>0 group by ingress_code_name ,ingress_code,ingress_rate_date,ingress_rate,ingress_client_id ;""" % (
            client_id)
        detail_make_query = detail_quer_part1
        for i in range(1, delta.days + 1):
            day = start_date_obj + datetime.timedelta(days=i)  # .replace("-","")
            day = day.strftime("%Y%02m%02d")
            detail_make_query += """ union all select ingress_code_name , ingress_client_id ,ingress_code,ingress_rate_date,ingress_rate,ingress_bill_time,ingress_call_cost,not_zero_calls,ingress_call_cost_intra,
ingress_call_cost_inter,ingress_bill_time_intra,ingress_bill_time_inter from public.cdr_report_detail%s """ % (day)
        detail_make_query += detail_quer_part3
        print("detail_make_query: ", detail_make_query)
        logger.debug("detail_make_query: " + detail_make_query)
        LOGGER_LIST.append("detail_make_query: " + detail_make_query)
        pg_cur.execute(detail_make_query)
        detail_result = pg_cur.fetchall()
    end = timer()
    logger.debug("execution time  in milliseconds: %s" % ((end - start) * 1000))
    return detail_result


def CheckTableNames(start_date_obj, delta):
    sql = "select table_name from information_schema.tables where table_name LIKE '%cdr_report_detail%';"
    try:
        pg_cur.execute(sql)
        TABLE_NAMES = pg_cur.fetchall()
        if TABLE_NAMES != None:
            TABLE_NAMES = [elem[0] for elem in TABLE_NAMES]
        else:
            TABLE_NAMES = []
        print("Table_Names Length: %s" % len(TABLE_NAMES))
        logger.debug("Table_Name Length: %s" % len(TABLE_NAMES))
    except Exception as e:
        logger.error(" Exception in getting  TABLE_NAMES:  %s" % (str(e)))
        ErrorInLogInvoice(str(e))
        print(" Exception in getting invoice TABLE_NAMES:  %s" % (str(e)))
        return (-1, " Exception in getting  TABLE_NAMES:  %s" % (str(e)))

        print("TABLE_NAMES", str(TABLE_NAMES), type(TABLE_NAMES))
    for i in range(1, delta.days + 1):
        day = start_date_obj + datetime.timedelta(days=i)  # .replace("-","")
        day = day.strftime("%Y%02m%02d")
        if "cdr_report_detail" + day not in TABLE_NAMES:
            table_date = "cdr_report_detail" + day
            error_msg = " Missing table:  %s" % (table_date)
            logger.error(error_msg)
            ErrorInLogInvoice(error_msg)
            print(error_msg)
            return (-2, error_msg)
    return (0, "OK")


def CheckTableNamesNew(param_dict):
    start_date = param_dict.get('start_date').replace("+00:00", "")
    end_date = param_dict.get('end_date').replace("+00:00", "")
    # print ("start_date ",start_date ,end_date)
    client_id = param_dict.get('client_id')

    start_date_obj = datetime.datetime.strptime(start_date, "%Y-%m-%d %H:%M:%S").date()
    end_date_obj = datetime.datetime.strptime(end_date, "%Y-%m-%d %H:%M:%S").date()
    start_datetime_obj = datetime.datetime.strptime(start_date, "%Y-%m-%d %H:%M:%S")
    end_datetime_obj = datetime.datetime.strptime(end_date, "%Y-%m-%d %H:%M:%S")

    delta = end_date_obj - start_date_obj

    sql = "select table_name as table_name from information_schema.tables where table_name LIKE '%cdr_report_detail%';"

    print("Checking for tables")
    # print(sql)
    try:
        pg_cur.execute(sql)
        rows = pg_cur.fetchall()
        if rows != None:
            TABLE_NAMES = [row[0] for row in rows]
        else:
            TABLE_NAMES = []
        print("Table_Names Length: %s" % len(TABLE_NAMES))
        logger.debug("Table_Name Length: %s" % len(TABLE_NAMES))
    except Exception as e:
        logger.error(" Exception in getting  TABLE_NAMES:  %s" % (str(e)))
        ErrorInLogInvoice(str(e))
        print(" Exception in getting invoice TABLE_NAMES:  %s" % (str(e)))
        return (-1, " Exception in getting  TABLE_NAMES:  %s" % (str(e)))

    # print ("TABLE_NAMES" , str(TABLE_NAMES), type(TABLE_NAMES))
    for i in range(0, delta.days + 1):
        day = start_date_obj + datetime.timedelta(days=i)  # .replace("-","")
        day = day.strftime("%Y%02m%02d")
        print("looking for cdr_report_detail" + day, "in:", TABLE_NAMES)
        if "cdr_report_detail" + day not in TABLE_NAMES:
            table_date = "cdr_report_detail" + day
            error_msg = " Missing table:  %s" % (table_date)
            logger.error(error_msg)
            ErrorInLogInvoice(error_msg)
            print(error_msg)
            return (-2, error_msg)
    return (0, "OK")


def GetBill(param_dict):
    basic_result = None
    details_result = None
    #
    start = timer()
    logger.debug("Entering Get Bill / Summery of charges")
    LOGGER_LIST.append("Entering Get Bill ")
    float_form = param_dict.get('float_form', '%.2f')
    rate_float_form = param_dict.get('rate_float_form', '%.4f')
    start_date = param_dict.get('start_date').replace("+00:00", "")
    end_date = param_dict.get('end_date').replace("+00:00", "")
    # print ("start_date ",start_date ,end_date)
    client_id = param_dict.get('client_id')

    if start_date == None or end_date == None or client_id == None:
        print("Error! Must suply client_id,start_date and end_date!")
        logger.debug(" Must suply client_id,start_date and end_date")
        return (-1, "Error! Must suply client_id,start_date and end_date!")
    try:
        start_date_obj = datetime.datetime.strptime(start_date, "%Y-%m-%d %H:%M:%S").date()
        end_date_obj = datetime.datetime.strptime(end_date, "%Y-%m-%d %H:%M:%S").date()
        start_datetime_obj = datetime.datetime.strptime(start_date, "%Y-%m-%d %H:%M:%S")
        end_datetime_obj = datetime.datetime.strptime(end_date, "%Y-%m-%d %H:%M:%S")
    except Exception as e:
        print(str(e))
        logger.error(str(e))
        logger.debug(" Exception in getting GEtBill:  %s" % (str(e)))
        ErrorInLogInvoice(str(e))
        return (-1, str(e))
    delta = end_date_obj - start_date_obj
    results_dict = {}
    total_amount = 0.0
    if delta.days < 0:  # single day - MUST NOT BE USED !

        return (-1, "Error in dates")
    # basic query
    basic_quer_part1 = """select ingress_client_id,sum(ingress_bill_time)  as bill_time , sum (not_zero_calls) as not_zero_calls , sum (ingress_call_cost)  as call_cost from 
(select ingress_client_id,ingress_bill_time ,not_zero_calls, ingress_call_cost,report_time from public.cdr_report_detail%s """ % (
        start_date_obj.strftime("%Y%02m%02d"))
    basic_quer_part3 = """) as A where ingress_client_id = %s and report_time between '%s' and '%s'  group by ingress_client_id ;""" % (
        client_id, start_datetime_obj,
        end_datetime_obj)
    basic_make_query = basic_quer_part1
    # detail query
    details_result = GetDetailTable(param_dict, 0, "NotNeeded", delta, start_date_obj)

    results_dict['details'] = details_result

    status, msg = CheckTableNames(start_date_obj, delta)
    if status != 0:
        return (-1, msg)

    else:
        for i in range(1, delta.days + 1):
            day = start_date_obj + datetime.timedelta(days=i)  # .replace("-","")
            day = day.strftime("%Y%02m%02d")
            basic_make_query += """union all select ingress_client_id,ingress_bill_time  ,not_zero_calls, ingress_call_cost,report_time from public.cdr_report_detail%s """ % (
                day)
    basic_make_query += basic_quer_part3
    print("getting invoice multiunion content query GetBill:", basic_make_query)
    logger.debug("daily_query GetBill: " + basic_make_query)
    LOGGER_LIST.append("daily_query GetBill: " + basic_make_query)
    try:
        pg_cur.execute(basic_make_query)
        basic_result = pg_cur.fetchone()
        # print ("basic_result GetBill:",basic_result)
        results_dict['basic'] = basic_result
        if basic_result == None:
            total_amount = 0
        else:
            total_amount = basic_result['call_cost']
    except Exception as e:
        logger.error(" Exception in getting invoice bill:  %s" % (str(e)))
        ErrorInLogInvoice(str(e))
        print(" Exception in getting invoice bill:  %s" % (str(e)))
        # logger.debug (" Exception in getting invoice details:  %s" % (str(e) ) )
        return (-1, " Exception in getting invoice bill:  %s" % (str(e)))
    '''receive client_id, start date and end date. return gross amount of  bill_time, calls end etc '''
    # select ingress_client_id,sum(ingress_bill_time) total from (  select ingress_client_id,ingress_bill_time from public.cdr_report_detail20160816  union all select ingress_client_id,ingress_bill_time  from public.cdr_report_detail20160815 ) t where ingress_client_id = 13477 group by ingress_client_id ;

    end = timer()
    logger.debug("execution time GET bill in milliseconds: %s" % ((end - start) * 1000))
    print("GET BILL total_amount", float_form % total_amount)
    LOGGER_LIST.append("GET BILL total_amount" + (float_form % total_amount))
    return (results_dict, float(float_form % total_amount))


def IfNoneReturnZero(value):
    '''takes value and if its None, returns zero instead'''
    if value is None:
        return Decimal(0.0)
    else:
        return Decimal(value)


def report_balance(id_client, balance_start_time, balance_end_time):
    balance_start_time = "2000-01-01"
    #
    start = timer()
    logger.debug("Entering Report Balance")
    LOGGER_LIST.append("Entering Report Balance")
    print("Entering Report Balance")
    # my $payment_hr;
    # my $cdr_cost_hr;
    reset_balance = 0.0
    sql_reset_hr = """select * from client_payment where client_id = %s and payment_type = 13 order by receiving_time desc limit 1""" % (
        id_client)
    print(sql_reset_hr)
    try:
        pg_cur.execute(sql_reset_hr)
        result_reset_hr = pg_cur.fetchone()
        # print ("result_reset_hr:",result_reset_hr)
        logger.debug("result_reset_hr:" + str(result_reset_hr))
    except Exception as e:
        logger.error(" Exception in report_balance.sql_reset_hr :  %s" % (str(e)))
        print("errorInv.txt", " Exception in report_balance.sql_reset_hr:  %s" % (str(e)))
        ErrorInLogInvoice(str(e))
        print("result_reset_hr:", result_reset_hr)
        return -1

    if result_reset_hr:
        reset_balance = IfNoneReturnZero(result_reset_hr.get("amount", 0)) + IfNoneReturnZero(
            result_reset_hr.get("egress_amount", 0))
        sql = """select sum(case when payment_type=4 then amount else 0 end) as invoice_received,sum(case when payment_type=5 then amount else 0 end) as payment_received,sum(case when payment_type=8 then amount else 0 end) as credit_note_sent,sum(case when payment_type=12 then amount else 0 end) as debit_note_sent,sum(case when payment_type=3 then amount else 0 end) as invoice_sent,sum(case when payment_type=6 then amount else 0 end) 
		as payment_sent,sum(case when payment_type=7 then amount else 0 end) as credit_note_received,sum(case when payment_type=11 then amount else 0 end) as debit_note_received
		 from client_payment where client_id=%s and payment_time between '%s' and '%s' and payment_time >= '%s' """ % (
            id_client, balance_start_time, balance_end_time, result_reset_hr['payment_time'])
        logger.debug("result_reset_hr:" + sql)
        print(sql)
        try:
            pg_cur.execute(sql)
            payment_hr = pg_cur.fetchone()
            print("payment_hr:", sql)
        except Exception as e:
            logger.error(" Exception in report_balance.sql_payment_hr :  %s" % (str(e)))
            print("errorInv.txt", " Exception in report_balance.sql_payment_hr:  %s" % (str(e)))
            print("payment_hr:", sql)
            logger.debug("errorInv.txt", " Exception in report_balance.sql_reset_hr Else :  %s" % (str(e)))
            return -1

        # $cdr_cost_hr = $dbh->selectrow_hashref( "select sum(case when ingress_client_id=$id_client then ingress_call_cost else 0 end) as total_ingress_cost, sum(case when egress_client_id=$id_client then egress_call_cost else 0 end) as total_egress_cost from cdr_report where report_time between '$balance_start_time' and '$balance_end_time' and report_time >= '$reset_hr->{ payment_time }' and (ingress_client_id=$id_client or egress_client_id=$id_client)" );
        sql_invoice_balance_hr = """select sum(case type when 3 then total_amount else 0 end) as invoice_received,sum(case type when 0 then total_amount else 0 end) as
		 invoice_send from invoice where client_id=%s and invoice_time between '%s' and '%s' and invoice_time >= '%s'
		  and state != -1""" % (id_client, balance_start_time, balance_end_time, result_reset_hr['payment_time'])
        print(sql_invoice_balance_hr)
        try:
            pg_cur.execute(sql_invoice_balance_hr)
            invoice_balance_hr = pg_cur.fetchone()
            print("sql_invoice_balance_hr:", sql_invoice_balance_hr)
        except Exception as e:
            logger.error(" Exception in report_balance.sql_reset_hr :  %s" % (str(e)))
            print("sql_invoice_balance_hr:", sql_invoice_balance_hr)
            logger.error(" Exception in report_balance.sql_reset_hr Else :  %s" % (str(e)))
            return -1

    else:
        result_reset_hr = {}
        sql = """select sum(case when payment_type=4 then amount else 0 end) as invoice_received,sum(case when payment_type=5 then amount else 0 end) as payment_received,
		sum(case when payment_type=8 then amount else 0 end) as credit_note_sent,sum(case when payment_type=12 then amount else 0 end) as debit_note_sent,sum(case when payment_type=3
		 then amount else 0 end) as invoice_sent,sum(case when payment_type=6 then amount else 0 end) as payment_sent,sum(case when payment_type=7 then amount else 0 end) as credit_note_received,
		 sum(case when payment_type=11 then amount else 0 end) as debit_note_received from client_payment where client_id=%s and payment_time between '%s' and '%s' """ % (
            id_client, balance_start_time, balance_end_time)
        logger.debug("result_reset_hr:" + sql)
        print("payment_hr:", sql)
        try:
            pg_cur.execute(sql)
            payment_hr = pg_cur.fetchone()

        except Exception as e:
            logger.error(" Exception in report_balance.sql_payment_hr Else :  %s" % (str(e)))
            print("errorInv.txt", " Exception in report_balance.sql_payment_hr: Else  %s" % (str(e)))
            print("payment_hr: Else", sql)
            logger.debug("errorInv.txt", " Exception in report_balance.sql_reset_hr Else :  %s" % (str(e)))
            return -1

        # $cdr_cost_hr = $dbh->selectrow_hashref( "select sum(case when ingress_client_id=$id_client then ingress_call_cost else 0 end) as total_ingress_cost, sum(case when egress_client_id=$id_client then egress_call_cost else 0 end) as total_egress_cost from cdr_report where report_time between '$balance_start_time' and '$balance_end_time' and report_time >= '$reset_hr->{ payment_time }' and (ingress_client_id=$id_client or egress_client_id=$id_client)" );
        sql_invoice_balance_hr = """select sum(case type when 3 then total_amount else 0 end) as invoice_received,sum(case type when 0 then total_amount else 0 end) as invoice_send from 
		invoice where client_id=%s and invoice_time between '%s'
		 and '%s'""" % (id_client, balance_start_time, balance_end_time)
        logger.debug("result_reset_hr:" + sql_invoice_balance_hr)
        LOGGER_LIST.append("result_reset_hr:" + sql_invoice_balance_hr)
        print("sql_invoice_balance_hr Else:", sql_invoice_balance_hr)
        try:
            pg_cur.execute(sql_invoice_balance_hr)
            invoice_balance_hr = pg_cur.fetchone()

        except Exception as e:
            logger.error(" Exception in report_balance.sql_reset_hr Else :  %s" % (str(e)))
            print("Exception in report_balance.sql_reset_hr: Else %s" % (str(e)))
            print("sql_invoice_balance_hr Else: ", sql_invoice_balance_hr)
            logger.debug("errorInv.txt", " Exception in report_balance.sql_reset_hr Else :  %s" % (str(e)))
            ErrorInLogInvoice(str(e))
            return -1
    if invoice_balance_hr == None:
        invoice_balance_hr = {}
    print("payment_hr,invoice_balance_hr,reset_balance:", payment_hr, invoice_balance_hr, reset_balance)
    # LOGGER_LIST.append("payment_hr,invoice_balance_hr,reset_balance:" + payment_hr+invoice_balance_hr+reset_balance )

    end = timer()
    logger.debug("execution time  in milliseconds: %s" % ((end - start) * 1000))

    return (IfNoneReturnZero(reset_balance) + IfNoneReturnZero(
        payment_hr.get("invoice_received", 0)) + IfNoneReturnZero(
        payment_hr.get("payment_received", 0)) + IfNoneReturnZero(payment_hr.get("credit_note_sent", 0))
            + IfNoneReturnZero(payment_hr.get("debit_note_sent", 0)) + IfNoneReturnZero(
                invoice_balance_hr.get("invoice_received", 0)) - IfNoneReturnZero(payment_hr.get("invoice_sent", 0))
            - IfNoneReturnZero(payment_hr.get("payment_sent", 0)) - IfNoneReturnZero(
                payment_hr.get("credit_note_received", 0)) - IfNoneReturnZero(payment_hr.get("debit_note_received", 0))
            - IfNoneReturnZero(invoice_balance_hr.get('invoice_send', 0))
            )


def GetAcountSummary(param_dict, inv_num):
    '''gets amd makes acount summary if its needed '''
    #
    start = timer()
    logger.debug("Entering Get Acount Summery")
    LOGGER_LIST.append("Entering Get Acount Summery")
    print("Entering Get Acount Summery")

    start_date = param_dict['start_date']
    end_date = param_dict.get('end_date', 0)

    sql = """select * from invoice where client_id=%s and state != -1 and status > 0 and type=0 and invoice_time < now()  and invoice_number != '%s'
	 order by invoice_time desc limit 1; """ % (param_dict['client_id'], inv_num)  #
    previous_balance = None
    payment_credit = 0
    non_recurring_charge = None
    logger.debug(sql)
    try:
        pg_cur.execute(sql)
        result = pg_cur.fetchone()
        print("sql for summery:", sql)
        LOGGER_LIST.append("sql for summery: " + sql)
    except Exception as e:
        logger.error(" Exception in getting invoice details:  %s" % (str(e)))
        print(" Exception in getting invoice summary:  %s" % (str(e)))
        print("sql for summery:", sql)
        logger.debug(" Exception in getting invoice summary:  %s" % (str(e)))
        return None

    if result != None:
        sql = """select sum(case payment_type when 3 then amount*(0-1) when 4 then amount when 5 then amount when 6 then amount*(0-1) else 0 end) 
		as total_payment, sum(case payment_type when 7 then amount*(0-1) when 8 then amount when 11 then amount when 12 then amount*(0-1) else 0 end) 
		as total_note from client_payment where payment_time between '%s' and '%s' and client_id=%s and payment_type in
		 (3,4,5,6,7,8,11,12)""" % (param_dict['start_date'], param_dict['end_date'], param_dict['client_id'])

        logger.debug(sql)
        try:
            pg_cur.execute(sql)
            result_sum = pg_cur.fetchone()
            print("sql for summery_sum:", sql)
        except Exception as e:
            logger.error(" Exception in getting invoice summary_sum:  %s" % (str(e)))
            print(" Exception in getting invoice summary_sum:  %s" % (str(e)))
            print("sql for summery_sum:", sql)
            logger.debug(" Exception in getting invoice summary_sum:  %s" % (str(e)))
            ErrorInLogInvoice(str(e))
            return None

        if param_dict.get('invoice_use_balance_type', False):
            print("balance bt!")
            if not result.get('new_balance', False):
                print("No new balance")
                LOGGER_LIST.append("No new balance")
                previous_balance = report_balance(param_dict['client_id'], 0, result['invoice_time'])  #
            else:
                previous_balance = result.get('new_balance', 0)
        else:
            if not result.get('new_balance', False):
                print("No new balance")
                LOGGER_LIST.append("No new balance")
                previous_balance = report_balance(param_dict['client_id'], 0, result['invoice_time'])  #
            else:
                previous_balance = result.get('new_balance', 0)
        payment_credit = result_sum.get('total_payment', 0)
        if payment_credit == None:
            payment_credit = 0
        non_recurring_charge = result_sum.get('total_note', 0)
        if non_recurring_charge == None:
            non_recurring_charge = 0

    else:
        print("No invoice history")
        LOGGER_LIST.append("No invoice history")

        sql = """select sum(case payment_type when 3 then amount*(0-1) when 4 then amount when 5 then amount when 6 then amount*(0-1) else 0 end) 
		as total_payment, sum(case payment_type when 7 then amount*(0-1) when 8 then amount when 11 then amount when 12 then amount*(0-1) else 0 end) 
		as total_note from client_payment where payment_time between '%s' and '%s' and client_id=%s and payment_type in
		 (3,4,5,6,7,8,11,12)""" % (start_date, end_date, param_dict['client_id'])

        logger.debug(sql)
        print("sql for summery_sum:", sql)
        try:
            pg_cur.execute(sql)
            result_sum = pg_cur.fetchone()

        except Exception as e:
            logger.error(" Exception in getting invoice summary_sum:  %s" % (str(e)))
            print(" Exception in getting invoice summary_sum:  %s" % (str(e)))
            print("sql for summery_sum:", sql)
            logger.debug(" Exception in getting invoice summary_sum:  %s" % (str(e)))
            ErrorInLogInvoice(str(e))
            return None

        previous_balance = 0
        payment_credit = result_sum.get('total_payment', 0)
        if payment_credit == None:
            payment_credit = 0
        non_recurring_charge = result_sum.get('total_note', 0)
        if non_recurring_charge == None:
            non_recurring_charge = 0

        param_dict['current_balance'] = previous_balance + payment_credit + non_recurring_charge + payment_credit
        end = timer()
        logger.debug("execution time  in milliseconds: %s" % ((end - start) * 1000))
        print("previous_balance: ", previous_balance, "payment_credit", payment_credit, "non_recurring_charge",
              non_recurring_charge)
        return {"previous_balance": previous_balance, "payment_credit": payment_credit,
                "non_recurring_charge": non_recurring_charge, "invoice_end_time:": "2001-10-21 00:00:00+00:00"}
    print("previous_balance: ", previous_balance)

    param_dict['current_balance'] = previous_balance + payment_credit + non_recurring_charge
    # logging
    end = timer()
    logger.debug("previous_balance:" + str(previous_balance) + "  payment_credit: " + str(
        payment_credit) + "  non_recurring_charge: " + str(non_recurring_charge) +
                 " invoice_end_time:" + str(result['invoice_time']))
    LOGGER_LIST.append("previous_balance:" + str(previous_balance) + "  payment_credit: " + str(
        payment_credit) + "  non_recurring_charge: " + str(non_recurring_charge) +
                       " invoice_end_time:" + str(result['invoice_time']))
    logger.debug("execution time  in milliseconds: %s" % ((end - start) * 1000))
    return {"previous_balance": previous_balance, "payment_credit": payment_credit,
            "non_recurring_charge": non_recurring_charge, "invoice_end_time": result['invoice_time']}


def InsertInLogInvoice():
    global invoice_log_id
    sql = """insert into invoice_log (start_time,status,cnt ) VALUES ( current_timestamp(0),0,1) returning id """
    print("InsertInLogInvoice( ):", sql)
    LOGGER_LIST.append("InsertInLogInvoice( ):" + sql)
    try:
        pg_cur.execute(sql)
        invoice_log_result = pg_cur.fetchone()
        invoice_log_id = invoice_log_result['id']
    except Exception as e:
        traceback.print_tb(e.__traceback__)
        logger.error("  Error in import log:  %s" % (str(e)))
        return -1

    return invoice_log_id


def ErrorInLogInvoice(reason):
    sql = """update  invoice_log  SET end_time = current_timestamp(0) ,status= 3    where id = %s """ % invoice_log_id
    print("Reason for error:", reason)
    LOGGER_LIST.append("Reason for error: " + reason)
    try:
        pg_cur.execute(sql)

    except Exception as e:
        traceback.print_tb(e.__traceback__)
        logger.error("  Error in error import log:  %s" % (str(e)))
        return -1

    return 0


def FinishInLogInvoice(status=2, cnt=1):
    sql = """update  invoice_log  SET end_time =  current_timestamp(0) ,status = %s  ,cnt = %s  where id = %s """ % (
        status, cnt, invoice_log_id)
    print("FinishInLogInvoice( ):", sql)
    LOGGER_LIST.append("FinishInLogInvoice( ):" + sql)
    try:
        pg_cur.execute(sql)

    except Exception as e:
        traceback.print_tb(e.__traceback__)
        logger.error("  Error in error import log:  %s" % (str(e)))
        return -1

    return 0


def LogInvoice(start_date, end_date, invoice_date, client, param_dict, inv_num, invoice_name, total_amount, pdf_path,
               auto=True):
    #
    start = timer()
    logger.debug("Entering Log Invoice")
    LOGGER_LIST.append("Entering Log Invoice")
    global log_id

    current_balance = param_dict['current_balance']

    try:

        # invoice_date = dt.datetime.strptime(invoice_date , "%Y-%m-%d %H:%M:%S .%f%z" )
        invoice_date = dt.datetime.now(pytz.timezone('GMT')).strftime('%Y-%m-%d %H:%M:%S %z')
        print("invoice_date", invoice_date)
    except:
        print("invoice_date", invoice_date, "%Y-%m-%d")

    if param_dict.get('is_invoice_usage_detail', False) != False:
        is_invoice_usage_detail = "True"
    else:
        is_invoice_usage_detail = "False"

    if param_dict.get('is_short_duration_call_surcharge_detail', False) != False:
        is_short_duration_call_surcharge_detail = "True"
    else:
        is_short_duration_call_surcharge_detail = "False"

    if auto:
        create_type = 0
    else:
        create_type = 1

    if param_dict.get('is_invoice_account_summary', False) != False:
        is_invoice_account_summary = "True"
    else:
        is_invoice_account_summary = "False"

    if not param_dict.get('dayusage', False):
        is_show_daily_usage = False
    else:
        is_show_daily_usage = True

    if param_dict.get('invoice_jurisdictional_detail', False) == True:
        invoice_jurisdictional_detail = "True"
    else:
        invoice_jurisdictional_detail = "False"

    if param_dict.get('is_send_as_link', False) == 0 or param_dict.get('is_send_as_link', False) == False:
        is_send_as_link = False
    else:
        is_send_as_link = True

    try:
        decimal_place = int(param_dict.get('decimal_place', '2')[0])
    except:
        decimal_place = 2
    try:
        decimals_num = int(param_dict.get('decimals_num', '2')[0])
    except:
        decimals_num = 2
    if param_dict.get('summary_of_payments',
                      False) == False:  # param_dict.get('invoice_include_payment',False) == False and
        client_invoice_include_payment = "False"
    else:
        client_invoice_include_payment = "True"

    if param_dict.get('detail_by_trunk', False):
        param_dict['is_show_detail_trunk'] = True
    if param_dict.get('is_show_detail_trunk', False) == False:
        is_show_detail_trunk = "False"
    else:
        is_show_detail_trunk = "True"

    if param_dict.get('show_code_summery', False):
        param_dict['is_show_code_100'] = True
    if param_dict.get('is_show_code_100', False) == False:
        is_show_code_100 = "False"
    else:
        is_show_code_100 = "True"

    if param_dict.get('show_trafic_code_name', False):
        param_dict['is_show_code_name'] = True
    if param_dict.get('is_show_code_name', False) == False:
        is_show_code_name = "False"
    else:
        is_show_code_name = "True"

    if param_dict.get('show_trafic_country', False):
        param_dict['is_show_country'] = True
    if param_dict.get('is_show_country', False) == False:
        is_show_country = "False"
    else:
        is_show_country = "True"

    '''taron [7:24 PM] 
I suggest you keep only "/invoice/2395201708091617.pdf "
then we will found the dile with config_value.db_value


[7:25] 
I mean let's say someone changing config value to download1
then the files won't work with older path in DB


[7:25] 
so it means you need to keep in DB only my suggested part


[7:27] 
without slash 
"invoice/2395201708091617.pdf"'''

    pdf_path = pdf_path.split("/")[-1]
    pdf_path = pdf_path + "/" + invoice_name + ".pdf"
    print("pdf_path", pdf_path)

    if param_dict.get('show_calls_date', False):
        param_dict['is_show_by_date'] = True
    if param_dict.get('is_show_by_date', False) == False:
        is_show_by_date = "False"
    else:
        is_show_by_date = "True"

    # ['show_code_summery'] =result['is_show_code_100']
    sql = """insert into invoice(invoice_balance_time,invoice_use_balance_type,invoice_log_id,usage_detail_fields,is_short_duration_call_surcharge_detail,
	invoice_include_payment,is_show_daily_usage,is_invoice_account_summary,generate_start_time, include_detail,total_amount,buy_total,sell_total,buy_minutes,sell_minutes,buy_service_charge,
	sell_service_charge,state,client_id,invoice_time,invoice_start,invoice_end,due_date,current_balance,invoice_number,type,invoice_zone,create_type,status,invoice_jurisdictional_detail,decimal_place,decimals_num,
	is_send_as_link,pdf_path,is_show_detail_trunk,is_show_code_100,is_show_country,is_show_code_name,is_show_by_date) values( '%s', %s, %s, '%s', %s, %s, %s, %s, '%s', %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, '%s', '%s', '%s', '%s', %s, %s, %s, '%s', %s, %s, %s, %s, %s, %s,'%s',%s ,%s,%s ,%s,%s)""" % (
        param_dict.get('balance_time', 0),
        client.get('invoice_use_balance_type', 0), log_id, client.get('usage_detail_fields', ""),
        is_short_duration_call_surcharge_detail,
        client_invoice_include_payment, is_show_daily_usage, is_invoice_account_summary, param_dict["start_date"],
        is_invoice_usage_detail,
        total_amount, 0, 0, 0, 0, 0, 0, 0, client.get('client_id', 0), invoice_date,
        param_dict['start_date_tmstamp'], param_dict['end_date_tmstamp'], param_dict.get('invoice_due_date', 0),
        current_balance, inv_num, client.get('invoice_type', 0),
        client.get('invoice_zone', 0), create_type, 2, invoice_jurisdictional_detail, decimal_place, decimals_num,
        is_send_as_link,
        pdf_path,
        is_show_detail_trunk, is_show_code_100, is_show_country, is_show_code_name,
        is_show_by_date)  # "/invoice/%s.pdf"%
    print("sql for inserting the invoice:", sql)
    LOGGER_LIST.append("sql for inserting the invoice: " + sql)
    print("Total amount ", total_amount)
    logger.debug("sql for inserting the invoice:" + sql)
    try:
        pg_cur.execute(sql)

    except Exception as e:
        logger.error(" Exception in logging invoice:  %s" % (str(e)))
        print("Exception in logging invoice:  %s" % (str(e)))
        return None
    end = timer()
    logger.debug("execution time  in milliseconds: %s" % ((end - start) * 1000))
    return


def MakeAccountSumary(acount_sumary, float_form, client, param_dict, current_bill):
    '''Makes pdf friendly acount sumary list'''
    #
    start = timer()
    logger.debug("Entering Making Acount summery table")
    if acount_sumary == None:
        return None
    float_form = param_dict.get('float_form', '%.2f')
    rate_float_form = param_dict.get('rate_float_form', '%.4f')
    # print ("Makes pdf friendly acount sumary list")
    # print ("acount_sumary: "+str(acount_sumary),"current_bill:"+str(current_bill),"client: "+str(client))
    payment_credit = acount_sumary.get('payment_credit', 0)
    if payment_credit == None:
        payments = 0.00
    else:
        payments = payment_credit  # -1 *
    prev_balance = acount_sumary.get('previous_balance', 0.00)
    if prev_balance == None:
        prev_balance = 0
    else:
        if int(prev_balance) != 0:
            prev_balance = prev_balance  # -1 *

    if client.get("include_tax", False):
        tax = (IfNoneReturnZero(client.get("tax", 0)) * Decimal(current_bill)) / Decimal(100)
    else:
        tax = Decimal(0.00)
    try:
        result = [["Previous Balance", float_form % (prev_balance)], ["Payments and Credits", float_form % (payments)]
            , ["Balance Forward", float_form % (payments + prev_balance)],
                  ["Long Distance Charges", float_form % (current_bill)],
                  ["Federal, State, and Local Taxes", float_form % (tax)]
            , ["Balance as of %s " % param_dict.get("invoice_date", "N/A"),
               float_form % (-float(current_bill) + float(payments) + float(prev_balance) - float(tax))]]
    except Exception as e:
        logger.error(" Exception in calculating  client balance result:  %s" % (str(e)))
        print("Exception in calculating  client balance result:  %s" % (str(e)))

    # if client['include_tax'] :
    #	result.append (["Federal, State, and Local Taxes:", float_form % ( tax  )])

    query = """select balance,ingress_balance  from client_balance where client_id = '%s';""" % (client['client_id'])
    print(query)
    logger.debug(query)
    LOGGER_LIST.append(query)
    try:
        pg_cur.execute(query)
        balance = pg_cur.fetchone()
        if balance is not None:
            print("balance", balance['balance'], type(balance['balance']), "ingress balance",
                  balance['ingress_balance'], type(balance['ingress_balance']))
            # logger.debug("Current Charges: %s. New Balances: %s, type: %s %s"%( balance.get('balance',"N/A"),balance.get('ingress_balance',"N/A"),
            # type(balance.get('balance',"N/A") ),type(balance.get('ingress_balance',"N/A")),type(balance.get('balance',"N/A")),type(balance.get('ingress_balance',"N/A")) ) )
            # print("Current Charges: %s. New Balances: %s, type: %s %s"%( balance.get('balance',"N/A"),balance.get('ingress_balance',"N/A"),
            # type(balance.get('balance',"N/A") ),type(balance.get('ingress_balance',"N/A")),type(balance.get('balance',"N/A")),type(balance.get('ingress_balance',"N/A")) ) )
            try:

                result.append(["Current Charges:", float_form % (float(balance['balance']))])
                result.append(["New Balances:", float(balance['ingress_balance'])])
            except Exception as e:
                logger.error(" Exception in getting client balance when balance is not None:  %s" % (str(e)))
                print("Exception in getting client balance:  %s" % (str(e)))
                ErrorInLogInvoice(str(e))
        else:
            print("Balance is None!")
    except Exception as e:
        logger.error(" Exception in getting client balance:  %s" % (str(e)))
        print("Exception in getting client balance:  %s" % (str(e)))
        ErrorInLogInvoice(str(e))

    query2 = """select client.mode, client.allowed_credit, client.unlimited_credit,  payment_term.name as payment_term_name from client left join payment_term on client.payment_term_id = 
	payment_term.payment_term_id where client_id = %s;""" % (client.get('client_id', 0))
    print("payment query and more: ", query2)
    logger.debug(query2)
    LOGGER_LIST.append(query2)
    try:
        pg_cur.execute(query2)
        resultQ = pg_cur.fetchone()
        # print (resultQ)
        if resultQ != None:
            print("RESULQ IS NOT NONE!")
            try:
                result.append(["Payment term:", resultQ['payment_term_name']])
            except Exception as e:
                logger.error(" Exception in getting client payment term:  %s" % (str(e)))
                print("Exception in getting client payment term:  %s" % (str(e)))
            try:
                if resultQ['mode'] == 2:
                    if resultQ['unlimited_credit']:
                        result.append(["Credit Remaining:", "Unlimited"])
                    else:
                        result.append(["Credit Remaining:", float_form % (abs(resultQ["allowed_credit"]))])  # math.
            except Exception as e:
                logger.error(" Exception in getting client credit remaining:  %s" % (str(e)))
                print("Exception in getting client credit remaining:  %s" % (str(e)))


    except Exception as e:
        logger.error(" Exception in getting client payment term and more:  %s" % (str(e)))
        print("Exception in getting client payment term and more:  %s" % (str(e)))
    # print ("Tabled Account Summery:" ,result)
    end = timer()
    logger.debug("execution time  in milliseconds: %s" % ((end - start) * 1000))
    return result


'''
def MakeShortSurcharge( param_dict,client,summary_of_charges,minutes,bill_time_secs,not_zero_calls,cost):
	#decides if surcharge is need and returns the surcharge
	pass 
'''


def GetSummary_of_payments(param_dict, client, sumary_of_payments, float_form):
    '''Getting sumarry of payments'''
    #
    start = timer()
    logger.debug("Entering Get Summery of Payments")
    LOGGER_LIST.append("Entering Get Summery of Payments")
    print("Entering Get Summery of Payments")
    # if sumary_of_payments != None:
    #	from_date = sumary_of_payments.get(  param_dict['start_date'] )  #"invoice_end_time" ,
    # else:
    float_form = param_dict.get('float_form', '%.2f')
    rate_float_form = param_dict.get('rate_float_form', '%.4f')
    from_date = param_dict['start_date']
    start_date = from_date
    end_date = param_dict.get('end_date', 0)

    sql = """select client_payment_id,payment_time,case payment_type when 3 then amount*(0-1) when 4 then amount when 5 then amount when 6 then amount*(0-1) else 0 end as total_payment from
client_payment where  client_id= %s and payment_type in (4,5) and payment_time between '%s' and '%s'  order by payment_time desc limit 10 ;""" % (
        client['client_id'], start_date, end_date)  # ,"client_payment_id """ 3 , 6

    logger.debug(sql)
    LOGGER_LIST.append(sql)
    print(sql)
    try:
        pg_cur.execute(sql)
        result = pg_cur.fetchall()
        print("sql for summery_payments:", sql)
        LOGGER_LIST.append("sql for summery_payments:" + sql)
    except Exception as e:
        logger.error(" Exception in getting invoice summary_payments:  %s" % (str(e)))
        print(" Exception in getting invoice summary_payments:  %s" % (str(e)))
        print("sql for summery_payments:", sql)
        logger.debug(" Exception in getting invoice summary_payments:  %s" % (str(e)))
        ErrorInLogInvoice(str(e))
        return None
    table = [["Date", "Payment received"]]
    for row in result:
        new_row = [row.get("payment_time", "N/A"), float_form % (row.get("total_payment", 0))]
        table.append(new_row)
    # print ("Sumary of payments: ",table)
    end = timer()
    logger.debug("execution time  in milliseconds: %s" % ((end - start) * 1000))
    return table


def SumaryOfCharges(basic_bill, param_dict, client, cost, minutes, float_form):
    ''' generates basic table for client bill with surcharges included.'''
    print("generates basic table for client bill with surcharges included")
    #
    start = timer()
    logger.debug("Summery of Short Call Charges")
    LOGGER_LIST.append("Summery of Short Call Charges")
    table = [["", "Total Minutes", "Total Calls", "Total Charges(US)"],
             ["Minutes Usage", minutes, basic_bill.get('not_zero_calls', 0), float_form % (cost)]]

    float_form = param_dict.get('float_form', '%.2f')
    rate_float_form = param_dict.get('rate_float_form', '%.4f')
    scc_perc = param_dict.get('scc_percent', 0) if param_dict.get('scc_percent', 0) != None else 0
    scc_below_sec = param_dict.get('scc_bellow', 0) if param_dict.get('scc_bellow', 0) != None else 0
    scc_charge = param_dict.get('scc_charge', 0) if param_dict.get('scc_charge', 0) != None else 0
    scc_type = param_dict.get('scc_type', 0) if param_dict.get('scc_type',
                                                               0) != None else 0  # if this is selected, then the penalty is 200 * short charge

    if not param_dict.get("is_short_duration_call_surcharge_detail", False):
        print("Carrier do not have surcharge include")
        print("cost:", cost)
        if client.get("include_tax", False):
            tax = (IfNoneReturnZero(client.get("tax", 0)) * Decimal(cost)) / Decimal(100.0)
            row_tax = ["Tax", "", "", float_form % (tax)]
            table.append(row_tax)
        else:
            tax = Decimal(0.0)
        row_total = ["TOTAL", "", "", float_form % (cost + tax)]
        table.append(row_total)
        return table
    result = None

    start_date = param_dict.get('start_date')
    end_date = param_dict.get('end_date')
    client_id = param_dict.get('client_id')

    if start_date == None or end_date == None or client_id == None:
        print("Error! Must suply client_id,start_date and end_date!")
        return {}
    try:
        start_date_obj = datetime.datetime.strptime(start_date, "%Y-%m-%d %H:%M:%S").date()
        end_date_obj = datetime.datetime.strptime(end_date, "%Y-%m-%d %H:%M:%S").date()
    except Exception as e:
        print(str(e))
        ErrorInLogInvoice(str(e))
        logger.error(str(e))
        logger.debug("error %s" % (str(e)))
        return None
    delta = end_date_obj - start_date_obj
    if delta.days < 0:  # single day
        today = end_date_obj.strftime("%Y%02m%02d")
        # basic query  select duration from public.cdr_report  where ingress_client_id =58 and not_zero_calls > 0 and duration < 1000
        daily_query_basic = """select duration from public.cdr_report_detail%s where ingress_client_id =%s and not_zero_calls > 0 and duration < %s;""" % (
            today, client_id, scc_below_sec)
        daily_query_basic_all_duration = """select duration from public.cdr_report_detail%s where ingress_client_id =%s and not_zero_calls > 0 ;""" % (
            today, client_id)
        print("surcharge daily_query: ", daily_query_basic)
        print("surcharge daily_query ALL durations: ", daily_query_basic_all_duration)
        pg_cur.execute(daily_query_basic)
        result = pg_cur.fetchall()
        pg_cur.execute(daily_query_basic_all_duration)
        result_all = pg_cur.fetchall()
    else:
        # basic query  ,ingress_client_id,not_zero_calls sum(duration) , count(*)
        basic_quer_part1 = """select  sum(duration) as dur ,sum(not_zero_calls) as nzc from 
	(select duration,ingress_client_id,not_zero_calls  from public.cdr_report_detail%s """ % (
            start_date_obj.strftime("%Y%02m%02d"))
        basic_quer_part1_all = """select sum(duration) as dur ,sum(not_zero_calls) as nzc  from 
	(select duration,ingress_client_id,not_zero_calls from public.cdr_report_detail%s """ % (
            start_date_obj.strftime("%Y%02m%02d"))

        basic_quer_part3 = """) t where ingress_client_id = %s and not_zero_calls > 0 and duration < %s ;""" % (
            client_id, scc_below_sec)
        basic_make_query = basic_quer_part1
        basic_quer_part3_all = """) t where ingress_client_id = %s and not_zero_calls > 0  ;""" % (client_id)
        basic_make_query_all = basic_quer_part1_all

        for i in range(1, delta.days + 1):
            day = start_date_obj + datetime.timedelta(days=i)  # .replace("-","")
            day = day.strftime("%Y%02m%02d")
            basic_make_query += """union all select duration ,ingress_client_id,not_zero_calls from public.cdr_report_detail%s """ % (
                day)
            basic_make_query_all += """union all select duration ,ingress_client_id,not_zero_calls from public.cdr_report_detail%s """ % (
                day)
        basic_make_query += basic_quer_part3
        basic_make_query_all += basic_quer_part3_all
        logger.debug("getting invoice basic surcharge query:" + basic_make_query)
        print("getting invoice basic surcharge query:" + basic_make_query)
        print("getting invoice multiunion surcharge query ALL:", basic_make_query_all)
        logger.debug("getting invoice multiunion surcharge query ALL:" + basic_make_query_all)
        LOGGER_LIST.append("getting invoice multiunion surcharge query ALL:" + basic_make_query_all)
        try:
            pg_cur.execute(basic_make_query)
            result = pg_cur.fetchone()
            pg_cur.execute(basic_make_query_all)
            result_all = pg_cur.fetchone()

        except Exception as e:
            logger.error(" error in getting surcharge multi:  %s" % (str(e)))
            print("error in getting surcharge multi %s" % (str(e)))
            ErrorInLogInvoice(str(e))
    print("len of surcharged calls", result['nzc'], result_all['nzc'])
    result['nzc'] = result['nzc'] or Decimal(0.0)
    try:
        percent_real = (result['nzc'] / result_all['nzc']) * 100  # we need this a whole percent
    except Exception as e:
        ErrorInLogInvoice(str(e))
        print(str(e))
        logger.error(str(e))
        # logger.debug ("error  %s" % (str(e) ))
        percent_real = 0
    print("percent of short calls %s,  from %s , scc_type is %s" % (percent_real, scc_perc, scc_type))
    LOGGER_LIST.append("percent of short calls %s,  from %s , scc_type is %s" % (percent_real, scc_perc, scc_type))

    if scc_type == 0 and percent_real <= scc_perc:
        LOGGER_LIST.append("scc will be applied to each call ")
        num_calls = result['nzc']  # len (result_all ) -   len (result)
        if num_calls is None:
            num_calls = 0
        charge = num_calls * scc_charge
    elif scc_type == 1 and percent_real > scc_perc:
        LOGGER_LIST.append("scc will be applied to exccess  calls only  ")
        num_calls = result['nzc'] - (scc_perc / 100) * result_all['nzc']
        charge = num_calls * scc_charge
    else:
        LOGGER_LIST.append("Rule is not met percent_real >=<  scc_per")
        row_total = ["TOTAL", minutes, basic_bill.get('not_zero_calls', 0), float_form % (cost)]
        table.append(row_total)
        return table
    if client.get("include_tax", False):
        tax = (IfNoneReturnZero(client.get("tax", 0)) * Decimal(cost + charge)) / Decimal(100)
        row_tax = ["Tax", "", "", float_form % (tax)]
        table.append(row_tax)
    else:
        tax = Decimal(0.00)
    print("Aplying surcharges to table")
    row_charge = ["Short Charges", "", num_calls, float_form % (charge)]
    row_total = ["TOTAL", "", "", float_form % (cost + charge + tax)]
    basic_bill['call_cost'] = cost + charge
    table.append(row_charge)
    table.append(row_total)
    end = timer()
    logger.debug("execution time  in milliseconds: %s" % ((end - start) * 1000))
    return table


def MakeDailyQueryTable(param_dict, client, float_form, day_usage):
    '''generates DAILY USAGE! table  '''
    #
    start = timer()
    logger.debug("Entering Make Daily Table")
    table = day_usage  # [["code Name", "Rate(US)", "Effective Date"]]
    float_form = param_dict.get('float_form', '%.2f')
    rate_float_form = param_dict.get('rate_float_form', '%.4f')
    start_date = param_dict.get('start_date')
    end_date = param_dict.get('end_date')
    client_id = param_dict.get('client_id')

    if start_date == None or end_date == None or client_id == None:
        print("Error! Must suply client_id,start_date and end_date!")
        return {}
    try:
        start_date_obj = datetime.datetime.strptime(start_date, "%Y-%m-%d %H:%M:%S").date()
        end_date_obj = datetime.datetime.strptime(end_date, "%Y-%m-%d %H:%M:%S").date()
        start_datetime_obj = datetime.datetime.strptime(start_date, "%Y-%m-%d %H:%M:%S")
        end_datetime_obj = datetime.datetime.strptime(end_date, "%Y-%m-%d %H:%M:%S")
    except Exception as e:
        print(str(e))
        ErrorInLogInvoice(str(e))
        logger.error(str(e))
        logger.debug("error in getting Daily query %s" % (str(e)))
        return table
    delta = end_date_obj - start_date_obj

    today = start_date_obj.strftime("%Y%02m%02d")
    # basic query
    basic_quer_part1 = """select date_trunc('day',report_time) as report_time ,sum(ingress_bill_time) as bill_time,sum(ingress_call_cost) as cost,sum(not_zero_calls) as calls,sum(ingress_call_cost_intra) as cost_intra,
sum(ingress_call_cost_inter) as cost_inter,sum(ingress_bill_time_intra) as bill_time_intra,sum(ingress_bill_time_inter) as bill_time_inter  from ( select  report_time,ingress_client_id,ingress_bill_time,ingress_call_cost,not_zero_calls,ingress_call_cost_intra,
ingress_call_cost_inter,ingress_bill_time_intra,ingress_bill_time_inter  from cdr_report_detail%s """ % (today)

    basic_quer_part3 = """) t where ingress_client_id = %s  and not_zero_calls>0 group by report_time ; """ % (
        client['client_id'])
    basic_make_query = basic_quer_part1
    result_dict = dict()
    result_list = list()
    result_dict[start_date_obj.strftime("%Y-%02m-%02d")] = [0, 0, 0, 0, 0, 0, 0,
                                                            0]  # result_dict is a dict keys every date from billing period
    result_list.append(start_date_obj.strftime("%Y-%02m-%02d"))
    for i in range(1, delta.days + 1):
        day = start_date_obj + datetime.timedelta(days=i)  # .replace("-","")
        result_dict[day.strftime("%Y-%02m-%02d")] = [0, 0, 0, 0, 0, 0, 0, 0]
        result_list.append(day.strftime("%Y-%02m-%02d"))
        day = day.strftime("%Y%02m%02d")
        basic_make_query += """UNION ALL    select report_time,ingress_client_id,ingress_bill_time,ingress_call_cost,not_zero_calls,ingress_call_cost_intra,
ingress_call_cost_inter,ingress_bill_time_intra,ingress_bill_time_inter  from cdr_report_detail%s  """ % (day)

    basic_make_query += basic_quer_part3

    print("DAILY  QUERY:", basic_make_query, result_dict)
    LOGGER_LIST.append("DAILY  QUERY:" + basic_make_query)
    logger.debug(basic_make_query)
    try:
        pg_cur.execute(basic_make_query)
        result = pg_cur.fetchall()

    except Exception as e:
        logger.error(" error in getting surcharge multi:  %s" % (str(e)))
        print("error in getting daily %s" % (str(e)))
        return table
    # print ("RESULT OF DAILY:" , result)
    total = def_dict(int)

    day_totals = def_dict(lambda: def_dict(float))
    for row in result:
        # result_dict [  str (row["report_time"].date() )  ] [idx] += row [idx]
        day = str(row["report_time"].date())
        # day_totals [ day  ] = str (row["report_time"].date() )
        day_totals[str(row["report_time"].date())]["cost"] += float(row["cost"])
        day_totals[str(row["report_time"].date())]["cost_intra"] += float(row["cost_intra"])
        day_totals[str(row["report_time"].date())]["cost_inter"] += float(row["cost_inter"])
        day_totals[str(row["report_time"].date())]["cost_ij"] += float(row["cost"]) - float(row["cost_inter"]) - float(
            row["cost_intra"])
        day_totals[str(row["report_time"].date())]["bill_time"] += float(row["bill_time"] / 60)
        day_totals[str(row["report_time"].date())]["bill_time_intra"] += float(row["bill_time_intra"] / 60)
        day_totals[str(row["report_time"].date())]["bill_time_inter"] += float(row["bill_time_inter"] / 60)
        day_totals[str(row["report_time"].date())]["bill_time_ij"] += float(row["bill_time"] / 60) - float(
            row["bill_time_inter"] / 60) - float(row["bill_time_intra"] / 60)
        # print ("row", row["report_time"], row["cost"],row  ["cost_intra"] , row  ["cost_inter"],row["bill_time"]/60 ,row  ["bill_time_intra"]/60 - row  ["bill_time_inter"]/60 )
        # day_val = result_dict [  str (row["report_time"].date() )  ]
        # day_totals [str (row["report_time"].date()] = day_val
        call_inted = row["cost"] - row["cost_intra"] - row["cost_inter"]
        total['call_inted'] += call_inted
        total['cost'] += row["cost"]
        bill_time_inted = row["bill_time"] / 60 - row["bill_time_intra"] / 60 - row["bill_time_inter"] / 60
        total['bill_time_inted'] += +float(bill_time_inted)  # float( day_val[2] )
        total['bill_time_intra'] += float(row["bill_time_intra"] / 60)
        total['bill_time_inter'] += float(row["bill_time_inter"] / 60)
        total['bill_time'] += float(row["bill_time"] / 60)  # float(day_val[3])+
        total['cost_intra'] += row["cost_intra"]
        total['cost_inter'] += row["cost_inter"]
    # print ("day_totals", str(day_totals))
    for key in day_totals.keys():
        row = day_totals[key]
        result_dict[key] = [float(row["bill_time_inter"]), float(row["cost_inter"]),
                            float(row["bill_time_intra"]), float(row["cost_intra"]),
                            float(row["bill_time_ij"]), float(row["cost_ij"]), float(row['bill_time']),
                            float(row["cost"])]  # day_val[6] +
    #  ,day_val[3] + row  ["bill_time"] / 60 	+ row  ["cost"]  ]
    # print ("result_dict: ", result_dict)

    for key in result_list:
        all_cost = result_dict[key][7]
        if all_cost != 0:
            row = result_dict[key]
            row.insert(0, key)
            table_row = []
            for elem in row:
                if not isinstance(elem, str):
                    elem = float_form % (elem)
                table_row.append(elem)
            table.append(table_row)
        else:
            print("Daily row was not added", all_cost, key)
    if len(table) > 2:
        table.append(["Total", float_form2 % (total['bill_time_inter']), float_form % (total['cost_inter']),
                      float_form2 % (total['bill_time_intra']), float_form % (total['cost_intra']),
                      float_form2 % (total['bill_time_inted']), float_form2 % (total['call_inted']),
                      float_form2 % (total['bill_time']), float_form % (total['cost'])])
    # print ("DAILY TABLE: ",table)
    end = timer()
    logger.debug("execution time  in milliseconds: %s" % ((end - start) * 1000))
    return table


def GetJurisdict(param_dict, client, acount_sumary, float_form):
    '''Getting juridct breakdown and making it table '''
    table = [["Code Name", " Rate", "Cost", "Rate", "Cost", "Rate", " Cost", "Rate", "Cost"]]

    #
    start = timer()
    logger.debug("Entering Get GEt Jursidiction")
    LOGGER_LIST.append("Entering Get GEt Jursidiction")
    float_form = param_dict.get('float_form', '%.2f')
    rate_float_form = param_dict.get('rate_float_form', '%.4f')
    start_date = param_dict.get('start_date')
    end_date = param_dict.get('end_date')
    client_id = param_dict.get('client_id')

    if start_date == None or end_date == None or client_id == None:
        print("Error! Must suply client_id,start_date and end_date!")
        return {}
    try:
        start_date_obj = datetime.datetime.strptime(start_date, "%Y-%m-%d %H:%M:%S").date()
        end_date_obj = datetime.datetime.strptime(end_date, "%Y-%m-%d %H:%M:%S").date()
        start_datetime_obj = datetime.datetime.strptime(start_date, "%Y-%m-%d %H:%M:%S")
        end_datetime_obj = datetime.datetime.strptime(end_date, "%Y-%m-%d %H:%M:%S")
    except Exception as e:
        print(str(e))
        logger.error(str(e))
        return table
    delta = end_date_obj - start_date_obj
    if delta.days < 0:  # single day
        # it cab be
        return table

    else:
        today = start_date_obj.strftime("%Y%02m%02d")
        # basic query
        basic_quer_part1 = """ select  ingress_code_name,sum(ingress_call_cost) as cost  , sum ( ingress_call_cost_intra   )
		as ingress_call_cost_intra   , sum(ingress_call_cost_inter ) as ingress_call_cost_inter
		, sum( ingress_bill_time_intra ) as ingress_bill_time_intra
		 ,sum(ingress_bill_time_inter) as ingress_bill_time_inter ,
		 sum ( ingress_bill_time)  as  ingress_bill_time ,sum (ingress_call_cost_ij ) as cost_ij  from  (    select 
		 ingress_code_name,ingress_call_cost  ,  ingress_bill_time,ingress_call_cost_intra , ingress_call_cost_inter 
		 , ingress_bill_time_intra ,ingress_bill_time_inter ,ingress_call_cost_ij 
    from    public.cdr_report_detail%s  where
 ingress_client_id =%s  and report_time  between '%s' and '%s' """ % (today, client_id, start_datetime_obj,
                                                                      end_datetime_obj)

        basic_quer_part3 = """ )as A  group by ingress_code_name ; """
        basic_make_query = basic_quer_part1

        for i in range(1, delta.days + 1):
            day = start_date_obj + datetime.timedelta(days=i)  # .replace("-","")
            day = day.strftime("%Y%02m%02d")
            basic_make_query += """UNION ALL    select 
		 ingress_code_name,ingress_call_cost, ingress_bill_time, ingress_call_cost_intra ,
		 ingress_call_cost_inter , ingress_bill_time_intra  ,ingress_bill_time_inter ,ingress_call_cost_ij 
		 from    public.cdr_report_detail%s  where
 ingress_client_id =%s and report_time  between '%s' and '%s' """ % (day, client_id,
                                                                     start_datetime_obj, end_datetime_obj)
        basic_make_query += basic_quer_part3
        print("JURISDICTION MULTIDAY QUERY:", basic_make_query)
        logger.debug("JURISDICTION MULTIDAY QUERY:" + basic_make_query)
        LOGGER_LIST.append("JURISDICTION MULTIDAY QUERY:" + basic_make_query)
        try:
            pg_cur.execute(basic_make_query)
            result = pg_cur.fetchall()
            print("JUR RESULT len ", len(result))
        except Exception as e:
            logger.error(" error in getting JURISDICTION MULTIDAY:  %s" % (str(e)))
            print("error in getting JURISDICTION MULTIDAY %s" % (str(e)))
            logger.debug("error in getting JURISDICTION MULTIDAY %s" % (str(e)))
            ErrorInLogInvoice(str(e))
            return table
    # print ("LEN of RESULT OF JURY:" , len (result))

    if len(result) == 0:
        print("No Jury Found")
        logger.debug("No Jury found")
        LOGGER_LIST.append("No Jury found")
        return table

    # ingress_code_name |    cost    | ingress_call_cost_intra | ingress_call_cost_inter | ngress_bill_time_intra
    # | ingress_bill_time_inter |    cost    | ingress_bill_time

    # print ("cp1")
    for row in result:
        # if not row.get ("ingress_code_name",False):
        #	continue
        ingress_code_name = row.get("ingress_code_name", "N/A")
        if ingress_code_name == None:
            print("ingress code name is None")
            continue  # ingress_code_name = "Null"
        cost = float(row.get("cost", 0))
        ingress_call_cost_inter = float(row.get("ingress_call_cost_inter", 0))
        ingress_call_cost_intra = float(row.get("ingress_call_cost_intra", 0))
        ingress_call_cost_ij = float(row.get("cost_ij", 0))
        ingress_bill_time = row.get("ingress_bill_time", 0) / 60
        ingress_bill_time_inter = row.get("ingress_bill_time_inter", 0) / 60
        ingress_bill_time_intra = row.get("ingress_bill_time_intra", 0) / 60
        # print (ingress_code_name,    cost  ,ingress_call_cost_intra, ingress_call_cost_inter,ingress_bill_time_intra ,ingress_bill_time_inter , ingress_bill_time  )
        # print ("cp2")
        try:
            ingress_rate = cost / ingress_bill_time
        except Exception as e:
            print("ingress_rate = cost  / ingress_bill_time", str(e))
            ingress_rate = 0

        try:
            ingress_rate_inter = ingress_call_cost_inter / ingress_bill_time_inter
        except Exception as e:
            ingress_rate_inter = 0
            print("ingress_rate = cost  / ingress_bill_tim_inter", str(e))

        try:
            ingress_rate_intra = ingress_call_cost_inter / ingress_bill_time_intra
        except Exception as e:
            ingress_rate_intra = 0
            print("ingress_rate = cost  / ingress_bill_tim_intra", str(e))

        ### Totals!
        # print ("cp3")
        total = def_dict(int)
        total['cost_inted'] += ingress_call_cost_ij
        total['cost'] += cost
        total['ingress_rate_inted'] = ""
        total['ingress_rate_intra'] = ""  # ingress_bill_time_intra
        total['ingress_rate_inter'] = ""  # ingress_bill_time_inter
        total['ingress_rate'] = ""  # ingress_rate
        total['cost_intra'] += ingress_call_cost_intra
        total['cost_inter'] += ingress_call_cost_inter
        # print ("cp4")

        table.append([ingress_code_name, rate_float_form % (ingress_rate_inter),
                      float_form % (ingress_call_cost_inter),
                      rate_float_form % (ingress_rate_intra),
                      float_form % (ingress_call_cost_intra),
                      rate_float_form % (ingress_rate), float_form % (ingress_call_cost_ij),
                      rate_float_form % (ingress_rate),
                      float_form % (cost)])

    if len(table) > 1:
        table.append(["Total", "", float_form % (total['cost_inter']), "", float_form % (total['cost_intra']), "",
                      float_form % (total['cost_inted']), "", float_form % (total['cost'])])

    print("jurisdiction_table len :", len(table))

    # float_form = "%0.2f"
    end = timer()
    logger.debug("execution time  in milliseconds: %s" % ((end - start) * 1000))
    return table


def MakeDayUsageHeader():
    '''Daily usage '''
    table = [["Date", "Interstate", "", "Intrastate", "", "IJ", "", "Total", ""],
             ["", "Minute", "Cost", "Minute", "Cost", "Minute", "Cost", "Minute", "Cost"]]

    return table


def Make_Ingress_Prefix_Jur(param_dict, client, float_form):
    # Summary By Trunk
    start = timer()

    start = timer()
    logger.debug("Entering Summary By Trunk Table Jur")
    LOGGER_LIST.append("Entering Summary By Trunk Table Jur")
    print("Summary By Trunk")
    table = [['', '', 'Interstate', '', '', 'Intrastate', '', '', 'IJ', '', ''],
             ["Trunk ", "Prefix", "Calls", "Duration", "Amount", "Calls", "Duration",
              "Amount", "Calls", "Duration", "Amount"]]
    float_form = param_dict.get('float_form', '%.2f')
    rate_float_form = param_dict.get('rate_float_form', '%.4f')
    start_date = param_dict.get('start_date')
    end_date = param_dict.get('end_date')
    client_id = param_dict.get('client_id')

    if start_date == None or end_date == None or client_id == None:
        print("Error! Must suply client_id,start_date and end_date!")
        return {}
    try:
        start_date_obj = datetime.datetime.strptime(start_date, "%Y-%m-%d %H:%M:%S").date()
        end_date_obj = datetime.datetime.strptime(end_date, "%Y-%m-%d %H:%M:%S").date()
        start_datetime_obj = datetime.datetime.strptime(start_date, "%Y-%m-%d %H:%M:%S")
        end_datetime_obj = datetime.datetime.strptime(end_date, "%Y-%m-%d %H:%M:%S")
    except Exception as e:
        print(str(e))
        logger.error(str(e))
        logger.debug("error in getting Show traffic country %s" % (str(e)))
        return table
    delta = end_date_obj - start_date_obj

    today = start_date_obj.strftime("%Y%02m%02d")

    basic_quer_part1 = """ select  alias,ingress_prefix,sum(ingress_bill_time) as bill_time ,sum(ingress_call_cost) as cost ,sum(not_zero_calls) as calls ,
	sum(ingress_bill_time_intra) as bill_time_intra ,sum(ingress_call_cost_intra) as cost_intra ,sum(intra_not_zero_calls) as intra_calls ,
	sum(ingress_bill_time_inter) as bill_time_inter ,sum(ingress_call_cost_inter) as cost_inter ,sum(inter_not_zero_calls) as inter_calls 
	 from  (    select alias,ingress_prefix, ingress_bill_time   , ingress_call_cost ,not_zero_calls ,
	 ingress_bill_time_intra  ,ingress_call_cost_intra  ,intra_not_zero_calls  , 
	 ingress_bill_time_inter  ,ingress_call_cost_inter  ,inter_not_zero_calls  
	 from    public.cdr_report_detail%s  right join   resource on  cdr_report_detail%s.ingress_id =  resource.resource_id  where
 ingress_client_id =%s  and report_time between '%s' and '%s' """ % (
        today, today, client_id, start_datetime_obj, end_datetime_obj)

    basic_quer_part3 = """ )as A group by alias,ingress_prefix ; """
    basic_make_query = basic_quer_part1

    # print ("today is ",today )
    for i in range(1, delta.days + 1):
        day = start_date_obj + datetime.timedelta(days=i)  # .replace("-","")
        day = day.strftime("%Y%02m%02d")
        basic_make_query += """UNION ALL   select alias,ingress_prefix, ingress_bill_time ,ingress_call_cost , not_zero_calls ,
	 ingress_bill_time_intra  ,ingress_call_cost_intra  ,intra_not_zero_calls  , 
	 ingress_bill_time_inter  ,ingress_call_cost_inter  ,inter_not_zero_calls  
  from   public.cdr_report_detail%s  right join   resource on  cdr_report_detail%s.ingress_id =  resource.resource_id  where
 ingress_client_id =%s and report_time between '%s' and '%s' """ % (
            day, day, client_id, start_datetime_obj, end_datetime_obj)
    basic_make_query += basic_quer_part3

    print("SUMARY BY INGRESS prefix:", basic_make_query)
    logger.debug("SUMARY BY INGRESS prefix:" + basic_make_query)
    LOGGER_LIST.append("SUMARY BY INGRESS prefix:" + basic_make_query)
    try:
        pg_cur.execute(basic_make_query)
        result = pg_cur.fetchall()

    except Exception as e:
        logger.error(" Error in getting sumary by prefix  %s" % (str(e)))
        print("error in getting sumary by prefixe %s" % (str(e)))
        logger.debug("error in getting sumary byprefix %s" % (str(e)))
        return table
    print("RESULT OF SUMARY BY INGRESS prefix:", result)
    LOGGER_LIST.append("RESULT OF SUMARY BY INGRESS prefix:" + str(result))
    # ingress_country| | ingress_rate | ingress_client_rate | bill_time | cost | calls | cost_intra | cost_inter | bill_time_intra | bill_time_inter
    # [['United States', 3, None, 0.001, 378, Decimal('0.006300'), 9, Decimal('0.000000'), Decimal('0.000000'), 0, 0], ['United States', 3, None, 10.0, 756, Decimal('0.012600'), 18, Decimal('0.000000'), Decimal('0.000000'), 0, 0]]
    total = def_dict(int)
    for row in result:  # ,
        if row.get('alias', None) == None:
            continue  # ingr_country = "Null"
        else:
            ingr_prefix = row.get('ingress_prefix', "Null")
            alias = row.get('alias', "Null")
        # if len (alias) > 9 :
        # 	alias =alias[0:6] + "..."
        total['intra_calls'] += row['intra_calls']
        total['bill_time_intra'] += row['bill_time_intra'] / 60
        total['cost_intra'] += row['cost_intra']
        total['inter_calls'] += row['inter_calls']
        total['bill_time_inter'] += row['bill_time_inter'] / 60
        total['cost_inter'] += row['cost_inter']
        total['ij_calls'] += row['calls'] - row['intra_calls'] - row['inter_calls']
        total['bill_time_ij'] += row['bill_time'] / 60 - row['bill_time_inter'] / 60 - row['bill_time_intra'] / 60
        total['cost_ij'] += row['cost'] - row['cost_intra'] - row['cost_inter']

        table.append([alias, ingr_prefix, row['intra_calls'], float_form2 % (row['bill_time_intra'] / 60),
                      float_form % (row['cost_intra']),
                      row['inter_calls'], float_form2 % (row['bill_time_inter'] / 60), float_form % (row['cost_inter']),
                      row['calls'] - row['intra_calls'] - row['inter_calls'],
                      float_form2 % (row['bill_time'] / 60 - row['bill_time_inter'] / 60 - row['bill_time_intra'] / 60),
                      float_form % (row['cost'] - row['cost_intra'] - row['cost_inter'])])

    if len(table) > 1:
        table.append(["Total", "", total['intra_calls'], float_form2 % (total['bill_time_intra']),
                      float_form % (total['cost_intra']),
                      total['inter_calls'], float_form2 % (total['bill_time_inter']),
                      float_form % (total['cost_inter']),
                      total['ij_calls'],
                      float_form2 % (total['bill_time_ij']), float_form % (total['cost_ij'])])
    end = timer()
    logger.debug("execution time  in milliseconds: %s" % ((end - start) * 1000))
    return table


def GetIngress_prefix(param_dict, client, float_form):
    # Summary By Trunk

    if param_dict.get('jurisdict', False):  # invoice_jurisdictional_detail
        prefix = Make_Ingress_Prefix_Jur(param_dict, client, float_form)
        return prefix

    start = timer()
    logger.debug("Entering Summary By Trunk Table")
    table = [["Trunk ", "Prefix", "Calls Total", "Duration", "Amount"]]
    start_date = param_dict.get('start_date')
    end_date = param_dict.get('end_date')
    client_id = param_dict.get('client_id')

    if start_date == None or end_date == None or client_id == None:
        print("Error! Must suply client_id,start_date and end_date!")
        return {}
    try:
        start_date_obj = datetime.datetime.strptime(start_date, "%Y-%m-%d %H:%M:%S").date()
        end_date_obj = datetime.datetime.strptime(end_date, "%Y-%m-%d %H:%M:%S").date()
        start_datetime_obj = datetime.datetime.strptime(start_date, "%Y-%m-%d %H:%M:%S")
        end_datetime_obj = datetime.datetime.strptime(end_date, "%Y-%m-%d %H:%M:%S")
    except Exception as e:
        print(str(e))
        logger.error(str(e))
        return table
    delta = end_date_obj - start_date_obj
    if delta.days < 0:  # single day Do not use !
        pass

    else:
        today = start_date_obj.strftime("%Y%02m%02d")

        # """ select case when ingress_prefix is null then alias else alias||'_'||ingress_prefix end as ingress_prefix,sum(ingress_bill_time) as bill_time ,sum(ingress_call_cost)
        basic_quer_part1 = """ select alias,ingress_prefix  ,sum(ingress_bill_time) as bill_time ,sum(ingress_call_cost) 
		as cost ,sum(not_zero_calls) as calls from  (    select ingress_prefix, alias,ingress_bill_time  ,not_zero_calls ,ingress_call_cost ,ingress_id from    public.cdr_report_detail%s  right join   resource on  cdr_report_detail%s.ingress_id =  resource.resource_id  where
 ingress_client_id =%s and  report_time between '%s' and '%s' """ % (
            today, today, client_id, start_datetime_obj, end_datetime_obj)

        basic_quer_part3 = """ )as A group by ingress_prefix,alias; """
        basic_make_query = basic_quer_part1

        for i in range(1, delta.days + 1):
            day = start_date_obj + datetime.timedelta(days=i)  # .replace("-","")
            day = day.strftime("%Y%02m%02d")
            basic_make_query += """UNION ALL   select 
ingress_prefix, alias,ingress_bill_time  ,not_zero_calls ,ingress_call_cost ,ingress_id 
  from    public.cdr_report_detail%s   right join     resource on  cdr_report_detail%s.ingress_id =  resource.resource_id   where
 ingress_client_id =%s and report_time between '%s' and '%s' """ % (
                day, day, client_id, start_datetime_obj, end_datetime_obj)
        basic_make_query += basic_quer_part3

        print("INGRESS PREFIX MULTIDAY QUERY:", basic_make_query)
        logger.debug("INGRESS PREFIX MULTIDAY QUERY:" + basic_make_query)
        LOGGER_LIST.append("INGRESS PREFIX MULTIDAY QUERY:" + basic_make_query)
        try:
            pg_cur.execute(basic_make_query)
            result = pg_cur.fetchall()

        except Exception as e:
            logger.error(" Error in getting ingress prefix:  %s" % (str(e)))
            print("error in getting ingress prefix %s" % (str(e)))
            logger.debug("error in getting ingress prefix %s" % (str(e)))
            return table
    print("LEN RESULT OF INGRESS PREFIX:", len(result))
    total = def_dict(int)
    for row in result:
        if row[
            'calls'] != 0:  # The Trunk that has 0 amount should not be shown. http://jira.denovolab.com/browse/CLAS-2510
            alias = row.get('alias', "N/A")
            # if len(alias) > 20:
            # 	alias = alias[0:18] + "..."
            total['calls'] += row['calls']
            total['bill_time'] += row['bill_time'] / 60
            total['cost'] += row['cost']
            table.append([alias, row.get('ingress_prefix', "N/A"), row['calls'], float_form2 % (row['bill_time'] / 60),
                          float_form % (row['cost'])])

    if len(table) > 1:
        table.append(["Total", "", total['calls'], float_form % (total['bill_time']), float_form % (total['cost'])])
    end = timer()
    logger.debug("execution time  in milliseconds: %s" % ((end - start) * 1000))
    return table


def MakeShowDetailByCodeName(param_dict, client, float_form):
    # last change from Jira CLAS-2764
    start = timer()
    logger.debug("Entering Detail by code name Table")
    table = [['Trunk Name', 'Duration', 'connected calls', 'Avg Rate', 'Cost']]
    float_form = param_dict.get('float_form', '%.2f')
    rate_float_form = param_dict.get('rate_float_form', '%.4f')
    start_date = param_dict.get('start_date')
    end_date = param_dict.get('end_date')
    client_id = param_dict.get('client_id')

    if start_date == None or end_date == None or client_id == None:
        print("Error! Must suply client_id,start_date and end_date!")
        return {}
    try:
        start_date_obj = datetime.datetime.strptime(start_date, "%Y-%m-%d %H:%M:%S").date()
        end_date_obj = datetime.datetime.strptime(end_date, "%Y-%m-%d %H:%M:%S").date()
        start_datetime_obj = datetime.datetime.strptime(start_date, "%Y-%m-%d %H:%M:%S")
        end_datetime_obj = datetime.datetime.strptime(end_date, "%Y-%m-%d %H:%M:%S")
    except Exception as e:
        print(str(e))
        logger.error(str(e))
        return table
    delta = end_date_obj - start_date_obj
    if delta.days < 0:
        return None


    else:
        today = start_date_obj.strftime("%Y%02m%02d")  # duration, connected calls, Avg Rate, cost
        basic_quer_part1 = """ select  ingress_code_name,sum(duration) as duration,sum(not_zero_calls) as calls, sum(ingress_call_cost) as cost 
      from  ( select ingress_code_name, alias,duration  ,not_zero_calls ,   ingress_call_cost ,ingress_id  
   from    public.cdr_report_detail%s  right join   
  resource on  cdr_report_detail%s.ingress_id =  resource.resource_id  where
 ingress_client_id =%s and  report_time between '%s' and '%s' """ % (
            today, today, client_id, start_datetime_obj, end_datetime_obj)

        basic_quer_part3 = """ )as A group by ingress_code_name,alias; """
        basic_make_query = basic_quer_part1

        for i in range(1, delta.days + 1):
            day = start_date_obj + datetime.timedelta(days=i)  # .replace("-","")
            day = day.strftime("%Y%02m%02d")
            basic_make_query += """UNION ALL  select ingress_code_name, alias,duration  ,not_zero_calls ,   ingress_call_cost ,ingress_id  
  from    public.cdr_report_detail%s   right join     resource on  cdr_report_detail%s.ingress_id =  resource.resource_id   where
 ingress_client_id =%s and report_time between '%s' and '%s' """ % (
                day, day, client_id, start_datetime_obj, end_datetime_obj)
        basic_make_query += basic_quer_part3

        print("INGRESS code_name DETAIL  QUERY:", basic_make_query)
        logger.debug("INGRESS code_name DETAIL QUERY:" + basic_make_query)
        try:
            pg_cur.execute(basic_make_query)
            result = pg_cur.fetchall()

        except Exception as e:
            logger.error(" Error in getting ingress code_name detail:  %s" % (str(e)))
            print("error in getting ingress code_name detail  %s" % (str(e)))
            logger.debug(" Error in getting ingress code_name detail :  %s" % (str(e)))
            return table
    print("INGRESS code_name DETAIL  QUERY Result:", result)

    total = def_dict(int)

    for row in result:
        if row.get('ingress_code_name', None) == None:
            continue
        # duration, connected calls, Avg Rate, cost
        total["duration"] += row.get("duration", 0) / 60
        total["calls"] += row.get("calls", 0)
        try:
            total["rate"] += row.get("cost", 0) / row.get("duration", 0) / 60
        except:
            total["rate"] += 0
        total["cost"] += row.get("cost", 0)
        try:
            rate = float(row.get("cost", 0)) / float(float(row.get("duration", 0)) / 60)
        except:
            rate = 0

        table.append(
            [row.get('ingress_code_name', "N/A"), float_form2 % (row.get("duration", 0) / 60), row.get("calls", 0),
             rate_float_form % (rate)
                , float_form % (row.get("cost", 0))])

    if len(table) > 1:
        table.append(["Total", float_form2 % (total["duration"]), total["calls"],
                      rate_float_form % (float(total["cost"]) / float(total["duration"])),
                      float_form % (total["cost"])])

    print("detail by code name table:", table)

    end = timer()
    logger.debug("execution time  in milliseconds: %s" % ((end - start) * 1000))
    return table


def Make_show_trafic_code_name_jur(param_dict, client, float_form):
    start = timer()
    logger.debug(" Make_show_trafic_code_name_jur")
    LOGGER_LIST.append(" Make_show_trafic_code_name_jur")
    print(" Make_show_trafic_code_name_jur")
    table = [['', 'Interstate', '', '', 'Intrastate', '', '', 'Intermediate', '', '', 'Total', '', ''],
             ["Code Name", "Calls", "Duration", "Amount", "Calls", "Duration",
              "Amount", "Calls", "Duration", "Amount", "Calls", "Duration", "Amount"]]
    float_form = param_dict.get('float_form', '%.2f')
    rate_float_form = param_dict.get('rate_float_form', '%.4f')
    start_date = param_dict.get('start_date')
    end_date = param_dict.get('end_date')
    client_id = param_dict.get('client_id')

    if start_date == None or end_date == None or client_id == None:
        print("Error! Must suply client_id,start_date and end_date!")
        return {}
    try:
        start_date_obj = datetime.datetime.strptime(start_date, "%Y-%m-%d %H:%M:%S").date()
        end_date_obj = datetime.datetime.strptime(end_date, "%Y-%m-%d %H:%M:%S").date()
        start_datetime_obj = datetime.datetime.strptime(start_date, "%Y-%m-%d %H:%M:%S")
        end_datetime_obj = datetime.datetime.strptime(end_date, "%Y-%m-%d %H:%M:%S")
    except Exception as e:
        print(str(e))
        logger.error(str(e))
        logger.error("error in getting  Make_show_trafic_code_name_jur%s" % (str(e)))
        return table
    delta = end_date_obj - start_date_obj

    today = start_date_obj.strftime("%Y%02m%02d")

    basic_quer_part1 = """ select  ingress_code_name,sum(ingress_bill_time) as bill_time ,sum(ingress_call_cost) as cost ,sum(not_zero_calls) as calls ,
	sum(ingress_bill_time_intra) as bill_time_intra ,sum(ingress_call_cost_intra) as cost_intra ,sum(intra_not_zero_calls) as intra_calls ,
	sum(ingress_bill_time_inter) as bill_time_inter ,sum(ingress_call_cost_inter) as cost_inter ,sum(inter_not_zero_calls) as inter_calls 
	 from  (    select ingress_code_name, ingress_bill_time   , ingress_call_cost ,not_zero_calls ,
	 ingress_bill_time_intra  ,ingress_call_cost_intra  ,intra_not_zero_calls  , 
	 ingress_bill_time_inter  ,ingress_call_cost_inter  ,inter_not_zero_calls  
	 from    cdr_report_detail%s  where
 ingress_client_id =%s  and report_time between '%s' and '%s' """ % (
        today, client_id, start_datetime_obj, end_datetime_obj)

    basic_quer_part3 = """ )as A group by ingress_code_name; """
    basic_make_query = basic_quer_part1

    for i in range(1, delta.days + 1):
        day = start_date_obj + datetime.timedelta(days=i)  # .replace("-","")
        day = day.strftime("%Y%02m%02d")
        basic_make_query += """UNION ALL   select ingress_code_name, ingress_bill_time ,ingress_call_cost , not_zero_calls ,
	 ingress_bill_time_intra  ,ingress_call_cost_intra  ,intra_not_zero_calls  , 
	 ingress_bill_time_inter  ,ingress_call_cost_inter  ,inter_not_zero_calls  
  from   cdr_report_detail%s  where
 ingress_client_id =%s and report_time between '%s' and '%s' """ % (
            day, client_id, start_datetime_obj, end_datetime_obj)
    basic_make_query += basic_quer_part3

    print("SUMARY BY  Make_show_trafic_code_name_jur:", basic_make_query)
    logger.debug("SUMARY BY  Make_show_trafic_code_name_jur:" + basic_make_query)
    LOGGER_LIST.append("SUMARY BY  Make_show_trafic_code_name_jur:" + basic_make_query)
    try:
        pg_cur.execute(basic_make_query)
        result = pg_cur.fetchall()

    except Exception as e:
        logger.error(" Error in getting sumary by code name:  %s" % (str(e)))
        print("error in getting sumary by code name %s" % (str(e)))
        return table
    print("RESULT OF SUMARY BY INGRESS CODE JUR len :", len(result))
    # ingress_country| | ingress_rate | ingress_client_rate | bill_time | cost | calls | cost_intra | cost_inter | bill_time_intra | bill_time_inter
    # [['United States', 3, None, 0.001, 378, Decimal('0.006300'), 9, Decimal('0.000000'), Decimal('0.000000'), 0, 0], ['United States', 3, None, 10.0, 756, Decimal('0.012600'), 18, Decimal('0.000000'), Decimal('0.000000'), 0, 0]]

    total = def_dict(int)
    for row in result:
        # print ("New Ingress row")
        if row.get('ingress_code_name', None) == None:
            continue  # ingr_country = "Null"
        else:
            ingress_code_name = row.get('ingress_code_name', "Null")
            if len(ingress_code_name) > 10:
                ingress_code_name = ingress_code_name[:8] + "..."

        total["intra_calls"] += row['intra_calls']
        total["bill_time_intra"] += row['bill_time_intra'] / 60
        total["cost_intra"] += row['cost_intra']

        total["inter_calls"] += row['inter_calls']
        total["bill_time_inter"] += row['bill_time_inter'] / 60
        total["cost_inter"] += row['cost_inter']

        total["ij_calls"] += row['calls'] - row['intra_calls'] - row['inter_calls']
        total["bill_time_ij"] += row['bill_time'] / 60 - row['bill_time_inter'] / 60 - row['bill_time_intra'] / 60
        total["cost_ij"] += row['cost'] - row['cost_intra'] - row['cost_inter']

        total["calls"] += row['calls']
        total["bill_time"] += row['bill_time'] / 60
        total["cost"] += row['cost']
        # print ("New Ingress row aftet totals ")
        try:
            table.append([ingress_code_name, row['intra_calls'], float_form0 % (row['bill_time_intra'] / 60),
                          float_form % (row['cost_intra']),
                          row['inter_calls'], float_form0 % (row['bill_time_inter'] / 60),
                          float_form % (row['cost_inter']),
                          row['calls'] - row['intra_calls'] - row['inter_calls'],
                          float_form0 % (row['bill_time'] / 60 - row['bill_time_inter'] / 60 - row[
                              'bill_time_intra'] / 60),
                          float_form % (row['cost'] - row['cost_intra'] - row['cost_inter']),
                          row['calls'], float_form0 % (row['bill_time'] / 60), float_form % (row['cost'])])
        except Exception as e:
            print("Error in Make_show_trafic_code_name_jur table generation", e)

    if len(table) > 1:
        try:
            table.append(["Total", total["intra_calls"], float_form0 % (total["bill_time_intra"]),
                          float_form % (total["cost_intra"]),
                          total["inter_calls"], float_form0 % (total["bill_time_inter"]),
                          float_form % (total["cost_inter"]),
                          total["ij_calls"], float_form0 % (total["bill_time_ij"]), float_form % (total["cost_ij"]),
                          float_form0 % (total["calls"]), float_form0 % (total["bill_time"]),
                          float_form % (total["cost"])])
        except:
            pass
    end = timer()
    print("Make_show_trafic_code_name_jur table len", len(table))
    print("execution time of ingress_ code name in milliseconds: %s" % ((end - start) * 1000))
    logger.debug("execution time  in milliseconds: %s" % ((end - start) * 1000))
    return table


# Code Name/country : Total Calls : Total minutes : Total Cost/Amount
def Make_show_trafic_code_name(param_dict, client, float_form):
    if param_dict.get('jurisdict', False):  # invoice_jurisdictional_detail
        jurisdict = Make_show_trafic_code_name_jur(param_dict, client, float_form)
        return jurisdict
    start = timer()
    logger.debug("Entering Show Traffic Code Name Table")
    float_form = param_dict.get('float_form', '%.2f')
    rate_float_form = param_dict.get('rate_float_form', '%.4f')
    table = [["Code Name", "Calls Total", "Duration", "Amount"]]
    start_date = param_dict.get('start_date')
    end_date = param_dict.get('end_date')
    client_id = param_dict.get('client_id')

    if start_date == None or end_date == None or client_id == None:
        print("Error! Must suply client_id,start_date and end_date!")
        return {}
    try:
        start_date_obj = datetime.datetime.strptime(start_date, "%Y-%m-%d %H:%M:%S").date()
        end_date_obj = datetime.datetime.strptime(end_date, "%Y-%m-%d %H:%M:%S").date()
        start_datetime_obj = datetime.datetime.strptime(start_date, "%Y-%m-%d %H:%M:%S")
        end_datetime_obj = datetime.datetime.strptime(end_date, "%Y-%m-%d %H:%M:%S")
    except Exception as e:
        print(str(e))
        logger.error(str(e))
        return table
    delta = end_date_obj - start_date_obj
    if delta.days < 0:  # single day
        today = end_date_obj.strftime("%Y%02m%02d")
        # WARNING. PROBABLY SUM OF INTER COST AND INTRA COST ARE NOT NEEEDED!
        daily_query_basic = """ select ingress_code_name,sum(ingress_bill_time) as bill_time,sum(ingress_call_cost) as cost,sum(not_zero_calls) as calls  from cdr_report_detail%s left join public.cdr_report_detail%s on cdr_report_detail%s.ingress_client_id =cdr_report_detail%s.ingress_client_id where
 cdr_report_detail%s.ingress_client_id =%s and duration > 0 and report_time between '%s' and '%s'  group by ingress_code_name;
 """ % (today, today, today, today, today, client_id, start_datetime_obj, end_datetime_obj)

        print("SUMARY BY CODE NAME DAILY QUERY: ", daily_query_basic)
        logger.debug("SUMARY BY CODE NAME DAILY QUERY: " + daily_query_basic)
        try:
            pg_cur.execute(daily_query_basic)
            result = pg_cur.fetchall()
        except Exception as e:
            logger.error(" error in getting ingress prefix for one day:  %s" % (str(e)))
            print("error in getting ingress prefix for one day %s" % (str(e)))
            return table


    else:
        today = start_date_obj.strftime("%Y%02m%02d")
        # basic query
        basic_quer_part1 = """ select  ingress_code_name,sum(ingress_bill_time) as bill_time ,sum(ingress_call_cost) as cost ,sum(not_zero_calls) as calls from  (    select ingress_code_name,
		 ingress_bill_time   ,ingress_call_cost ,not_zero_calls from    cdr_report_detail%s  where
 ingress_client_id =%s and report_time between '%s' and '%s' """ % (today, client_id, start_datetime_obj,
                                                                    end_datetime_obj)

        basic_quer_part3 = """ )as A group by ingress_code_name; """
        basic_make_query = basic_quer_part1

        for i in range(1, delta.days + 1):
            day = start_date_obj + datetime.timedelta(days=i)  # .replace("-","")
            day = day.strftime("%Y%02m%02d")
            basic_make_query += """UNION ALL   select ingress_code_name, ingress_bill_time ,ingress_call_cost , not_zero_calls 
  from   cdr_report_detail%s where  ingress_client_id =%s and report_time between '%s' and '%s' """ % (day
                                                                                                                                       ,
                                                                                                       client_id,
                                                                                                       start_datetime_obj,
                                                                                                       end_datetime_obj)
        basic_make_query += basic_quer_part3

        print("SUMARY BY CODE NAME MULTIDAY QUERY:", basic_make_query)
        LOGGER_LIST.append("SUMARY BY CODE NAME MULTIDAY QUERY:" + str(basic_make_query))

        try:
            pg_cur.execute(basic_make_query)
            result = pg_cur.fetchall()

        except Exception as e:
            logger.error(" Error in getting sumary by code name:  %s" % (str(e)))
            print("error in getting sumary by code name %s" % (str(e)))
            logger.debug("error in getting sumary by code name %s" % (str(e)))
            return table

    total = def_dict(int)
    for row in result:
        if row.get('ingress_code_name', None) == None:
            continue  # ingress_code = "Null"
        else:
            ingress_code = row.get('ingress_code_name', "Null")

        total["calls"] += row['calls']
        total["bill_time"] += row['bill_time'] / 60
        total["cost"] += row['cost']
        table.append([ingress_code, row['calls'], float_form2 % (row['bill_time'] / 60), float_form % (row['cost'])])
    if len(table) > 1:
        table.append(
            ["Total", float_form % (total["intra"]), float_form2 % (total["bill_time"]), float_form % (total["cost"])])
    end = timer()
    logger.debug("execution time  in milliseconds: %s" % ((end - start) * 1000))
    return table


def Make_show_trafic_country_Jur(param_dict, client, float_form):
    #
    start = timer()
    logger.debug("Show Traffic Country_Jur")
    LOGGER_LIST.append("Show Traffic Country_Jur")
    print("Show Traffic Country_Jur")
    table = [['', 'Interstate', '', '', 'Intrastate', '', '', 'Intermediate', '', '', 'Total''', '', ],
             ["Country", "Calls", "Duration", "Amount", "Calls", "Duration",
              "Amount", "Calls", "Duration", "Amount", "Calls", "Duration", "Amount"]]
    float_form = param_dict.get('float_form', '%.2f')
    rate_float_form = param_dict.get('rate_float_form', '%.4f')
    start_date = param_dict.get('start_date')
    end_date = param_dict.get('end_date')
    client_id = param_dict.get('client_id')

    if start_date == None or end_date == None or client_id == None:
        print("Error! Must suply client_id,start_date and end_date!")
        return {}
    try:
        start_date_obj = datetime.datetime.strptime(start_date, "%Y-%m-%d %H:%M:%S").date()
        end_date_obj = datetime.datetime.strptime(end_date, "%Y-%m-%d %H:%M:%S").date()
        start_datetime_obj = datetime.datetime.strptime(start_date, "%Y-%m-%d %H:%M:%S")
        end_datetime_obj = datetime.datetime.strptime(end_date, "%Y-%m-%d %H:%M:%S")
    except Exception as e:
        print(str(e))
        logger.error(str(e))
        logger.debug("error in getting Show traffic country %s" % (str(e)))
        return table
    delta = end_date_obj - start_date_obj

    today = start_date_obj.strftime("%Y%02m%02d")

    basic_quer_part1 = """ select  ingress_country,sum(ingress_bill_time) as bill_time ,sum(ingress_call_cost) as cost ,sum(not_zero_calls) as calls ,
	sum(ingress_bill_time_intra) as bill_time_intra ,sum(ingress_call_cost_intra) as cost_intra ,sum(intra_not_zero_calls) as intra_calls ,
	sum(ingress_bill_time_inter) as bill_time_inter ,sum(ingress_call_cost_inter) as cost_inter ,sum(inter_not_zero_calls) as inter_calls 
	 from  (    select ingress_country, ingress_bill_time   , ingress_call_cost ,not_zero_calls ,
	 ingress_bill_time_intra  ,ingress_call_cost_intra  ,intra_not_zero_calls  , 
	 ingress_bill_time_inter  ,ingress_call_cost_inter  ,inter_not_zero_calls  
	 from    cdr_report_detail%s  where
 ingress_client_id =%s  and report_time between '%s' and '%s' """ % (
        today, client_id, start_datetime_obj, end_datetime_obj)

    basic_quer_part3 = """ )as A group by ingress_country; """
    basic_make_query = basic_quer_part1

    # print ("today is ",today )
    for i in range(1, delta.days + 1):
        day = start_date_obj + datetime.timedelta(days=i)  # .replace("-","")
        day = day.strftime("%Y%02m%02d")
        basic_make_query += """UNION ALL   select ingress_country, ingress_bill_time ,ingress_call_cost , not_zero_calls ,
	 ingress_bill_time_intra  ,ingress_call_cost_intra  ,intra_not_zero_calls  , 
	 ingress_bill_time_inter  ,ingress_call_cost_inter  ,inter_not_zero_calls  
  from   cdr_report_detail%s  where
 ingress_client_id =%s and report_time between '%s' and '%s' """ % (
            day, client_id, start_datetime_obj, end_datetime_obj)
    basic_make_query += basic_quer_part3

    print("SUMARY BY INGRESS COUNTRY:", basic_make_query)
    logger.debug("SUMARY BY INGRESS COUNTRY:" + basic_make_query)
    LOGGER_LIST.append("SUMARY BY INGRESS COUNTRY:" + basic_make_query)
    try:
        pg_cur.execute(basic_make_query)
        result = pg_cur.fetchall()

    except Exception as e:
        logger.error(" Error in getting sumary by code name:  %s" % (str(e)))
        print("error in getting sumary by code name %s" % (str(e)))
        logger.debug("error in getting sumary by code name %s" % (str(e)))
        return table
    print("RESULT OF SUMARY BY INGRESS COUNTRY JUR:", result)

    total = def_dict(int)
    for row in result:
        if row.get('ingress_country', None) == None:
            print("skipped!")
            continue  # ingr_country = "Null"
        else:
            ingr_country = row.get('ingress_country', "Null")
            if len(ingr_country) > 11:
                ingr_country = ingr_country[:9] + "..."

        total["intra_calls"] += row['intra_calls']
        total["bill_time_intra"] += row['bill_time_intra'] / 60
        total["cost_intra"] += row['cost_intra']

        total["inter_calls"] += row['inter_calls']
        total["bill_time_inter"] += row['bill_time_inter'] / 60
        total["cost_inter"] += row['cost_inter']

        total["ij_calls"] += row['calls'] - row['intra_calls'] - row['inter_calls']
        total["bill_time_ij"] += row['bill_time'] / 60 - row['bill_time_inter'] / 60 - row['bill_time_intra'] / 60
        total["cost_ij"] += row['cost'] - row['cost_intra'] - row['cost_inter']

        total["calls"] += row['calls']
        total["bill_time"] += row['bill_time'] / 60
        total["cost"] += row['cost']

        table.append([ingr_country, row['intra_calls'], float_form0 % (row['bill_time_intra'] / 60),
                      float_form % (row['cost_intra']),
                      row['inter_calls'], float_form0 % (row['bill_time_inter'] / 60), float_form % (row['cost_inter']),
                      row['calls'] - row['intra_calls'] - row['inter_calls'],
                      float_form0 % (row['bill_time'] / 60 - row['bill_time_inter'] / 60 - row['bill_time_intra'] / 60),
                      float_form % (row['cost'] - row['cost_intra'] - row['cost_inter']),
                      row['calls'], float_form0 % (row['bill_time'] / 60), float_form % (row['cost'])])
    if len(table) > 1:
        table.append(
            ["Total", total["intra_calls"], float_form0 % (total["bill_time_intra"]),
             float_form % (total["cost_intra"]),
             total["inter_calls"], float_form0 % (total["bill_time_inter"]), float_form % (total["cost_inter"]),
             total["ij_calls"], float_form0 % (total["bill_time_ij"]), float_form % (total["cost_ij"]),
             float_form0 % (total["calls"]), float_form0 % (total["bill_time"]), float_form % (total["cost"])])

    end = timer()
    print("len of ingress country jur", len(table))
    logger.debug("execution time  in milliseconds: %s" % ((end - start) * 1000))
    return table


def Make_show_trafic_country(param_dict, client, float_form):
    #
    if param_dict.get('jurisdict', False):  # invoice_jurisdictional_detail
        jurisdict = Make_show_trafic_country_Jur(param_dict, client, float_form)
        return jurisdict

    start = timer()
    logger.debug("Show Traffic Country")
    LOGGER_LIST.append("Show Traffic Country")
    table = [["Country", "Calls Total", "Duration", "Amount"]]
    float_form = param_dict.get('float_form', '%.2f')
    rate_float_form = param_dict.get('rate_float_form', '%.4f')
    start_date = param_dict.get('start_date')
    end_date = param_dict.get('end_date')
    client_id = param_dict.get('client_id')

    if start_date == None or end_date == None or client_id == None:
        print("Error! Must suply client_id,start_date and end_date!")
        return {}
    try:
        start_date_obj = datetime.datetime.strptime(start_date, "%Y-%m-%d %H:%M:%S").date()
        end_date_obj = datetime.datetime.strptime(end_date, "%Y-%m-%d %H:%M:%S").date()
        start_datetime_obj = datetime.datetime.strptime(start_date, "%Y-%m-%d %H:%M:%S")
        end_datetime_obj = datetime.datetime.strptime(end_date, "%Y-%m-%d %H:%M:%S")
    except Exception as e:
        print(str(e))
        logger.error(str(e))
        logger.debug("error in getting Show traffic country %s" % (str(e)))
        return table
    delta = end_date_obj - start_date_obj
    if delta.days < 0:  # single day
        pass


    else:
        today = start_date_obj.strftime("%Y%02m%02d")
        # basic query
        basic_quer_part1 = """ select  ingress_country,sum(ingress_bill_time) as bill_time ,sum(ingress_call_cost) as cost ,sum(not_zero_calls) as calls from  (    select ingress_country,
		 ingress_bill_time   ,ingress_call_cost ,not_zero_calls from    cdr_report_detail%s  where
 ingress_client_id =%s  and report_time between '%s' and '%s' """ % (today, client_id, start_datetime_obj,
                                                                     end_datetime_obj)

        basic_quer_part3 = """ )as A group by ingress_country; """
        basic_make_query = basic_quer_part1

        # print ("today is ",today )
        for i in range(1, delta.days + 1):
            day = start_date_obj + datetime.timedelta(days=i)  # .replace("-","")
            day = day.strftime("%Y%02m%02d")
            basic_make_query += """UNION ALL   select ingress_country, ingress_bill_time ,ingress_call_cost , not_zero_calls 
  from   cdr_report_detail%s  where
 ingress_client_id =%s and report_time between '%s' and '%s' """ % (
                day, client_id, start_datetime_obj, end_datetime_obj)
        basic_make_query += basic_quer_part3

        print("SUMARY BY INGRESS COUNTRY:", basic_make_query)
        logger.debug("SUMARY BY INGRESS COUNTRY:" + basic_make_query)
        LOGGER_LIST.append("SUMARY BY INGRESS COUNTRY:" + basic_make_query)
        try:
            pg_cur.execute(basic_make_query)
            result = pg_cur.fetchall()

        except Exception as e:
            logger.error(" Error in getting sumary by code name:  %s" % (str(e)))
            print("error in getting sumary by code name %s" % (str(e)))
            logger.debug("error in getting sumary by code name %s" % (str(e)))
            return table
    # print ("RESULT OF SUMARY BY INGRESS COUNTRY:" , result)
    # ingress_code_name | ingress_rate_type | ingress_rate | ingress_client_rate | bill_time | cost | calls | cost_intra | cost_inter | bill_time_intra | bill_time_inter
    # [['United States', 3, None, 0.001, 378, Decimal('0.006300'), 9, Decimal('0.000000'), Decimal('0.000000'), 0, 0], ['United States', 3, None, 10.0, 756, Decimal('0.012600'), 18, Decimal('0.000000'), Decimal('0.000000'), 0, 0]]

    total = def_dict(int)

    for row in result:
        if row.get('ingress_country', None) == None:
            continue  # ingr_country = "Null"
        else:
            ingr_country = row.get('ingress_country', "Null")
        total["calls"] += row['calls']
        total["bill_time"] += row['bill_time'] / 60
        total["cost"] += row['cost']
        table.append([ingr_country, row['calls'], float_form2 % (row['bill_time'] / 60), float_form % (row['cost'])])
    if len(table) > 1:
        table.append(
            ["Total", float_form % (total["calls"]), float_form2 % (total["bill_time"]), float_form % (total["cost"])])
    end = timer()
    logger.debug("execution time  in milliseconds: %s" % ((end - start) * 1000))
    return table


def Make_show_code_summery(param_dict, client, float_form):
    '''Likewise, "Top ten codes" will show the cost of the top ten destination (codes)'''
    #
    start = timer()
    logger.debug("Entering TOP 10 code summery")
    LOGGER_LIST.append("Entering TOP 10 code summery")
    print("Entering TOP 10 code summery")
    float_form = param_dict.get('float_form', '%.2f')
    rate_float_form = param_dict.get('rate_float_form', '%.4f')
    table = [["Code", "Calls", "Duration", "Amount"]]
    start_date = param_dict.get('start_date')
    end_date = param_dict.get('end_date')
    client_id = param_dict.get('client_id')
    try:
        sql = """select resource_id from resource where ingress is true and client_id = %s""" % client_id
        pg_cur.execute(sql)
        result = pg_cur.fetchall()
    except Exception as e:
        logger.error(" error in getting ingress_id in  Make_show_code_summery():  %s" % (str(e)))
        print("error in getting ingress_id in  Make_show_code_summery(): %s" % (str(e)))
        return table
    if start_date == None or end_date == None or client_id == None:
        print("Error! Must suply client_id,start_date and end_date!")
        return {}
    try:
        start_datetime_obj = datetime.datetime.strptime(start_date, "%Y-%m-%d %H:%M:%S")
        end_datetime_obj = datetime.datetime.strptime(end_date, "%Y-%m-%d %H:%M:%S")
    except Exception as e:
        print("error in Make_show_code_summery() converting dates  %s" % (str(e)))
        logger.error("error in Make_show_code_summery() converting dates  %s" % (str(e)))
        return table
    try:
        url = config['cdr2']["url"]
    except:
        url = "http://192.99.10.113:8889"
    result_list = []
    try:
        for row in result:
            res = get_cdr_records.ShowCodeSummery(start_datetime_obj, end_datetime_obj, row.get('resource_id', None),
                                                  url)
            result_list.extend(res)
    except:
        logger.error("CDR api not available")
        print("CDR api not available")
        return None

    total = def_dict(int)

    for row in result_list:

        if row.get('orig_code', "NULL") != "NULL":
            total["calls"] += row['calls']
            total["bill_time"] += row.get('ingress_client_bill_time', 0) / 60
            total["cost"] += row.get('ingress_client_cost', 0)
            table.append([row.get('orig_code', "N/A"), row.get('calls', "N/A"),
                          float_form2 % (row.get('ingress_client_bill_time', -1) / 60),
                          float_form % (row.get('ingress_client_cost', "-1"))])
    table.append(["Total", total["calls"], float_form2 % (total["bill_time"]), float_form % (total["cost"])])
    end = timer()
    print("Make_show_code_summery TABLE:", table)
    logger.debug("execution time  in milliseconds: %s" % ((end - start) * 1000))
    return table


def MakeUsage(param_dict, client, float_form):
    # Usage Detail
    start = timer()
    logger.debug("Entering Make Usage Detail Table")
    LOGGER_LIST.append("Entering Make Usage Detail Table")
    table = [["Code Name", "Completed Calls", "Total Minutes", "Rate(US)", " Effective Date", "Total Charges(US)"]]
    float_form = param_dict.get('float_form', '%.2f')
    rate_float_form = param_dict.get('rate_float_form', '%.4f')
    start_date = param_dict.get('start_date')
    end_date = param_dict.get('end_date')
    client_id = param_dict.get('client_id')
    results_dict = {}
    if start_date == None or end_date == None or client_id == None:
        print("Error! Must suply client_id,start_date and end_date!")
        logger.debug("  Error! Must suply client_id,start_date and end_date!")
        return {}
    try:
        start_date_obj = datetime.datetime.strptime(start_date, "%Y-%m-%d %H:%M:%S").date()
        end_date_obj = datetime.datetime.strptime(end_date, "%Y-%m-%d %H:%M:%S").date()
        start_datetime_obj = datetime.datetime.strptime(start_date, "%Y-%m-%d %H:%M:%S")
        end_datetime_obj = datetime.datetime.strptime(end_date, "%Y-%m-%d %H:%M:%S")
    except Exception as e:
        print(str(e))
        logger.error(str(e))
        logger.debug("  Error in getting USAGE DETAIL :  %s" % (str(e)))
        return table
    delta = end_date_obj - start_date_obj
    if delta.days < 0:  # single day
        # it cab be
        return table


    else:
        today = start_date_obj.strftime("%Y%02m%02d")
        # basic query
        basic_quer_part1 = """ select ingress_code_name as Code_Name,sum (not_zero_calls) as Completed_Calls, round(sum ( ingress_bill_time )/60::numeric,2) as Total_Minutes ,
		  round(sum (ingress_call_cost) /  round(sum ( ingress_bill_time )/60::numeric,2)::numeric,5) as Rate, to_timestamp (ingress_rate_date)as Effective_Date,
		  round(sum(ingress_call_cost),2) as Total_Charges
		  from( select ingress_code_name,not_zero_calls, ingress_bill_time,ingress_call_cost,ingress_rate_date 
		   from cdr_report_detail%s where ingress_client_id =%s and  duration>0   """ % (today, client_id)

        basic_quer_part3 = """ )as A  group by   ingress_code_name , to_timestamp (ingress_rate_date) ; """
        basic_make_query = basic_quer_part1

        # left join public.cdr_report_detail%s on cdr_report_detail%s.ingress_client_id =cdr_report_detail%s.ingress_client_id
        for i in range(1, delta.days + 1):
            day = start_date_obj + datetime.timedelta(days=i)  # .replace("-","")
            day = day.strftime("%Y%02m%02d")
            basic_make_query += """UNION ALL    select ingress_code_name,not_zero_calls, ingress_bill_time,round(ingress_call_cost,2) as ingress_call_cost,ingress_rate_date 
		   from cdr_report_detail%s where ingress_client_id =%s and  duration>0 """ % (day, client_id)
        basic_make_query += basic_quer_part3

        print("USAGE DETAIL QUERY:", basic_make_query)
        logger.debug("USAGE DETAIL QUERY:" + basic_make_query)
        LOGGER_LIST.append("USAGE DETAIL QUERY:" + basic_make_query)
        try:
            pg_cur.execute(basic_make_query)
            result = pg_cur.fetchall()

        except Exception as e:
            logger.error(" error in getting USAGE DETAIL :  %s" % (str(e)))
            print("error in getting USAGE DETAIL  %s" % (str(e)))
            logger.debug("  Error in getting USAGE DETAIL :  %s" % (str(e)))
            return table
    print("LEN of RESULT OF USAGE DETAIL :", len(result))
    # print (result)
    jury_dict = {}

    if len(result) == 0:
        print("No Usage Detail Found")
        return table
    # code_name | completed_calls | total_minutes | rate | effective_date | total_charges

    for row in result:
        code_name = row.get("code_name", "Null")
        completed_calls = row.get("completed_calls", "Null")
        total_minutes = row.get("total_minutes", "Null")
        rate = rate_float_form % row.get("rate", 0.0)
        try:
            effective_date = row.get("effective_date").strftime("%Y-%m-%d %H:%M:%S")
        except Exception as e:
            # print(e, type(row.get("effective_date","Null+")))
            effective_date = row.get("effective_date", "Null+")
        total_charges = float_form % (row.get("total_charges", 0))
        try:
            table.append([code_name, completed_calls, total_minutes, rate, effective_date, total_charges])
        except Exception as e:
            print("Error in creating table for usage detail", e)
    end = timer()
    logger.debug("execution time  in milliseconds: %s" % ((end - start) * 1000))
    return table


def detail_by_trunk_jur(param_dict, client, float_form):
    start = timer()
    logger.debug("  detail_by_trunk_jur")
    LOGGER_LIST.append("  detail_by_trunk_jur")
    print("  detail_by_trunk_jur")
    table = [['', 'Interstate', '', '', 'Intrastate', '', '', 'IJ', '', '', 'Total''', '', ]
        , ["Trunk Name", "Calls", "Duration", "Avg Rate", "Cost", "Calls", "Duration", "Avg Rate", "Cost", "Calls",
           "Duration", "Avg Rate", "Cost"]]
    start_date = param_dict.get('start_date')
    end_date = param_dict.get('end_date')
    client_id = param_dict.get('client_id')

    if start_date == None or end_date == None or client_id == None:
        print("Error! Must suply client_id,start_date and end_date!")
        return {}
    try:
        start_date_obj = datetime.datetime.strptime(start_date, "%Y-%m-%d %H:%M:%S").date()
        end_date_obj = datetime.datetime.strptime(end_date, "%Y-%m-%d %H:%M:%S").date()
        start_datetime_obj = datetime.datetime.strptime(start_date, "%Y-%m-%d %H:%M:%S")
        end_datetime_obj = datetime.datetime.strptime(end_date, "%Y-%m-%d %H:%M:%S")
    except Exception as e:
        print(str(e))
        logger.error(str(e))
        logger.error("error in getting  Make_show_trafic_code_name_jur%s" % (str(e)))
        return table
    delta = end_date_obj - start_date_obj

    today = start_date_obj.strftime("%Y%02m%02d")

    basic_quer_part1 = """ select  ingress_code_name,sum(ingress_bill_time) as bill_time ,sum(ingress_call_cost) as cost ,sum(not_zero_calls) as calls ,
	sum(ingress_bill_time_intra) as bill_time_intra ,sum(ingress_call_cost_intra) as cost_intra ,sum(intra_not_zero_calls) as intra_calls ,
	sum(ingress_bill_time_inter) as bill_time_inter ,sum(ingress_call_cost_inter) as cost_inter ,sum(inter_not_zero_calls) as inter_calls 
	 from  (    select ingress_code_name, ingress_bill_time   , ingress_call_cost ,not_zero_calls ,
	 ingress_bill_time_intra  ,ingress_call_cost_intra  ,intra_not_zero_calls  , 
	 ingress_bill_time_inter  ,ingress_call_cost_inter  ,inter_not_zero_calls  
	 from    cdr_report_detail%s  where
 ingress_client_id =%s  and report_time between '%s' and '%s' """ % (
        today, client_id, start_datetime_obj, end_datetime_obj)

    basic_quer_part3 = """ )as A group by ingress_code_name; """
    basic_make_query = basic_quer_part1

    for i in range(1, delta.days + 1):
        day = start_date_obj + datetime.timedelta(days=i)  # .replace("-","")
        day = day.strftime("%Y%02m%02d")
        basic_make_query += """UNION ALL   select ingress_code_name, ingress_bill_time ,ingress_call_cost , not_zero_calls ,
	 ingress_bill_time_intra  ,ingress_call_cost_intra  ,intra_not_zero_calls  , 
	 ingress_bill_time_inter  ,ingress_call_cost_inter  ,inter_not_zero_calls  
  from   cdr_report_detail%s  where
 ingress_client_id =%s and report_time between '%s' and '%s' """ % (
            day, client_id, start_datetime_obj, end_datetime_obj)
    basic_make_query += basic_quer_part3

    print("SUMARY BY  Make_show_trafic_code_name_jur:", basic_make_query)
    logger.debug("SUMARY BY  Make_show_trafic_code_name_jur:" + basic_make_query)
    LOGGER_LIST.append("SUMARY BY  Make_show_trafic_code_name_jur:" + basic_make_query)
    try:
        pg_cur.execute(basic_make_query)
        result = pg_cur.fetchall()

    except Exception as e:
        logger.error(" Error in getting sumary by Trunk:  %s" % (str(e)))
        print("error in getting sumary by Trunk %s" % (str(e)))
        return table
    print("RESULT OF SUMARY BY TRUNK JUR:", result)
    # ingress_country| | ingress_rate | ingress_client_rate | bill_time | cost | calls | cost_intra | cost_inter | bill_time_intra | bill_time_inter
    # [['United States', 3, None, 0.001, 378, Decimal('0.006300'), 9, Decimal('0.000000'), Decimal('0.000000'), 0, 0], ['United States', 3, None, 10.0, 756, Decimal('0.012600'), 18, Decimal('0.000000'), Decimal('0.000000'), 0, 0]]
    float_form3 = "%0.3f"

    total = def_dict(int)

    for row in result:
        if row.get('ingress_code_name', None) == None:
            print("The row is empty")
            continue
        else:
            ingress_code_name = row.get('ingress_code_name', "Null")
            if len(ingress_code_name) > 9:
                ingress_code_name = ingress_code_name[:7] + "..."
        try:
            try:
                rate = float_form % (float(row['cost'] - row['cost_intra'] - row['cost_inter']) / (
                        row['bill_time'] / 60 - row['bill_time_inter'] / 60 - row['bill_time_intra'] / 60))
            except:
                rate = float_form % (0)

            try:
                rate_intra = float_form % (float(row['cost_intra']) / (row.get('bill_time_intra', 0) / 60))
            except:
                rate_intra = float_form % (0)

            try:
                rate_inter = float_form % (float(row['cost_inter']) / (row.get('bill_time_inter', 0) / 60))
            except:
                rate_inter = float_form % (0)

            total["intra_calls"] += row['intra_calls']
            total["bill_time_intra"] += row['bill_time_intra'] / 60
            total["cost_intra"] += row['cost_intra']

            total["inter_calls"] += row['inter_calls']
            total["bill_time_inter"] += row['bill_time_inter'] / 60
            total["cost_inter"] += row['cost_inter']

            total["ij_calls"] += row['calls'] - row['intra_calls'] - row['inter_calls']
            total["bill_time_ij"] += row['bill_time'] / 60 - row['bill_time_inter'] / 60 - row['bill_time_intra'] / 60
            total["cost_ij"] += row['cost'] - row['cost_intra'] - row['cost_inter']

            table.append([ingress_code_name, row.get('intra_calls'), float_form % (row.get('bill_time_intra', 0) / 60),
                          rate_intra,
                          float_form % (row['cost_intra']),
                          row.get('inter_calls'), float_form % (row.get('bill_time_inter', 0) / 60), rate_inter,
                          float_form % (row.get('cost_inter', 0)),
                          row['calls'] - row['intra_calls'] - row['inter_calls'],
                          float_form % (row['bill_time'] / 60 - row['bill_time_inter'] / 60 - row[
                              'bill_time_intra'] / 60), rate,
                          float_form % (row.get('cost') - row['cost_intra'] - row.get('cost_inter', 0))])

        except Exception as e:
            logger.error(" Error in getting summary by trunk  %s" % (str(e)))
            print("error in getting sumary by trunk %s" % (str(e)))
            continue

    if len(table) > 1:
        table.append(["Total", total["intra_calls"], float_form % (total["bill_time_intra"]), "",
                      float_form % (total["cost_intra"]),
                      total["inter_calls"], float_form % (total["bill_time_inter"]), "",
                      float_form % (total["cost_inter"]),
                      total["ij_calls"], float_form % (total["bill_time_ij"]), "", float_form % (total["cost_ij"])])

    end = timer()
    logger.debug("execution time  in milliseconds: %s" % ((end - start) * 1000))
    return table


def detail_by_trunk(param_dict, client, float_form):
    # if param_dict.get('jurisdict',False):  #invoice_jurisdictional_detail
    #	jurisdict = detail_by_trunk_jur(param_dict,client,float_form)
    #	return jurisdict

    # http://jira.denovolab.com/browse/CLAS-5789
    table = [['', 'Total Calls', 'Total Minutes', 'ACD', 'Amount']]
    start = timer()
    logger.debug("Entering Detail by Trunk Table")
    LOGGER_LIST.append("Entering Detail by Trunk Table")
    start_date = param_dict.get('start_date')
    end_date = param_dict.get('end_date')
    client_id = param_dict.get('client_id')

    if start_date == None or end_date == None or client_id == None:
        print("Error! Must suply client_id,start_date and end_date!")
        return {}
    try:
        start_date_obj = datetime.datetime.strptime(start_date, "%Y-%m-%d %H:%M:%S").date()
        end_date_obj = datetime.datetime.strptime(end_date, "%Y-%m-%d %H:%M:%S").date()
        start_datetime_obj = datetime.datetime.strptime(start_date, "%Y-%m-%d %H:%M:%S")
        end_datetime_obj = datetime.datetime.strptime(end_date, "%Y-%m-%d %H:%M:%S")
    except Exception as e:
        print(str(e))
        logger.error(str(e))
        return table
    delta = end_date_obj - start_date_obj
    if delta.days < 0:
        return None

    # last change from Jira CLAS-2764
    start = timer()
    logger.debug("Entering  detail_by_trunk Table")

    start_date = param_dict.get('start_date')
    end_date = param_dict.get('end_date')
    client_id = param_dict.get('client_id')

    if start_date == None or end_date == None or client_id == None:
        print("Error! Must suply client_id,start_date and end_date!")
        return {}
    try:
        start_date_obj = datetime.datetime.strptime(start_date, "%Y-%m-%d %H:%M:%S").date()
        end_date_obj = datetime.datetime.strptime(end_date, "%Y-%m-%d %H:%M:%S").date()
        start_datetime_obj = datetime.datetime.strptime(start_date, "%Y-%m-%d %H:%M:%S")
        end_datetime_obj = datetime.datetime.strptime(end_date, "%Y-%m-%d %H:%M:%S")
    except Exception as e:
        print(str(e))
        logger.error(str(e))
        return table
    delta = end_date_obj - start_date_obj
    if delta.days < 0:
        return None
    else:
        today = start_date_obj.strftime("%Y%02m%02d")  # duration, connected calls, Avg Rate, cost
        basic_quer_part1 = """  select sum (not_zero_calls_6) as short_calls ,sum (inter_not_zero_calls) as inter_calls,sum (intra_not_zero_calls)as intra_calls , (sum (not_zero_calls)- (sum (intra_not_zero_calls  )+ 
 ( sum (inter_not_zero_calls) ) )) as indeterminate_calls, round (sum (ingress_call_cost_intra),2) as intra_cost ,round(sum (ingress_call_cost_inter),2) as inter_cost,
  round((sum (ingress_call_cost)-(sum (ingress_call_cost_intra) + sum (ingress_call_cost_inter))),2) as indeterminate_cost, round(sum (ingress_bill_time_intra)/60::numeric,2) as
   intra_minutes, round(sum(ingress_bill_time_inter)/60::numeric,2) as inter_minutes ,
   round ( (sum ( ingress_bill_time ) -(sum (ingress_bill_time_intra)+sum(ingress_bill_time_inter))) /60::numeric,2) as indeterminate_minutes  
    from  ( select ingress_id, not_zero_calls_6, inter_not_zero_calls,intra_not_zero_calls,not_zero_calls,ingress_call_cost_intra,ingress_call_cost_inter,ingress_call_cost,
    ingress_bill_time_intra,ingress_bill_time_inter,ingress_bill_time 
     from cdr_report_detail%s where ingress_client_id =%s """ % (today, client_id)

        basic_quer_part3 = """ )as A  """  # group by ingress_id
        basic_make_query = basic_quer_part1

        for i in range(1, delta.days + 1):
            day = start_date_obj + datetime.timedelta(days=i)  # .replace("-","")
            day = day.strftime("%Y%02m%02d")
            basic_make_query += """UNION ALL select  ingress_id, not_zero_calls_6, inter_not_zero_calls,intra_not_zero_calls,not_zero_calls,ingress_call_cost_intra,ingress_call_cost_inter,ingress_call_cost,
    ingress_bill_time_intra,ingress_bill_time_inter,ingress_bill_time 
     from cdr_report_detail%s where ingress_client_id =%s """ % (day, client_id)
        basic_make_query += basic_quer_part3

        print("Detail by Trunk QUERY:", basic_make_query)
        logger.debug("Detail by Trunk QUERY:" + basic_make_query)
        LOGGER_LIST.append(":" + basic_make_query)
        try:
            pg_cur.execute(basic_make_query)
            result = pg_cur.fetchone()  # all()

        except Exception as e:
            logger.error(" Error in getting ingress code_name detail:  %s" % (str(e)))
            print("error in getting ingress code_name detail  %s" % (str(e)))
            logger.debug(" Error in getting ingress code_name detail :  %s" % (str(e)))
            return table
    print("Detail by Trunk QUERY result:", result)

    float_form3 = "%0.2f"
    total = def_dict(int)
    # table = [['' , 'Total Calls',  'Total Minutes' , 'ACD', 'Amount' ] ]
    # for row in result:
    row = result
    if row != None:
        # [[1122, 1544, 548, 937, Decimal('0.43'), Decimal('0.83'), Decimal('0.52'), Decimal('191.40'), Decimal('455.30'), Decimal('269.40')], [206610, 0, 0, 556206, Decimal('0.00'), Decimal('0.00'), Decimal('450.27'), Decimal('0.00'), Decimal('0.00'), Decimal('150090.30')]]
        # short_calls ,inter_calls,intra_calls , indeterminate_calls, inter_cost,intra_cost ,  indeterminate_cost,  intra_minutes,inter_minutes ,indeterminate_minutes
        try:
            inter_rate = float(row['inter_minutes']) / row['inter_calls']
        except:
            inter_rate = 0.00
        try:
            intra_rate = float(row['intra_minutes']) / row['intra_calls']
        except:
            intra_rate = 0.00
        try:
            indeter_rate = float(row['indeterminate_minutes']) / row['indeterminate_calls']
        except:
            indeter_rate = 0.00
        try:
            inter_cost = float(row['inter_cost'])
        except Exception as e:
            inter_cost = 0.00
        try:
            intra_cost = float(row['intra_cost'])
        except Exception as e:
            intra_cost = 0.00
        try:
            indeterminate_cost = float(row['indeterminate_cost'])
        except Exception as e:
            indeterminate_cost = 0.00

        table.append(
            ["Interstate", row['inter_calls'], row['inter_minutes'], float_form3 % (inter_rate),
             float_form % inter_cost])
        table.append(
            ["Intrastate", row['intra_calls'], row['intra_minutes'], float_form3 % (intra_rate),
             float_form % intra_cost])
        table.append(
            ["Indeterminate", row['indeterminate_calls'], row['indeterminate_minutes'], float_form3 % (indeter_rate),
             float_form % indeterminate_cost])
        table.append(["Short calls (under 6 seconds)", row['short_calls']])

        total_cost = inter_cost + intra_cost + indeterminate_cost

        table.append(["Total Amount", "", "", "", float_form % (total_cost)])

    # table.append (  ["Total", float_form % (  total["bill_time"]) ,total["calls"],"",float_form % ( total["cost"] ) ])

    end = timer()
    logger.debug("execution time  in milliseconds: %s" % ((end - start) * 1000))
    return table


def showCallsDate(param_dict, client, float_form):
    #
    start = timer()
    logger.debug("Entering Show Calls by Date Table")
    LOGGER_LIST.append("Entering Show Calls by Date Table")
    table = [["Date", "Calls", "Duration", "Amount"]]
    start_date = param_dict.get('start_date')
    end_date = param_dict.get('end_date')
    client_id = param_dict.get('client_id')

    if start_date == None or end_date == None or client_id == None:
        print("Error! Must suply client_id,start_date and end_date!")
        return {}
    try:
        start_date_obj = datetime.datetime.strptime(start_date, "%Y-%m-%d %H:%M:%S").date()
        end_date_obj = datetime.datetime.strptime(end_date, "%Y-%m-%d %H:%M:%S").date()
        start_datetime_obj = datetime.datetime.strptime(start_date, "%Y-%m-%d %H:%M:%S")
        end_datetime_obj = datetime.datetime.strptime(end_date, "%Y-%m-%d %H:%M:%S")
    except Exception as e:
        print(str(e))
        logger.error(str(e))
        return table
    delta = end_date_obj - start_date_obj
    if delta.days < 0:
        return None


    else:
        today = start_date_obj.strftime("%Y%02m%02d")

        basic_quer_part1 = """ select  report_time::timestamp::date , sum(ingress_call_cost) as cost 
		 , sum (not_zero_calls) as calls, sum (duration) as duration
		   from (SELECT  report_time ,not_zero_calls ,ingress_call_cost , duration   from  public.cdr_report_detail%s   where
 ingress_client_id =%s and  report_time between '%s' and '%s' """ % (
            today, client_id, start_datetime_obj, end_datetime_obj)

        basic_quer_part3 = """ )as A group by  report_time::timestamp::date order by report_time; """
        basic_make_query = basic_quer_part1

        for i in range(1, delta.days + 1):
            day = start_date_obj + datetime.timedelta(days=i)  # .replace("-","")
            day = day.strftime("%Y%02m%02d")
            basic_make_query += """UNION ALL  select  report_time ,not_zero_calls ,ingress_call_cost , duration  
			 from    public.cdr_report_detail%s    
			  where  ingress_client_id =%s and report_time between '%s' and '%s' """ % (day,
                                                                                        client_id, start_datetime_obj,
                                                                                        end_datetime_obj)
        basic_make_query += basic_quer_part3

        print("SHOW CALLS BY DATE QUERY:", basic_make_query)
        logger.debug("SHOW CALLS BY DATE QUERY:" + basic_make_query)
        LOGGER_LIST.append("SHOW CALLS BY DATE QUERY:" + basic_make_query)
        try:
            pg_cur.execute(basic_make_query)
            result = pg_cur.fetchall()

        except Exception as e:
            logger.error(" Error in getting show code by date:  %s" % (str(e)))
            print("error in getting  show code by date %s" % (str(e)))
            logger.debug(" Error in getting show code by date:  %s" % (str(e)))
            return table
    # print ("SHOW BY CODE DATE RESULT:" , result)

    total = def_dict(int)

    for row in result:
        if int(row.get("cost", 0)) != 0:
            total["bill_time"] += row['duration'] / 60
            total["cost"] += row['cost']
            total["calls"] += row['calls']
            table.append(
                [row.get("report_time", "N/A"), row.get("calls", 0), float_form % (row.get("duration", 0) / 60),
                 float_form %
                 (row.get("cost", 0))])
    if len(table) > 1:
        table.append(["Total", total["calls"], float_form % (total["bill_time"]), float_form % (total["cost"])])
    # print ("show code date table:",table)

    end = timer()
    logger.debug("execution time  in milliseconds: %s" % ((end - start) * 1000))
    return table


def IncludeOriginationBilling(param_dict, client, float_form):
    if True:
        return None
    #
    start = timer()
    logger.debug("Entering Include Orination billing Table")
    LOGGER_LIST.append("Entering Include Orination billing Table")
    float_form = param_dict.get('float_form', '%.2f')
    rate_float_form = param_dict.get('rate_float_form', '%.4f')
    table = [["Routing digits", "Count"]]
    start_date = param_dict.get('start_date')
    end_date = param_dict.get('end_date')
    client_id = param_dict.get('client_id')

    if start_date == None or end_date == None or client_id == None:
        print("Error! Must suply client_id,start_date and end_date!")
        return {}
    try:
        start_date_obj = datetime.datetime.strptime(start_date, "%Y-%m-%d %H:%M:%S").date()
        end_date_obj = datetime.datetime.strptime(end_date, "%Y-%m-%d %H:%M:%S").date()
        start_datetime_obj = datetime.datetime.strptime(start_date, "%Y-%m-%d %H:%M:%S")
        end_datetime_obj = datetime.datetime.strptime(end_date, "%Y-%m-%d %H:%M:%S")
    except Exception as e:
        print(str(e))
        logger.error(str(e))
        ErrorInLogInvoice(str(e))
        return table
    delta = end_date_obj - start_date_obj
    if delta.days < 0:
        return None


    else:
        today = start_date_obj.strftime("%Y%02m%02d")

        basic_quer_part1 = """ select  ingress_code ,sum (count) as count  
		   from (SELECT  ingress_code ,count(*)   from  public.cdr_report_detail%s   where
 ingress_client_id =%s and   report_time  between '%s' and '%s' group by ingress_code """ % (today, client_id,
                                                                                             start_datetime_obj,
                                                                                             end_datetime_obj)

        basic_quer_part3 = """ )as A group by ingress_code;  """
        basic_make_query = basic_quer_part1

        for i in range(1, delta.days + 1):
            day = start_date_obj + datetime.timedelta(days=i)  # .replace("-","")
            day = day.strftime("%Y%02m%02d")
            basic_make_query += """UNION ALL  select   ingress_code ,count(*)  
			 from    public.cdr_report_detail%s    
			  where  ingress_client_id =%s and report_time between '%s' and '%s'  group by ingress_code """ % (day,
                                                                                                               client_id,
                                                                                                               start_datetime_obj,
                                                                                                               end_datetime_obj)
        basic_make_query += basic_quer_part3

        print("Include Origination Billing QUERY:", basic_make_query)
        logger.debug("Include Origination Billing QUERY:" + basic_make_query)
        LOGGER_LIST.append("Include Origination Billing QUERY:" + basic_make_query)
        try:
            pg_cur.execute(basic_make_query)
            result = pg_cur.fetchall()

        except Exception as e:
            logger.error(" Error in Include Origination Billing:  %s" % (str(e)))
            print("error in getting Include Origination Billing %s" % (str(e)))
            logger.debug("Error in Include Origination Billing:  %s" % (str(e)))
            return table
    # print ("Include Origination Billing RESULT:" , result)
    total = def_dict(int)
    for row in result:
        if row.get("ingress_code", None) != None:
            total["count"] += row['count']
            table.append([row.get("ingress_code", 0), row.get("count", 0)])

    if len(table) > 1:
        table.append(["Total", '%d' % (total["count"])])

    print("Include Origination Billing table length:", len(table))
    end = timer()
    logger.debug("execution time  in milliseconds: %s" % ((end - start) * 1000))
    return table


def id_generator(size=12, chars=string.ascii_uppercase + string.ascii_lowercase + string.digits):
    # do not use random numbers, Invoice name should be -  <client_id>_<date time of generation> , http://jira.denovolab.com/browse/CLAS-2510
    return datetime.datetime.now().strftime("%Y%m%d%H%M")


# return ''.join(random.choice(chars) for _ in range(size))


def makeNewInv(param_dict, autoinvoice=False):
    global log_id, logo_file
    print("PARAM DICT FOR NEW INVOICE:", str(param_dict))
    LOGGER_LIST.append("PARAM DICT FOR NEW INVOICE: %s" % str(param_dict))

    status, msg = CheckTableNamesNew(param_dict)
    print(status)
    if status != 0:
        return -3, msg

    try:
        pg_cur.execute(CLIENT_INVOICE_SETTINGS % param_dict.get('client_id'))
        switch_settings = pg_cur.fetchone()
        if not switch_settings:
            pg_cur.execute(SWITCH_INVOICE_SETTINGS)
            switch_settings = pg_cur.fetchone()
        print("switch_settings", switch_settings)
    except Exception as e:
        msg = "Error in getting switch settings:  %s" % (str(e))
        logger.error(msg)
        print(msg)
        return [-1, msg]

    zone = param_dict.get('invoice_zone')
    if zone == None:
        zone = "+00:00"
    address = param_dict.get('address')
    if address == None:
        address = ""
    due_date = param_dict.get('invoice_due_date')  # .replace("-","/")
    if due_date == None:
        msg = "invoice_due_date must be provide"
        print(msg)
        logger.error(msg)
        return [-1, msg]

    ### Get client info
    query = "select * from client where client_id= %s" % (param_dict['client_id'])
    print("Query for getting client info: ", query)
    pg_cur.execute(query)
    client = pg_cur.fetchone()
    address = client.get('address', "N/A")
    print("Client address: ", client.get('address', "N/A"))
    company = client.get('company', "N/A")
    print("Client company: ", client.get('company', "N/A"))
    zone = client.get('invoice_zone', "N/A")
    invoice_id = param_dict.get('invoice_number', "1111")
    start_date = param_dict.get('start_date', "No Start date")
    end_date = param_dict.get('end_date', "No End date")
    try:
        param_dict['start_date_tmstamp'] = dt.datetime.strptime(start_date, "%Y-%m-%d %H:%M:%S")
        param_dict['end_date_tmstamp'] = dt.datetime.strptime(end_date, "%Y-%m-%d %H:%M:%S")
    except:
        param_dict['start_date_tmstamp'] = dt.datetime.strptime(start_date, "%Y-%m-%d %H:%M:%S")
        param_dict['end_date_tmstamp'] = dt.datetime.strptime(end_date, "%Y-%m-%d %H:%M:%S")
    period = str(start_date)[:10] + " - " + str(end_date)[:10]
    invoice_date = param_dict.get("invoice_date", "N/A")

    if not 'decimal_place' in param_dict:
        if autoinvoice == True:
            rate_round_to = client['decimal_place']
        else:  # manual invoice creation, must take decimal places from other place
            sql = """SELECT invoice_decimal_digits FROM system_parameter"""
            try:
                pg_cur.execute(sql)
                result = pg_cur.fetchone()
                if result != None:
                    rate_round_to = result.get('invoice_decimal_digits', client['decimal_place'])
                else:
                    rate_round_to = client['decimal_place']
            except Exception as e:
                print(" Exception in getting system parameter for Dec Place:  %s" % (str(e)))
                traceback.print_tb(e.__traceback__)
                logger.error("  Exception in getting system parameter for Dec Place:  %s" % (str(e)))
                round_to = 4
        param_dict['decimal_place'] = str(rate_round_to)
    else:
        try:
            rate_round_to = int(param_dict['decimal_place'][0])
        except:
            try:
                rate_round_to = int(param_dict['decimal_place'])
            except:
                rate_round_to = 4
    print('rate_round_to {}'.format(rate_round_to))
    if rate_round_to not in [0, 1, 2, 3, 4, 5, 6, 7, 8, 9]:
        rate_round_to = 4
    rate_float_form = "%0.{}f".format(rate_round_to)
    param_dict['rate_float_form'] = rate_float_form

    if not 'decimals_num' in param_dict or param_dict['decimals_num'] is None:
        sql = """SELECT invoice_decimal_digits FROM system_parameter"""
        try:
            pg_cur.execute(sql)
            result = pg_cur.fetchone()
            if result != None:
                round_to = result.get('invoice_decimal_digits', client['decimal_place'])
            else:
                round_to = client['decimal_place']
        except Exception as e:
            print(" Exception in getting system parameter for Dec Place:  %s" % (str(e)))
            traceback.print_tb(e.__traceback__)
            logger.error("  Exception in getting system parameter for Dec Place:  %s" % (str(e)))
            round_to = 2
        param_dict['decimals_num'] = str(round_to)
    else:
        try:
            round_to = int(param_dict['decimals_num'])
        except:
            round_to = 2
    print('round_to {}'.format(round_to))
    if round_to not in [0, 1, 2, 3, 4]:
        round_to = 4
    float_form = "%0.{}f".format(round_to)
    param_dict['float_form'] = float_form

    bill_dict, total_amount = GetBill(param_dict)
    # print ("BILL: ",bill_dict,"total_amount ",total_amount)# print ("BILL: ",bill_dict,"total_amount ",total_amount)
    if bill_dict == -1:
        print("Invoice will be not generated", total_amount)  # total_amount in this case is error msg
        logger.error("Invoice will be not generated %s" % total_amount)
        FinishInLogInvoice(3, 0)
        return [total_amount, -11]
    if bill_dict == {}:
        msg = "Problem with invoice for client. GetBill returned None"
        print("Problem with invoice for client. GetBill returned None")
        return [-1, msg]

    if param_dict.get('invoice_zero', True) == True and int(math.ceil(total_amount)) == 0 and False:
        print("Invoice will be not generated")
        logger.debug("Invoice will be not generated")
        FinishInLogInvoice(2, 0)
        return ["", -10]

    # Daily Usage
    if param_dict.get('dayusage', False) or param_dict.get('is_show_daily_usage', None):
        # this is daily usage!
        day_usage = MakeDayUsageHeader()
        day_usage_detail = MakeDailyQueryTable(param_dict, client, float_form,
                                               day_usage)  # MakeDailyQueryTable(bill_dict.get('details', []) ,day_usage)
    else:
        print("No dayusage. No is_show_daily_usage")
        day_usage_detail = None
    day_usage = day_usage_detail

    print("PARAM DICT FOR NEW INVOICE2:", str(param_dict), param_dict.get('is_invoice_account_summary', False))

    acount_sumary = None
    if param_dict.get('is_invoice_account_summary', False):  # or autoinvoice == False :
        acount_sumary = GetAcountSummary(param_dict, invoice_id)

    if param_dict.get('is_invoice_usage_detail', False):
        #
        usage_detail = MakeUsage(param_dict, client,
                                 float_form)  # MakeDailyQueryTable(bill_dict.get('details', []) ,day_usage)
    else:
        print("No usage_detail")
        usage_detail = None

    if param_dict.get('summary_of_payments', False):
        summary_of_payments = GetSummary_of_payments(param_dict, client, acount_sumary, float_form)
    else:
        print("No summary_of_payments")
        summary_of_payments = None

    # Jurisdictional Breakdown
    if param_dict.get('jurisdict', False):  # invoice_jurisdictional_detail
        jurisdict = GetJurisdict(param_dict, client, acount_sumary, float_form)
    else:
        print("No Jurisdictional  Breakdown")
        jurisdict = None
    param_dict['invoice_jurisdictional_detail'] = param_dict.get('jurisdict', False)

    if param_dict.get('ingress_prefix', False):
        ingress_prefix = GetIngress_prefix(param_dict, client, float_form)
    else:
        print("No ingress_prefix")
        ingress_prefix = None

    if param_dict.get('invoice_show_details', False):  # Yes invoice_show_details is for show code_name totals!
        ShowDetailByCodeName = MakeShowDetailByCodeName(param_dict, client, float_form)
    else:
        print("No MakeShowDetailByCodeName")
        ShowDetailByCodeName = None

    if param_dict.get('show_trafic_code_name', False):
        show_trafic_code_name = Make_show_trafic_code_name(param_dict, client, float_form)
    else:
        print("No show_trafic_code_name")
        show_trafic_code_name = None

    if param_dict.get('show_trafic_country', False):
        show_trafic_country = Make_show_trafic_country(param_dict, client, float_form)
    else:
        print("No show_trafic_country")
        show_trafic_country = None

    if param_dict.get('show_code_summery', False):
        show_code_summery = None  # not use in the moment
    # show_code_summery = Make_show_code_summery(param_dict,client,float_form)
    # print("Show_show_code_summery_table:" ,show_code_summery)
    else:
        print("Show_show_code_summery_table:")
        show_code_summery = None

    if param_dict.get('detail_by_trunk', False):
        # pass  #this is not correct and not used
        detail_by_trunk_table = detail_by_trunk(param_dict, client, float_form)
    # detail_by_trunk_table = None
    else:
        print("No detail_by_trunk")
        detail_by_trunk_table = None

    if param_dict.get('show_calls_date', False):
        show_calls_date = showCallsDate(param_dict, client, float_form)
    else:
        print("No show_calls_date")
        show_calls_date = None
    # print ("minava1")
    if param_dict.get('include_origination_billing', False):
        include_origination_billing = IncludeOriginationBilling(param_dict, client, float_form)
    else:
        print("No include_origination_billing")
        include_origination_billing = None

    basic_bill = bill_dict['basic']
    if basic_bill == None:
        basic_bill = dict()
    minutes = float_form2 % (float(basic_bill.get('bill_time', 0) / 60))
    cost = float_form % (basic_bill.get('call_cost', 0))
    # print ("minava3")
    summary_of_charges = SumaryOfCharges(basic_bill, param_dict, client, basic_bill.get('call_cost', 0), minutes,
                                         float_form)
    # print ("minava4")
    print("summary_of_charges :", summary_of_charges)
    acount_sumary = MakeAccountSumary(acount_sumary, float_form, client, param_dict, basic_bill.get('call_cost', 0))
    print("show_code_summery :", show_code_summery)
    client_info = [client['company'], address, invoice_id, str(invoice_date)[:10], due_date, period, zone]
    invoice_name = str(invoice_id) + id_generator()  # +"_"
    print("ShowDetailByCodeName  ", ShowDetailByCodeName)
    try:
        DownloadLogo(param_dict['client_id'])
    except Exception as e:
        logger.error(" Error in Downloading logo: %s" % (str(e)))
        print("error in Downloading logo %s" % (str(e)))
        # DownloadLogoNoResize()
        DownloadLogo()
        # set the blank logo instead
        LOGGER_LIST.append(" Error in Downloading logo: %s" % (str(e)))
        # logo_file = "blank_logo.jpg"

    # getting invoice path
    try:
        pdf_path = config['other']["pdf_path"]
        if os.path.isdir(pdf_path):
            # randomfile = ''.join(random.choice(string.ascii_uppercase + string.digits) for _ in range(7))
            # f = open(randomfile,'w')
            # f.close()
            pass
        else:
            os.makedirs(pdf_path)
    except Exception as e:
        print("pdf_path exception %s" % e)
        logger.debug("pdf_path exception %s" % e)
        pdf_path = "app/invoice"
    # it is not used now
    breakdown_by_rate_table = None

    # if not param_dict.get('show_account_summary', False):
    # 	print("Do not display Account summary")
    # 	acount_sumary = None
    # 	print(param_dict.get('show_account_summary', False))
    # else:
    # 	print("display Account summary")
    # 	print(param_dict.get('show_account_summary', False))

    try:
        if param_dict.get('jurisdict', False):
            doc = IC.LetterMaker("%s/%s.pdf" % (pdf_path, invoice_name), "Denovolab", client_info, summary_of_charges,
                                 acount_sumary, summary_of_payments, usage_detail, jurisdict, day_usage, ingress_prefix,
                                 show_trafic_code_name, show_trafic_country, show_code_summery, detail_by_trunk_table,
                                 show_calls_date, include_origination_billing, 'resized_' + logo_file, switch_settings,
                                 ShowDetailByCodeName, breakdown_by_rate_table, True)
        else:
            doc = IC.LetterMaker("%s/%s.pdf" % (pdf_path, invoice_name), "Denovolab", client_info, summary_of_charges,
                                 acount_sumary, summary_of_payments, usage_detail, jurisdict, day_usage, ingress_prefix,
                                 show_trafic_code_name, show_trafic_country, show_code_summery, detail_by_trunk_table,
                                 show_calls_date, include_origination_billing, 'resized_' + logo_file, switch_settings,
                                 ShowDetailByCodeName, breakdown_by_rate_table)
        doc.createDocument()
        doc.savePDF()
    except Exception as e:
        print(" Exception in createDocument:  %s" % (str(e)))
        traceback.print_tb(e.__traceback__)
        logger.error(" Exception in createDocument:  %s" % (str(e)))
        return [-1, invoice_id]
    LogInvoice(start_date, end_date, invoice_date, client, param_dict, invoice_id, invoice_name, total_amount, pdf_path,
               autoinvoice)
    FinishInLogInvoice(2, 1)
    return ["%s.pdf" % (invoice_name), invoice_id]


def checkInvoiceDays(param_dict):
    '''check if the period of this invoice is overlapping with old invoices'''
    #
    start = timer()
    logger.debug("Entering new invoice for overlapping with older invoices")
    client_id = param_dict['client_id']
    invoice_start = param_dict['start_date']
    invoice_end = param_dict['end_date']

    try:
        pg_cur.execute(SWITCH_INVOICE_SETTINGS)
        result = pg_cur.fetchone()['overlap_invoice_protection']
        print('overlap_invoice_protection', result)
        if result == False:
            return [0, 0]

    except Exception as e:
        logger.error(" Error in Invoice overlapping   %s" % (str(e)))
        print("error in Invoice overlapping  %s" % (str(e)))
        result[0, 0]

    sql = """select * from invoice where client_id = %s and invoice_end >'%s' and invoice_start <'%s' and state > -1 limit 1 ;  """ % (
        client_id, invoice_start, invoice_end)

    print("Invoice overlapping QUERY:", sql)
    logger.debug("Invoice overlapping QUERY: %s" % sql)
    try:
        pg_cur.execute(sql)
        result = pg_cur.fetchone()
        print(result)
    except Exception as e:
        logger.error(" Error in Invoice overlapping   %s" % (str(e)))
        print("error in Invoice overlapping  %s" % (str(e)))
        logger.debug("error in Invoice overlapping  %s" % (str(e)))
        return [-1, str(e)]
    if result is not None:
        print("returning -1")
        logger.debug("Overlapping invoice number: %s       Start: %s   End: %s " % (
            result.get("invoice_number", "N/A"), result.get("invoice_start", "N/A")
            , result.get("invoice_end", "N/A")))
        return [-1, "invoice number: %s       Start: %s   End: %s " % (
            result.get("invoice_number", "N/A"), result.get("invoice_start", "N/A"),
            result.get("invoice_end", "N/A"))]  # overlapping !
    else:
        print("returning 0", result)
        return [0, 0]


def MakeInvoiceAuto(data, pg_cur_a=""):
    global log_id, pg_cur
    if pg_cur_a != "":
        pg_cur = pg_cur_a
    # data = client
    log_id = InsertInLogInvoice()
    # ,logo_file
    start = timer()
    LOGGER_LIST = []
    logger.debug("Starting new auto invoice")
    param_dict = GetParamsAuto(data)
    auto = True
    if param_dict.get('create_type') == 1:
        auto = False
        print("This invoice is Regen of Manual Invoice")
    else:
        print("This invoice will be Auto Invoice")
    logger.debug("Options set: %s " % str(param_dict))
    try:
        filename, invoice_number = makeNewInv(param_dict, True)
    except Exception as e:
        print(" Exception in making new invoice:  %s" % (str(e)))
        logger.error(" Exception in making new invoice:  %s" % (str(e)))
        ErrorInLogInvoice(str(e))
        return [-1, -1, -1]
    if filename == -1 or invoice_number == -1:
        logger.error("Creating new Invoice Failed. Aborting process")
        [-1, -1, invoice_number]

    # this was used for early debug
    '''if int(config['log']["debug_level"]) > 0 :
		if filename == -1:
			filename = "last_error"
		try:
			with open('./app/invoice/'+str(filename)+'.txt', mode='wt', encoding='utf-8') as myfile:
				myfile.write('\n'.join(LOGGER_LIST))
		except Exception as e: 
			print (" Exception in saving log file:  %s" % (str(e) ) )
			logger.error(" Exception in saving log file %s:" % (str(e) ) ) 
			#ErrorInLogInvoice( str(e) )
			#return [-1,-1,-1]
	
	for debug_elem in LOGGER_LIST:
		logger.debug(debug_elem)
	'''
    return [0, filename, invoice_number]


def GetEmailTemplate(cursor):
    sql = """select invoice_from ,invoice_to ,invoice_cc ,invoice_subject ,invoice_content from mail_tmplate limit 1 ;"""
    try:
        cursor.execute(sql)
        email_template = cursor.fetchone()
    except Exception as e:
        logger.error(" Exception in getting EmailTemplate:  %s" % (str(e)))
        print(" Exception in getting EmailTemplate:  %s" % (str(e)))
        return -1
    return email_template


def GetClient(params, pg_cur1=None):
    if pg_cur1 != None:
        pg_cur = pg_cur1
    sql = """ select * from client where client_id =  %s """ % params['client_id']
    try:
        pg_cur.execute(sql)
        client = pg_cur.fetchone()
    except Exception as e:
        # print ("SQL for updating cdr path in DB:  %s" % (str(e) ) )
        logger.error("Exception in  Get Client Details  %s" % (str(e)))
        traceback.print_tb(e.__traceback__)
        return -1
    return client


def syncSubProcessEmailCdr(params, data):
    '''sync handlder for email invoice and cdr download link generation '''
    try:
        pg_cur = new_cursor()
        data = dict(data)
        send_to = data['email'][0]
        smtp_info = get_smtp_info(pg_cur)
        EmailTemplate = GetEmailTemplate(pg_cur)
        client = GetClient(params, pg_cur)

        # Get config.ini
        config = configparser.ConfigParser()
        config.read('./config.ini')

        print("smtp_info", smtp_info, "EmailTemplate", EmailTemplate, "client", client)
        subject = '''Hello %s. This is resend of your invoice'''
        data = {'start_date': str(params['invoice_start']).replace("+00:00", ""),
                'end_date': str(params['invoice_end']).replace("+00:00", "")}
        print("'pdf_path'", params['pdf_path'])
        try:
            fields = GetCdrFields(pg_cur)
            cdr_link = getCdrLink(client, pg_cur, params['invoice_id'], data, fields)
        except Exception as e:
            print("Error inside get cdr link ", e)
        try:
            result = email_invoice(
                config['other']['pdf_path'] + "/" + params['pdf_path'][params['pdf_path'].rfind('/') + 1:], client,
                data, smtp_info, send_to, True, params['invoice_number'], pg_cur)
        except Exception as e:
            print("Error inside email invoice ", e)
            traceback.print_tb(e.__traceback__)

    except Exception as e:
        print("Error inside created subproccess ", e)


def reSendEmail(params, data, pg_cur=None):
    print("reSendEmail")
    try:
        # p = multiprocessing.Process(target=syncSubProcessEmailCdr, args=(params,data)
        syncSubProcessEmailCdr(params, data)
    # p.daemon = False
    # p.start()
    # jobs.append(p)

    except Exception as e:
        print("Error in creating subproccess ", e)
    return "OK"


# jobs.append(p)

def email_invoice(filename, client, data, smtp_info, send_to="", resend=False, invoice_number=None, pg_cur1=None):
    '''Sends email with invoice or link to invoice with given filename'''
    global CDR_UI, config
    FTP_HOST = config["other"].get('ftp', "").replace("http://", "")
    pg_cur = None
    if pg_cur1 != None:
        pg_cur = pg_cur1
    EmailTemplate = GetEmailTemplate(pg_cur)
    ##CDR link
    print(smtp_info)
    logger.debug(smtp_info)
    mail_subject = str(EmailTemplate.get('invoice_subject',
                                         "N/A"))  # subject % ( client.get("client_name",client.get("name","N/A") ) )
    mail_subject = mail_subject.replace('{company_name}', client.get('company', 'N/A')).replace('{start_date}', str(
        data.get('start_date', 'N/A'))).replace('{end_date}', str(data.get('end_date', 'N/A')))
    if EmailTemplate.get('invoice_from', "default") != "default":
        send_from_id = EmailTemplate.get('invoice_from', 1)
        sql = """select * from mail_sender where id = %s""" % send_from_id
        pg_cur.execute(sql)
        _send_from = dict(pg_cur.fetchone())
        if _send_from:
            _send_from['switch_alias'] = smtp_info['switch_alias']
            smtp_info = _send_from
    invoice_cc = EmailTemplate.get('invoice_cc', "")
    if not resend:
        send_to = client['billing_email']  # .split(";")
    if len(send_to) == 0:
        print('client %s  email is not configured' % client['name'])
        logger.error('client %s  email is not configured' % client['name'])
        return -1
    # email attachment or as links
    send_attachment = None
    files = []
    sql = "select invoice_send_mode from invoice_settings s,client_invoice_settings c where c.invoice_setting_id=s.id and client_id={};".format(
        client['client_id'])
    pg_cur.execute(sql)
    result = pg_cur.fetchone()
    if result != None:
        send_attachment = result.get('invoice_send_mode', None)

    if send_attachment is None:
        try:
            sql = "select invoice_send_mode from client where client_id={};".format(client['client_id'])
            pg_cur.execute(sql)
            result = pg_cur.fetchone()
        except:
            sql = "select (not is_send_as_link)::integer as invoice_send_mode  from client where client_id={};".format(
                client['client_id'])
            pg_cur.execute(sql)
            result = pg_cur.fetchone()
        if result != None:
            send_attachment = result.get('invoice_send_mode', None)
    if send_attachment is None:
        sql = """SELECT invoice_send_mode FROM system_parameter"""
        if invoice_number == None:
            invoice_number = filename[:-16]  # 1120201705220835.pdf
        print("invoice_number", invoice_number)
        try:
            pg_cur.execute(sql)
            result = pg_cur.fetchone()
            if result != None:
                send_attachment = result.get('invoice_send_mode', 1)
            else:
                send_attachment = 1
        except Exception as e:
            print(" Exception in getting system parameter ")
            traceback.print_tb(e.__traceback__)
            logger.error("  Exception in getting system parameter for Send attachment:  %s" % (str(e)))
            send_attachment = 1
    print("send_attachment:", send_attachment)
    if send_attachment == 0:
        # print ("select invoice_id ,total_amount from invoice where invoice_number = '%s'"%invoice_number)
        try:
            sql = "select invoice_id ,total_amount,decimals_num,cdr_link from invoice where invoice_number = '%s'" % invoice_number
            pg_cur.execute(sql)
            result = pg_cur.fetchone()
            invoice_id = result['invoice_id']
            float_form = "%0.2f"
            if result['decimals_num']:
                float_form = "%0.{}f".format(result['decimals_num'])
            total_amount = float_form % result['total_amount']
            cdr_link = result['cdr_link']
            files = []
            base64_encode = base64.b64encode(
                bytes(str(invoice_id), 'utf-8'))  # int(invoice_number).to_bytes(2, byteorder='big')
            print("invoice_number", invoice_number, "FTP_HOST", FTP_HOST, "base64_encode", base64_encode.decode())
            links_pdf = "<a href = 'http://%s/%s' > Download Invoice here </a> <br>" % (
                FTP_HOST, base64_encode.decode())
        except Exception as e:
            print(" Exception in getting  invoice_id:  %s" % (str(e)))
            logger.error("Exception in invoice_id :  %s" % (str(e)))
            links_pdf = "Link to your invoice can`t be uptain. Please contact the support"
            files = []
            invoice_id = "N/A"
            total_amount = "N/A"
            cdr_link = None
    else:
        try:
            sql = "select invoice_id ,total_amount,decimals_num,cdr_link from invoice where invoice_number = '%s'" % invoice_number
            pg_cur.execute(sql)
            result = pg_cur.fetchone()
            invoice_id = result['invoice_id']
            float_form = "%0.2f"
            if result['decimals_num']:
                float_form = "%0.{}f".format(result['decimals_num'])
            total_amount = float_form % result['total_amount']
            cdr_link = result['cdr_link']
            links_pdf = "<i>pdf link not configured</i>"
            files = [filename]
        except Exception as e:
            print(" Exception in getting  invoice_id:  %s" % (str(e)))
            logger.error("Exception in invoice_id :  %s" % (str(e)))
            invoice_id = "N/A"
            total_amount = "N/A"
            cdr_link = None

    ##CDR link
    print("CDR_UI", CDR_UI, str(invoice_id))
    send_links_static = False

    if client.get('attach_cdrs_list', False):
        try:
            if not cdr_link:
                fields = GetCdrFields(pg_cur)
                getCdrLink(client, pg_cur, invoice_id, data, fields)
                sql = "select cdr_link from invoice where invoice_number = '%s'" % invoice_number
                pg_cur.execute(sql)
                result = pg_cur.fetchone()
                cdr_link = result['cdr_link']
            cdr_ui = CDR_UI % str(invoice_id)
            cdr_link_ui = config['cdr']["switch"] + cdr_ui
            links_cdr = "<a href='%s'>CDR LINK </a>  " % cdr_link_ui
        # cdr_exception = False
        except Exception as e:
            print("exception is getting cdr links %s" % e)
            logger.error("exception is getting cdr links %s" % e)
            links_cdr = "<i>error in cdr link</i>"
        # cdr_exception = True
        print("cdr_links", cdr_link)
    else:
        print("No is_link_cdr ")
        cdr_link = ["N/A"]
        links_cdr = "<i>cdr link not configured</i> "
    print(filename, invoice_number)
    print("Files", str(files))
    print("links_pdf", links_pdf)
    print("links_cdr", links_cdr)
    mail_content = EmailTemplate.get('invoice_content',
                                     "N/A mail content in system cursor. Please report to administrator!")
    mail_content = mail_content.replace('{company_name}', client.get('company', 'N/A')).replace('{switch_alias}', str(
        smtp_info.get('switch_alias', 'N/A'))).replace('{start_date}',
                                                       str(data.get('start_date', 'N/A'))).replace('{end_date}', str(
        data.get('end_date', 'N/A'))).replace('{invoice_number}', str(invoice_number)).replace('{invoice_link}',
                                                                                               links_pdf).replace(
        '{client_name}', client.get('name', 'N/A')).replace('{invoice_amount}', str(total_amount)).replace('{cdr_url}',
                                                                                                           links_cdr)
    from version import VERSION
    # mail_content += '<p style="color: gray; font-size: 7pt">dnl invoice generator ver {ver} date {date}</p>'.format_map(VERSION)
    logger.debug(mail_content)
    logger.debug(mail_subject)
    print("mail_subject ", mail_subject, send_to, invoice_cc)
    ret_msg = send_mail(smtp_info, send_to, invoice_cc, mail_subject, mail_content, files)
    if ret_msg == 'OK':
        send_email_log(client, send_to, mail_subject, mail_content, 'http://%s/invoice/%s' % (FTP_HOST, filename),
                       False, "", pg_cur)
    else:
        send_email_log(client, send_to, mail_subject, mail_content, 'http://%s/invoice/%s' % (FTP_HOST, filename), True,
                       ret_msg, pg_cur)
    # CDR_UI = "/homes/cdr_from_invoice/%s"
    return ret_msg


def GetInvoiceDetails(data):
    pg_cur_l = new_cursor()
    data = dict(data)
    # invoice_number = data['invoice_number'][0]
    invoice_number = data['invoice_number']
    if type(invoice_number) == type([]):
        invoice_number = invoice_number[0]
    try:
        int(invoice_number)
    except:
        return -2
    print("invoice_number:", invoice_number)
    sql = """select * from invoice where  invoice_number = '%s'""" % invoice_number

    print("sql for getting invoice number " + sql)
    try:
        pg_cur_l.execute(sql)
        result = pg_cur_l.fetchone()
    except Exception as e:
        # print ("SQL for updating cdr path in DB:  %s" % (str(e) ) )
        logger.error("Exception in  GetInvoiceDetails  %s" % (str(e)))
        traceback.print_tb(e.__traceback__)
        pg_cur_l.close()
        return -1

    # print ("Old Invoice Dict:", str(result))
    invoice_option = dict()  # options[0]
    invoice_option['invoice_number'] = invoice_number
    invoice_option['client_id'] = result['client_id']
    invoice_option['end_date'] = str(result['invoice_end']).replace("+00:00", "")
    invoice_option['start_date'] = str(result['invoice_start']).replace("+00:00", "")
    invoice_option['balance_time'] = str(result['invoice_balance_time']).replace("+00:00", "").replace("00:00:00", "")
    invoice_option['invoice_date'] = str(result['invoice_time']).replace("+00:00", "").replace("00:00:00", "")
    invoice_option['invoice_due_date'] = str(result['due_date']).replace("+00:00", "").replace("00:00:00", "")
    invoice_option['is_invoice_usage_detail'] = result['usage_detail_fields']
    invoice_option['jurisdict'] = result['invoice_jurisdictional_detail']
    invoice_option['is_short_duration_call_surcharge_detail'] = result['is_short_duration_call_surcharge_detail']
    invoice_option['is_invoice_account_summary'] = result['is_invoice_account_summary']
    invoice_option['detail_by_trunk'] = result['is_show_detail_trunk']
    invoice_option['include_origination_billing'] = 1  # result[''] # направи
    invoice_option['show_calls_date'] = result['is_show_by_date']
    invoice_option['summary_of_payments'] = result['invoice_include_payment']
    invoice_option['show_code_summery'] = result['is_show_code_100']
    invoice_option['show_trafic_country'] = result['is_show_country']
    invoice_option['show_trafic_code_name'] = result['is_show_code_name']
    invoice_option['is_send_as_link'] = result.get('is_send_as_link', False)
    invoice_option['create_type'] = result.get('create_type', 0)
    invoice_option['dayusage'] = result['is_show_daily_usage']

    # print (invoice_option['start_date'] , invoice_option['end_date'] ,invoice_option['balance_time'],invoice_option['invoice_date'],invoice_option['invoice_due_date'] )
    pg_cur_l.close()
    return invoice_option


def GetInvoiceDetailsAll(data):
    pg_cur = new_cursor()
    data = dict(data)
    invoice_number = data['invoice_number'][0]
    sql = """select * from invoice where  invoice_number = '%s'""" % invoice_number

    print("sql for getting invoice number " + sql)
    try:
        pg_cur.execute(sql)
        result = pg_cur.fetchone()
        return result, pg_cur
    except Exception as e:
        print("Exception in  GetInvoiceDetails  %s" % (str(e)))
        # logger.error("Exception in  GetInvoiceDetails  %s" % (str(e) ) )
        traceback.print_tb(e.__traceback__)


def markForResend(data):
    pg_cur = new_cursor()
    data = dict(data)
    invoice_number = data['invoice_number'][0]
    send_to_email = data['email'][0]
    sql = """UPDATE invoice SET api_requests = 1 ,api_params = '%s' where  invoice_number = '%s'""" % (
        send_to_email, invoice_number)  # api_requests = 1 is for resend

    print("sql for marking the invoice for resending! " + sql)
    try:
        pg_cur.execute(sql)
        pg_cur.close()
        return "OK"
    except Exception as e:
        print("Exception in  GetInvoiceDetails  %s" % (str(e)))
        # logger.error("Exception in  GetInvoiceDetails  %s" % (str(e) ) )
        traceback.print_tb(e.__traceback__)
        pg_cur.close()
        return ("Exception in  marking Invoice for resending  %s" % (str(e)))


def MakeInvoice(data, regenInvoice=False, SkipOverlapping=False, auto=False):
    '''Handles name creation'''
    global log_id, pg_cur, logger
    # print ('''Handles name creation''')
    pg_cur, logger = setup()
    log_id = InsertInLogInvoice()
    # ,logo_file
    start = timer()
    LOGGER_LIST = []
    logger.debug("Starting new invoice")
    # print ("Data:",data)
    logger.debug("UI request: " + str(data))
    data = dict(data)  # original is in multidict Flask  - not anymore
    # print (data ) #data['username']
    param_dict = GetParams(data, regenInvoice)
    print("param_dict", param_dict)
    try:
        param_dict['client_id'] = param_dict['client_id'][0]
        if type(param_dict['start_date']) == type([]):
            param_dict['start_date'] = param_dict['start_date'][0]
        if type(param_dict['end_date']) == type([]):
            param_dict['end_date'] = param_dict['end_date'][0]
        if type(param_dict['invoice_due_date']) == type([]):
            param_dict['invoice_due_date'] = param_dict['invoice_due_date'][0]
        if type(param_dict['invoice_date']) == type([]):
            param_dict['invoice_date'] = param_dict['invoice_date'][0]
        early_possible_date = config['other'].get('test_early_date', '2000-00-00') + " 00:00:00"  # = 2017-02-22
        pattern = " '%d-%m-%Y %H:%M:%S'"
        start_request_epoch = int(time.mktime(time.strptime(param_dict['client_id'], pattern)))
        early_possible_epoch = int(time.mktime(time.strptime(early_possible_date, pattern)))
        if start_request_epoch < early_possible_epoch:
            print("Start date is too early!")
            logger.debug("Start date is too early!")
            param_dict['start_date'] = start_request_epoch

    # print(param_dict)
    except  Exception as e:
        print("Bad request, ! %s   Trying with fallback" % str(e))
        try:
            param_dict['client_id'] = param_dict['client_id']
            param_dict['start_date'] = param_dict['start_date']
            param_dict['end_date'] = param_dict['end_date']
            param_dict['invoice_due_date'] = param_dict['invoice_due_date']
            param_dict['invoice_date'] = param_dict['invoice_date']
            early_possible_date = '2017-02-21 00:00:00'  # = 2017-02-22
            pattern = "%Y-%m-%d %H:%M:%S"
            start_request_epoch = int(time.mktime(time.strptime(param_dict['start_date'], pattern)))
            early_possible_epoch = int(time.mktime(time.strptime(early_possible_date, pattern)))
            if start_request_epoch < early_possible_epoch:
                print("Start date is too early!")
                logger.debug("Start date is too early!")
                param_dict['start_date'] = start_request_epoch
        except  Exception as e:
            print("Bad request, wrong format for dates! %s" % str(e))
            logger.error("Bad request, wrong format for dates! %s" % str(e))
            return [-1, "Bad request, wrong format for request to the API! The error is  %s" % str(e)]

    param_dict['regen'] = 1
    check = checkInvoiceDays(param_dict)  # [0]
    # print ("check: ", check)
    logger.debug("Options set: %s " % str(param_dict))

    print("skipOverlapping", SkipOverlapping)
    if check[0] == -1 and False:  # SkipOverlapping ==
        # overlapping
        print("invoice overlapping")
        return [-1, "invoice overlapping" + check[1]]
    if param_dict == -1:
        return [-1, "Wrong API call"]
    try:
        filename, invoice_number = makeNewInv(param_dict, auto)
        if filename == -3:  # missing daily table
            logger.error("Creating new Invoice Failed. Missing tables")
            return [-3, invoice_number]
    except Exception as e:
        traceback.print_tb(e.__traceback__)
        logger.error(" Exception in making new invoice:  %s" % (str(e)))
        return [-1, " Exception in making new invoice:  %s" % (str(e))]
    if int(config['log']["debug_level"]) > 0:
        with open('./app/invoice/' + str(filename) + '.txt', mode='wt', encoding='utf-8') as myfile:
            myfile.write('\n'.join(LOGGER_LIST))
    return [filename, invoice_number]


def VoidInvoicePost(data):
    try:
        pg_cur_l = new_cursor()
        param_dict = dict(data)
        print(param_dict)
        # param_dict = {}
        # for item in data :
        #	param_dict[item]=request.forms.get(item)

        sql = """update invoice set state =-1  where invoice_number ='%s' ; """ % (param_dict['invoice_number'][0])
        print("Sql for voiding invoice:", sql)
        pg_cur_l.execute(sql)
        print("executed")

    except Exception as e:
        logger.error(" Error in Invoice voiding   %s" % (str(e)))
        print("error in Invoice voiding   %s" % (str(e)))
        pg_cur_l.close()
        return str(e)
    pg_cur_l.close()
    return "Query executed: " + sql


def DeleteInvoicePost(data):
    pg_cur_l = new_cursor()
    try:
        param_dict = dict(data)
        print(param_dict)
        # param_dict = {}
        # for item in data :
        #	param_dict[item]=request.forms.get(item)
        # we will not change invoice number for regenerated invoices anymore
        # random_string = '_reg_'
        # random_string = random_string + ''.join(random.choice(string.ascii_uppercase + string.digits) for _ in range(7))
        sql = """update invoice set state =-1 where invoice_number ='%s' ; """ % param_dict['invoice_number'][0]
        print("Sql for voiding invoice:", sql)
        pg_cur.execute(sql)
        print("executed")

    except Exception as e:
        logger.error(" Error in Invoice voiding   %s" % (str(e)))
        print("error in Invoice voiding   %s" % (str(e)))
        pg_cur_l.close()
        return str(e)
    pg_cur_l.close()
    return "Query executed: " + sql


pg_cur, logger = main("no args")
